import { ParserMain } from "./processParser/parseMain"
import { device1 } from "./processParser/processConfig/device1/parser1"
import { device2 } from "./processParser/processConfig/device2/parser2"
/** 范例 */

const boardLocation = '3'
const widthSideAxis = '0'
const lengthSideAxis = '2'
const parserMain = new ParserMain() 
/** 测试数据 */
const jsonData = `
//数据说明
//（1）加工前先调6厘的刀，
//（2）做孔加工的时候找【T1】的刀加工
//（3）做造型的加工找【T2】的刀
//（4）开料时 找5厘的刀
{
  // 加工项信息
  "processList": [
    {
      // 加工指令数据集  
      "processItemInfo": {
        // 代码
        "code": "MARK",
        // 指令
        "order": "Mark",
        // 指令参数
        "codeParams": "// start  fileHead"
      },
      // 加工刀路信息
      "list": []
    },
    {
      "processItemInfo": {
        "code": "FSTART",
        "order": "F1"
      },
      "list": []
    },
    {
      "processItemInfo": {
        "code": "T",
        "order": "TD",
        "codeParams": {
          "diameter": 6
        }
      },
      "list": []
    },
    {
      //  一个加工项  模拟一个孔的加工
      "processItemInfo": {
       "knife":{
          "knifeName": "T1"
        },
        "code": "C",
        "order": "KnifeMove",
        "block":{
          "cutWidth":400,
          "cutLength":600
        }
      },
      "list": [
        // 移动到安全点
        { "x": 100.01, "y": 100.01, "z": 30.01, "dir": 0, "f": 10000 },
        //  下刀
        { "x": 100.01, "y": 100.01, "z": 0.01, "dir": 1, "f": 8000 },
        //  抬刀
        { "x": 100.01, "y": 100.01, "z": 30.01, "dir": 0, "f": 10000 }
      ]
    },
    {
      //  一个加工项  模拟一个造型的加工
      "processItemInfo": {
        "knife":{
          "knifeName": "T2"
        },
        "code": "C",
        "order": "KnifeMove",
        "block":{
          "cutWidth":400,
          "cutLength":600
        }
      },
      "list": [
        // 移动到安全点
        { "x": 150.01, "y": 150.01, "z": 30.01, "dir": 0, "f": 10000 },
        //  下刀
        { "x": 150.01, "y": 150.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 345.01, "y": 150.01, "z": 0.01, "r": 5, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 350.01, "y": 155.01, "z": 0.01, "dir": 2, "r": 5.01, "f": 8000 },
        // // 走刀
        // { "x": 350.01, "y": 150.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 350.01, "y": 160.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 150.01, "y": 160.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 150.01, "y": 150.01, "z": 0.01, "dir": 1, "f": 8000 },
        //  抬刀
        { "x": 150.01, "y": 150.01, "z": 30.01, "dir": 0, "f": 10000 }
      ]
    },
    {
      "processItemInfo": {
        "code": "MARK",
        "order": "Mark",
        "codeParams": "// cut block"
      },
      "list": []
    },
    {
      "processItemInfo": {
        "knife":{
          "diameter": 5
        },
        "code": "C",
        "order": "KnifeMove",
        "block":{
          "cutWidth":400,
          "cutLength":600
        }
      },
      "list": [
        // 移动到安全点
        { "x": 0.01, "y": 0.01, "z": 30.01, "dir": 0, "f": 10000 },
        // 下刀 到板面
        { "x": 0.01, "y": 0.01, "z": 18.01, "dir": 0, "f": 10000 },
        //切入 
        { "x": 0.01, "y": 0.01, "z": 6.01, "dir": 1, "f": 10000 },
        // 斜线下刀
        { "x": 20.01, "y": 0.01, "z": 0.01, "dir": 1, "f": 5000 },
        // 走刀
        { "x": 380.01, "y": 0.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 弧线走刀 G2
        { "x": 400.01, "y": 30.01, "z": 0.01, "r": 20.01, "dir": 2, "f": 8000 },
        // 走刀
        { "x": 400.01, "y": 600.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 走刀
        { "x": 50.01, "y": 600.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 弧线走刀 G3
        { "x": 0.01, "y": 560.01, "z": 0.01, "r": 40.01, "dir": 3, "f": 8000 },
        // 走刀
        { "x": 0.01, "y": 0.01, "z": 0.01, "dir": 1, "f": 8000 },
        // 斜线下刀没走的 走掉
        { "x": 20.01, "y": 0.01, "z": 0.01, "dir": 1, "f": 8000 },
        //  抬刀
        { "x": 20.01, "y": 0.01, "z": 30.01, "dir": 0, "f": 10000 }
      ]
    },
    {
      //  程序结束 ----停刀 + 停 轴
      "processItemInfo": {
        "code": "CE",
        "order": "CE1"
      },
      "list": [

      ]
    },
    {
      //   文件尾
      "processItemInfo": {
        "code": "FEND",
        "order": "FEND1"
      },
      "list": [

      ]
    }
  ]
}`

/** 测试数据 */
const txtData = `FSTART\nTD 5\nG0 X100 Y100 Z10 F8000\ng0 x100 y100 z18 f8000\ng2 x120 y120 z18 R20 f8000\ng3 x100 y100 z18 i20 j0 k0 f8000\nTN T2\nFEND\n`

function demo(){
    getDeviceList()

    setDevice()
    setDeviceByName()
    setDeviceByKey()
}

/**处理JSON格式的数据 */
async function handleJsonData() {
  let parserMain = new ParserMain();
  parserMain.setDeviceByName('解析器1');
  let output = ''
  let str = jsonData
  let strArr = str.split('\n')
  let arr = strArr.filter(e => !e.includes('//') && !e.includes('/**') || e.includes('"//'))
  arr.forEach(s => {
    s = s.trim()
  })
  let realStr = arr.join('\n')
  let dd: any = ''
  try {
    dd = JSON.parse(realStr)
  } catch (error) {
    console.error('输入的Json有误')
  }
  let tt = await parserMain?.getCode(dd) || ''
  output += tt
  console.log('handleJsonData', output)
}

handleJsonData()

/** 处理指令txt文本数据 */
function handleTxtData() {
  let parserMain = new ParserMain();
  parserMain.setDeviceByName('解析器1');
  let output = ''
  let str = txtData
  let strArr = str.split('\n')

  let code = ''

  let orderList = parserMain._device.getCodeListByOrderKeys(['FSTART', 'T', 'C', 'FEND', 'MARK'])

  const reg_G = /^[Gg](\d+\.?\d*)$/
  const reg_X = /^[Xx](\d+\.?\d*)$/
  const reg_Y = /^[Yy](\d+\.?\d*)$/
  const reg_Z = /^[Zz](\d+\.?\d*)$/
  const reg_F = /^[Ff](\d+\.?\d*)$/
  const reg_R = /^[Rr](\d+\.?\d*)$/
  const reg_I = /^[Ii](\d+\.?\d*)$/
  const reg_J = /^[Jj](\d+\.?\d*)$/
  const reg_K = /^[Kk](\d+\.?\d*)$/

  strArr.forEach(str => {
    let codeLineData = str.split(' ')
    if (Object.keys(codeLineData) && codeLineData.length > 0) {
      let orderKey = codeLineData[0]
      let paramsValue = codeLineData[1]
      let orderItem = orderList.find(e => e.order == orderKey)

      if (orderItem) {
        /**  */
        // console.log('orderKey', orderKey);
        if (orderItem.order == 'KnifeMove') {
          // let codeParams: CodeParams = {

          // }

        } else {
          code += parserMain.handleCode(orderItem.code, orderItem.order, paramsValue)
        }
      } else {
        let tempCodeParams = {}
        codeLineData.forEach(e => {
          if (['G0', 'G00', 'G1', 'G01', 'G2', 'G02', 'G3', 'G03',
            'g0', 'g00', 'g1', 'g01', 'g2', 'g02', 'g3', 'g03'
          ].includes(e)) {
            let match = e.match(reg_G)
            if (match != null) {
              if (match[1] == '0' || match[1] == '00') {
                tempCodeParams.dir = 0
              } else if (match[1] == '1' || match[1] == '01') {
                tempCodeParams.dir = 1
              } else if (match[1] == '2' || match[1] == '02') {
                tempCodeParams.dir = 2
              } else if (match[1] == '3' || match[1] == '03') {
                tempCodeParams.dir = 3
              }

            }
          } else if (e.includes('x') || e.includes('X')) {
            let match = e.match(reg_X)
            if (match != null) {
              tempCodeParams.x = match[1]
            }
          } else if (e.includes('y') || e.includes('Y')) {
            let match = e.match(reg_Y)
            if (match != null) {
              tempCodeParams.y = match[1]
            }
          } else if (e.includes('z') || e.includes('Z')) {
            let match = e.match(reg_Z)
            if (match != null) {
              tempCodeParams.z = match[1]
            }
          } else if (e.includes('f') || e.includes('F')) {
            let match = e.match(reg_F)
            if (match != null) {
              tempCodeParams.f = match[1]
            }
          } else if (e.includes('r') || e.includes('R')) {
            let match = e.match(reg_R)
            if (match != null) {
              tempCodeParams.r = match[1]
            }
          } else if (e.includes('i') || e.includes('I')) {
            let match = e.match(reg_I)
            if (match != null) {
              tempCodeParams.i = match[1]
            }
          } else if (e.includes('j') || e.includes('J')) {
            let match = e.match(reg_J)
            if (match != null) {
              tempCodeParams.j = match[1]
            }
          } else if (e.includes('k') || e.includes('K')) {
            let match = e.match(reg_K)
            if (match != null) {
              tempCodeParams.k = match[1]
            }
          }
        })

        // console.log('tempCodeParams.dir', tempCodeParams.dir, str);
        if (tempCodeParams.dir != undefined) {
          let tempProcessItem = {
            list: [tempCodeParams]
          }

          code += parserMain.handleCode('C', 'KnifeMove', tempProcessItem)
        } else {
          code += str + '\n'
        }

      }
    }
  })

  if (parserMain._device.isUseSimpleCode == true) {
    code = parserMain._device.handleSimpleCode(code)
  }
  output += code
  console.log('handleTxtData',output)
}

// handleTxtData()
//#region  获取解析器列表
function getDeviceList(){
    let list = parserMain.getDeviceList()
    // console.log(list)
}
//#endregion


//#region 设置要使用的解析器
 
function setDevice(){
    parserMain.setDevice(device1)
    // console.log(parserMain._device)
    
}

function setDeviceByName(){
    parserMain.setDeviceByName('解析器2')
    // console.log(parserMain._device)
}

function setDeviceByKey(){
    parserMain.setDeviceByKey('device1')
    // console.log(parserMain._device)
}
//#endregion

//#region 设置
