// import { Polyline } from "cadapi";
// import { InstancedBufferAttribute, Matrix4, Vector3 } from "three";
import type { Circle, Polyline } from 'cadapi'
import type { TextureType } from '../../vo/enums/TextureType'

// import { PaiKongType, PlaceStyle } from '../../Base/Enum'
import { ComposingType } from '../../algorithm/optimize/bang'
import { PlaceStyle } from '../../vo/enums/PlaceStyle'

// import Worker from "worker-loader!../Nest/Core/OptimizeWorker.worker";
// import type { PlaceBlock } from '../../Models/Orders'
import type { PlaceBlock } from '../../vo/order/PlaceBlock'

// import { BlockPlaceResult } from '../Places'
import { BlockPlaceResult, BoardPlaceResult, MaterialPlaceResult } from '../../vo/order/Places'
import { Box2 } from '../Box2'
import { InitClipperCpp } from '../ClipperCpp'
import { NestFiler } from '../Filer'

import { Vector2 } from '../Vector2'
import { Individual } from '../core/Individual'
import { NestCache } from '../core/NestCache'
import { NestDatabase } from '../core/NestDatabase'
import { OptimizeMachine } from '../core/OptimizeMachine'
import { Part } from '../core/Part'
import { Path } from '../core/Path'
import { PathGeneratorSingle } from '../core/PathGenerator'
import type { PlaceOrderTask } from '../../vo/order/PlaceOrderTask'
import { ScrapBoardHelper } from '../../vo/order/ScrapBoardHelper'

import { removeRepeatPoint } from '../base/MathComm'
import { BlockPlus } from '../../vo/order/BlockPlus'
import { BlockSizePlus } from '../../vo/order/BlockSizePlus'

// import type { RemainBoard } from '@/Source/Models/ScrapBorad'
import type { RemainBoard } from '../../vo/remain/RemainBoard'
import { CurveWrap, Points2Polyline } from './Curves2Parts'
import { PolylineHelper } from './PolylineHelper'
import { PlaceStore } from '@/imes/vo/order/PlaceStore'

export class LayoutEngine {
  pmID: number
  border: number
  cutDia: number
  cutGap: number
  boardW: number
  boardL: number
  scrapBoards: RemainBoard[] = []
  blocks = []
  blockNos = []
  task: PlaceOrderTask // 回掉函数
  filterBlockToPlace = false // 是否锁定板不优化

  isLastBoard = false // 最后一片 重新优化

  parts: Part[] = []
  board: Path
  threads: Worker[] = []
  result: Individual
  placeResult: MaterialPlaceResult

  db: NestDatabase
  comparePointKeys: string[]

  offset = 0

  /** 启用王者优化的最小缺角大小 */
  unregularSizeLimit = 15000

  constructor(obj) {
    console.log('优化算法  线程  初始化')
    this.pmID = obj.pmID
    this.cutDia = obj.cutDia
    this.cutGap = obj.cutGap
    this.border = obj.border
    this.boardW = obj.boardW
    this.boardL = obj.boardL
    this.scrapBoards = obj.scrapBoards
    this.blocks = obj.blocks
    for (const block of obj.blocks)
      this.blockNos[block.blockNo] = block

    this.task = obj.task
    this.filterBlockToPlace = obj.filterBlock
    this.isLastBoard = obj.isLastBoard
    // 清理缓存,这个缓存是全局的
    PathGeneratorSingle.Clear()
    NestCache.Clear()

    this.offset = (this.cutDia + this.cutGap) / 2 // 间隔 （刀直径 + 1）
    const binWidth = this.boardW - this.border * 2 + this.cutDia + this.cutGap
    const binLength = this.boardL - this.border * 2 + this.cutDia + this.cutGap

    // 注册bin
    const binPath = new Path([{ x: 0, y: 0 }, { x: binWidth, y: 0 }, { x: binWidth, y: binLength }, { x: 0, y: binLength }])
    binPath.Id = undefined
    PathGeneratorSingle.RegisterId(binPath)
    this.board = binPath

    // this.setLocator(3)
    // this.setLocator(0) // 左下
    this.setLocator(PlaceStore.sysConfig.boardLocation)
    //
    // this.unregularSizeLimit = PlaceStore.sysConfig.UnregularSizeLimit
  }

  setLocator(p: number) {
    const gravityDict = {
      0: ['xy', 'yx'], // 左下 0
      1: ['Xy', 'yX'], // 右下 1
      3: ['xY', 'Yx'], // 左上 3
      2: ['XY', 'YX'], // 右上 2
    }

    this.comparePointKeys = gravityDict[p.toString()]
  }

  async startThread() {
    // 初始化 板件
    await InitClipperCpp()

    // 确保 前余料板 placedPolyline 都生成
    const oddpaths = []
    for (const RemainBoard of this.scrapBoards) {
      const curve = ScrapBoardHelper.getPlacedPolyline(RemainBoard)
      curve.CloseMark = true
      const warp = new CurveWrap(curve, 3.5, false)
      const pts = warp.GetInsidePoints()
      // let pts = warp.Points;
      const path = new Path(pts)
      const path_0 = PathGeneratorSingle.Allocate(path)// 获得唯一path
      PathGeneratorSingle.RegisterId(path_0)// 注册id
      oddpaths.push(path_0)
      RemainBoard.isUsed = false
    }

    // 生产parts
    this.parts = []
    let autoID = 0

    for (const block of this.blocks) {
      const part = this.toPart(block, this.offset, this.board, autoID)
      this.parts.push(part)
      autoID++
    }

    if (this.parts.length == 0)
      return// 没有板

    // 创建 NestDatabase
    this.db = new NestDatabase()
    this.db.Paths = PathGeneratorSingle.paths
    this.db.Parts = this.parts
    this.db.Bin = this.board
    this.db.OddmentsBins = oddpaths
    this.db.ComparePointKeys = this.comparePointKeys

    const searchParams = new URLSearchParams(location.search)
    if (searchParams.has('single')) {
      const m = new OptimizeMachine()
      m.Bin = this.db.Bin
      m.PutParts(this.db.Parts)

      await InitClipperCpp()
      let best = Number.POSITIVE_INFINITY
      m.callBack = async (inv) => {
        if (inv.Fitness >= best)
          return
        best = inv.Fitness
        this.result = inv
        this.ToPlaceResult()
        this.task.receiverResult(this.pmID, this.placeResult, true, this.filterBlockToPlace, this.isLastBoard)
        const text = `single 板材${this.pmID} 发现更优 优化率:${inv.Fitness}`
        console.log(text)
      }
      await m.Start()
    }
    else {
      const f = new NestFiler()
      this.db.WriteFile(f)// 写入到文件中
      console.log('优化数据 db=', JSON.stringify(f._datas))

      let best = Number.POSITIVE_INFINITY

      const t = new Worker(new URL('../Nest/Core/OptimizeWorker.worker'))
      this.threads = [t]

      // 加载
      // let rtStr = `[1,17.97496389719577,18,-1,0,null,8,35,0,{"x":0,"y":0},199,0,{"x":7758777,"y":0},0,0,{"x":7475524,"y":21084400},50,0,{"x":11500677,"y":0},40,0,{"x":9458109,"y":18660200},113,0,{"x":5750492,"y":0},117,0,{"x":10011472,"y":5185300},38,0,{"x":5944846,"y":22234200},[0,0],[1210.0677,2441.02],-1,0,null,6,130,0,{"x":0,"y":0},12,0,{"x":7190000,"y":0},16,0,{"x":0,"y":20050000},184,0,{"x":4550000,"y":20050000},77,0,{"x":11739999,"y":0},26,0,{"x":7100000,"y":22541717},[0,0],[1215.9999,2428.2203000000004],-1,0,null,8,136,0,{"x":0,"y":0},157,0,{"x":0,"y":13050000},95,0,{"x":8990000,"y":13050000},65,0,{"x":9490000,"y":0},43,0,{"x":10650000,"y":0},14,0,{"x":0,"y":21500000},6,0,{"x":4550000,"y":21500000},186,0,{"x":9100000,"y":20740000},[0,0],[1218,2440.55],-1,0,null,5,129,0,{"x":0,"y":0},142,0,{"x":6050000,"y":0},84,0,{"x":0,"y":20050000},18,0,{"x":3550000,"y":20050000},85,0,{"x":8100000,"y":20050000},[0,0],[1210,2410],-1,0,null,8,152,0,{"x":0,"y":0},107,0,{"x":7910000,"y":0},169,0,{"x":0,"y":13010000},9,0,{"x":0,"y":20450200},8,0,{"x":4550000,"y":21050000},42,0,{"x":11460000,"y":0},46,0,{"x":11460000,"y":11540000},118,0,{"x":9100000,"y":21050000},[0,0],[1212,2431],-1,0,null,7,150,0,{"x":0,"y":0},80,0,{"x":7590000,"y":0},161,0,{"x":0,"y":13050000},92,0,{"x":8043800,"y":12823400},88,0,{"x":0,"y":20130200},11,0,{"x":3550000,"y":20633400},89,0,{"x":8100000,"y":20633400},[0,0],[1218.2,2432.34],-1,0,null,6,151,0,{"x":0,"y":0},137,0,{"x":6050000,"y":0},131,0,{"x":0,"y":13050000},134,0,{"x":5870000,"y":13050000},179,0,{"x":0,"y":22540000},90,0,{"x":7080200,"y":22540000},[0,0],[1210,2441],-1,0,null,10,149,0,{"x":0,"y":0},99,0,{"x":6050000,"y":0},133,0,{"x":0,"y":13050000},49,0,{"x":9600000,"y":0},78,0,{"x":9600000,"y":18211700},110,0,{"x":5870000,"y":21050000},47,0,{"x":10770000,"y":0},67,0,{"x":10770000,"y":6390800},86,0,{"x":0,"y":22540000},36,0,{"x":10770000,"y":14365800},[0,0],[1212,2441],-1,0,null,7,167,0,{"x":0,"y":0},166,0,{"x":0,"y":7440200},165,0,{"x":0,"y":14880400},15,0,{"x":7583800,"y":0},5,0,{"x":7583800,"y":10186100},10,0,{"x":7583800,"y":19770800},174,1,{"x":0,"y":22725699},[0,0],[1213.38,2377.5699],-1,0,null,7,159,0,{"x":0,"y":0},168,0,{"x":0,"y":7440200},153,0,{"x":0,"y":14880400},132,0,{"x":5870000,"y":14880400},1,0,{"x":7583800,"y":0},7,0,{"x":7583800,"y":9703200},21,0,{"x":7583800,"y":12596100},[0,0],[1213.38,2437.04],-1,0,null,6,155,0,{"x":0,"y":0},144,0,{"x":0,"y":8990000},158,0,{"x":0,"y":15680000},146,0,{"x":5870000,"y":0},156,0,{"x":6050000,"y":15680000},66,0,{"x":7510000,"y":7590000},[0,0],[1210,2413],-1,0,null,7,154,0,{"x":0,"y":0},143,0,{"x":0,"y":8990000},147,0,{"x":0,"y":15680000},138,0,{"x":5690000,"y":0},148,0,{"x":5870000,"y":15680000},79,0,{"x":7510000,"y":6979700},175,1,{"x":0,"y":23269999},[0,0],[1212.2,2431.9999000000003],-1,0,null,10,145,0,{"x":0,"y":0},162,0,{"x":0,"y":6690000},164,0,{"x":0,"y":12360767},163,0,{"x":0,"y":18031534},102,0,{"x":8583823,"y":0},108,0,{"x":8583823,"y":7690000},100,0,{"x":8583823,"y":15380000},54,0,{"x":7080200,"y":18031534},64,0,{"x":7510000,"y":0},45,0,{"x":0,"y":9495020},[0,0],[1213.3823,2370.2334],-1,0,null,12,128,0,{"x":0,"y":0},127,0,{"x":5870000,"y":0},125,0,{"x":0,"y":7190000},126,0,{"x":5870000,"y":7190000},33,0,{"x":0,"y":14380000},17,0,{"x":6763794,"y":14380000},29,0,{"x":6217041,"y":21866700},111,0,{"x":0,"y":20483549},96,0,{"x":11313794,"y":14380000},114,0,{"x":0,"y":14380000},74,0,{"x":1870000,"y":21945626},58,0,{"x":10767041,"y":22070000},[0,0],[1216.3794,2436.9826000000003],-1,0,null,14,139,0,{"x":0,"y":0},20,0,{"x":0,"y":6827900},19,0,{"x":0,"y":13520636},34,0,{"x":4758655,"y":9382427},93,0,{"x":6050000,"y":0},140,1,{"x":2885031,"y":17262709},28,0,{"x":0,"y":21968505},31,0,{"x":4550000,"y":21968505},48,0,{"x":11522449,"y":0},98,0,{"x":9890000,"y":0},60,0,{"x":9100000,"y":21968505},71,0,{"x":0,"y":19880476},56,0,{"x":9859631,"y":17510524},70,0,{"x":6546965,"y":7810000},[0,0],[1221.02,2439.5305],-1,0,null,18,94,0,{"x":0,"y":0},97,0,{"x":3840000,"y":0},104,1,{"x":7389466,"y":0},83,0,{"x":3840000,"y":7690000},203,0,{"x":0,"y":7810000},187,0,{"x":6600789,"y":10070000},207,0,{"x":0,"y":12995300},191,0,{"x":6893409,"y":15255300},185,0,{"x":0,"y":18180600},44,0,{"x":3651900,"y":18360000},25,0,{"x":6007900,"y":20440600},115,0,{"x":10342689,"y":10070000},57,0,{"x":10449767,"y":0},69,0,{"x":10449767,"y":4788400},72,0,{"x":10635309,"y":13910000},170,0,{"x":6007900,"y":23108600},119,0,{"x":10635309,"y":18698400},52,0,{"x":6092844,"y":7690000},[0,0],[1220.2689,2415.86],-1,0,null,20,211,0,{"x":0,"y":0},195,0,{"x":0,"y":5185300},181,0,{"x":0,"y":10370600},189,0,{"x":0,"y":15555900},201,0,{"x":3651900,"y":10370600},197,0,{"x":3651900,"y":15555900},193,0,{"x":3741900,"y":0},205,0,{"x":7303800,"y":5185300},209,0,{"x":7303800,"y":10370600},3,0,{"x":7303800,"y":15555900},4,0,{"x":7303800,"y":18223900},23,0,{"x":7393800,"y":0},27,0,{"x":0,"y":20741200},24,0,{"x":4550000,"y":20891900},30,0,{"x":7393800,"y":2668000},106,0,{"x":3741900,"y":5185300},160,0,{"x":10955700,"y":5094800},101,0,{"x":3741900,"y":8235300},37,0,{"x":9100000,"y":20891900},41,0,{"x":10955700,"y":12175000},[0,0],[1212.5700000000002,2355.9900000000002],-1,0,null,31,32,0,{"x":0,"y":0},180,0,{"x":4550000,"y":0},188,0,{"x":7100000,"y":0},182,0,{"x":9650000,"y":0},192,0,{"x":0,"y":2426800},202,0,{"x":2550000,"y":3665500},196,0,{"x":5100000,"y":3665500},190,0,{"x":7650000,"y":3665500},204,0,{"x":0,"y":7331000},194,0,{"x":2550000,"y":7331000},206,0,{"x":5100000,"y":7331000},198,0,{"x":7650000,"y":7331000},200,0,{"x":0,"y":10996500},177,0,{"x":2550000,"y":10996472},208,0,{"x":2550000,"y":13039047},210,0,{"x":5100000,"y":13039047},176,0,{"x":372132,"y":15464478},178,0,{"x":0,"y":17508377},171,0,{"x":0,"y":18558377},173,0,{"x":0,"y":19608377},39,0,{"x":10200000,"y":3665500},62,0,{"x":9862132,"y":13039047},172,1,{"x":5755530,"y":20193826},59,0,{"x":7650000,"y":13039047},76,0,{"x":8990000,"y":17511047},123,0,{"x":0,"y":20658376},51,0,{"x":10200000,"y":8137500},61,0,{"x":1450000,"y":20658377},75,0,{"x":2800000,"y":20658377},73,0,{"x":4150000,"y":20658377},68,0,{"x":5500000,"y":21243826},[0,0],[1220,2381.8376000000003],9,35,0,{"isVector2":true,"x":15.451070343735374,"y":16.34651510924327},8,135,0,{"x":0,"y":3925185},91,0,{"x":6050000,"y":8809129},183,0,{"x":0,"y":16975185},109,0,{"x":3741900,"y":16975185},124,0,{"x":262661,"y":665185},63,0,{"x":6050000,"y":6171301},116,0,{"x":2552661,"y":950420},55,0,{"x":7291900,"y":16619129},[0,66.5185],[989,2216.0485],152,0,{"isVector2":true,"x":62,"y":62.000000000000455},2,141,0,{"x":0,"y":0},13,0,{"x":0,"y":9850000},[0,0],[605,1129.43],144,0,{"isVector2":true,"x":186.6426800426124,"y":52.339509444329806},1,87,0,{"x":26544,"y":20434},[2.6544000000000003,2.0434],[357.6544,371.0434],143,0,{"isVector2":true,"x":186.6426800426124,"y":52.33950944432935},1,103,0,{"x":17062,"y":612763},[1.7062000000000002,61.276300000000006],[356.7062,366.2763],145,0,{"isVector2":true,"x":186.6423965090944,"y":52.33842250129328},2,105,0,{"x":0,"y":67071},53,0,{"x":873022,"y":3117071},[0,6.7071000000000005],[355,518.1271],33,0,{"isVector2":true,"x":29.647527772751403,"y":23.9973798061884},3,112,0,{"x":1794315,"y":1823511},81,0,{"x":3664315,"y":4144613},82,0,{"x":3664315,"y":694573},[179.4315,69.4573],[537.4315,659.4612999999999],20,0,{"isVector2":true,"x":20.50979557910309,"y":20.50979557910042},1,2,0,{"x":401639,"y":1617035},[40.163900000000005,161.70350000000002],[495.1639,444.9735],19,0,{"isVector2":true,"x":20.50979557910418,"y":20.50979557910332},1,22,0,{"x":346614,"y":1699385},[34.6614,169.9385],[489.6614,436.73850000000004],34,0,{"isVector2":true,"x":29.647527772751403,"y":23.9973798061884},3,121,0,{"x":1794315,"y":1823511},120,0,{"x":3654315,"y":686471},122,0,{"x":3654315,"y":4257856},[179.4315,68.64710000000001],[510.4315,680.7856],0]`;
      // let data = JSON.parse(rtStr);
      // let f1 = new NestFiler(data);
      // let inv = new Individual(this.db.Parts);
      // inv.ReadFile(f1);
      // best = inv.Fitness;
      // this.result = inv;
      // this.ToPlaceResult();
      // this.parent.receiverResult(this.pmID, this.placeResult, true);
      t.onmessage = (e) => {
        const f = new NestFiler(e.data)
        console.log('优化结果 e.data=', JSON.stringify(e.data))

        const inv = new Individual(this.db.Parts)
        inv.ReadFile(f)
        if (best <= inv.Fitness)
          return
        best = inv.Fitness
        this.result = inv
        this.ToPlaceResult()
        this.task.receiverResult(this.pmID, this.placeResult, true, this.filterBlockToPlace, this.isLastBoard)

        const text = `板材${this.pmID} 王者优化发现更优 优化率:${inv.Fitness}`
        console.log(text)
      }
      t.postMessage(f._datas)// Post给Worker
    }
  }

  stopThread() {
    if (this.threads.length == 0)
      return
    this.threads[0].terminate()
    const text = `板材${this.pmID} 新优化停止  最优优化率:${this.result ? this.result.Fitness : 0}`
    console.log(text)
    this.threads = []
  }

  Rotations = [
    [0, Math.PI], // 正文
    [0, Math.PI / 2, Math.PI, Math.PI * 1.5], // 可翻转
    [Math.PI / 2, Math.PI * 1.5], // 反纹
  ]

  private toPart(block: PlaceBlock, offset: number, binPath: Path, id: number): Part {
    // if (block.blockNo == '2311001545897')
    //   
    const part = new Part()
    part.Id = id

    const isYX = block.isUnRegular // 是否异形
    const wave = block.texture
    let face = block.isTurnFaceToPlace ? ComposingType.Reverse : ComposingType.Positive
    if (this.task.needTurnFaceToPlace)
      face = block.isTurnFaceToPlace ? ComposingType.Positive : ComposingType.Reverse

    const do2Face = block.isTwoFaceToPlace

    part.UserData = { bno: block.blockNo, size: block.area, isYX, wave, face, do2Face, width: block.cutWidth, length: block.cutLength }

    let path: Path
    // 小板开料轮廓
    const border = BlockPlus.getBorder(block, true, face == ComposingType.Positive ? PlaceStyle.FRONT : PlaceStyle.BACK)
    let polyLine: Polyline | Circle
    polyLine = PolylineHelper.createByCurve2d(border, true)
    polyLine = PolylineHelper.mergeCurve(polyLine)
    // 偏移 = 同刀辅助 + 刀半径 + 缝隙/2
    offset = block.blockDetail.useSameKnifeToHelpCutGap + block.placeMaterial.diameter / 2 + block.placeMaterial.cutKnifeGap / 2
    const cureW = new CurveWrap(polyLine, offset, true)
    let pts2 = cureW.GetOutsidePoints()
    // 确保pts2 不重复
    removeRepeatPoint(pts2, 0.01)
    const pl_models = BlockPlus.getOutModelPloyline(block, true, face == ComposingType.Positive ? PlaceStyle.FRONT : PlaceStyle.BACK)
    if (pl_models.length > 0) {
      // 对外扩 造型 精简
      const new_pls = []
      for (const pl of pl_models) {
        const cureW = new CurveWrap(pl, block.placeMaterial.diameter / 2 + block.placeMaterial.cutKnifeGap / 2, true)
        const pts = cureW.GetOrgPoints()
        removeRepeatPoint(pts, 0.01)
        const pl_nw = PolylineHelper.create(pts, true)
        new_pls.push(pl_nw)
      }

      const pl0 = PolylineHelper.create(pts2, true)
      const pl_full = BlockPlus.unionPolylines(pl0, new_pls)

      pts2 = pl_full.LineData.map((t) => { return { x: t.pt.x, y: t.pt.y } })
    }
    // 确保pts2 不重复
    removeRepeatPoint(pts2, 0.01)
    path = new Path(pts2)

    // if(block.blockNo == '2311691815099')
    // {

    //     const plstr = PolylineHelper.getStrPLs([polyLine]);
    //     ;
    // }

    // 判断是否 扩大了尺寸
    const orgSize = { w: polyLine.BoundingBox.max.x - polyLine.BoundingBox.min.x, l: polyLine.BoundingBox.max.y - polyLine.BoundingBox.min.y }
    const newSize = { w: path.BoundingBox.max.x - path.BoundingBox.min.x, l: path.BoundingBox.max.y - path.BoundingBox.min.y }
    // if( newSize.w > orgSize.w + 0.2 || newSize.l > orgSize.l + 0.2)
    // {
    //     const plstr = PolylineHelper.getStrPLs([polyLine]);
    //     console.log (`小板轮廓精简后变大：宽${orgSize.w} +${newSize.w - orgSize.w} 宽${orgSize.l} +${newSize.l - orgSize.l}`);
    //     ;
    // }

    const area = path.BoundingBox.area - path.Area
    if (pl_models.length == 0 && area < this.unregularSizeLimit && pts2.length >= 6) // 没有外扩 ， 缺角小，当做矩形来处理
    {
      const minX = polyLine.BoundingBox.min.x - offset
      const maxX = polyLine.BoundingBox.max.x + offset
      const minY = polyLine.BoundingBox.min.y - offset
      const maxY = polyLine.BoundingBox.max.y + offset
      path = new Path([
        { x: minX, y: minY },
        { x: maxX, y: minY },
        { x: maxX, y: maxY },
        { x: minX, y: maxY },
      ])
    }

    part.Init2(path, binPath, this.Rotations[wave as number])

    // if (!path.IsRect && part.RotatedStates.length == 0) //放不下,不简化路径
    // {
    //     const cureW2 = new CurveWrap(polyLine, 0, true);
    //     cureW2.KnifRadius = offset;
    //     const pts23 = cureW2.GetOutsidePoints();
    //     arrayRemoveDuplicateBySort(pts23, (p1, p2) => this.equalv2(p1, p2, 1e-2));
    //     // path = new Path(cureW.Points);
    //     path = new Path(pts23);
    //     part.Init2(path, binPath, this.Rotations[wave as number]);
    // }

    // // 添加造型洞
    // let  modelHoles = BlockPlus.getModelHole(block,true,PlaceStyle.FRONT);
    // if(block.IsTurnFaceToPlace) modelHoles =BlockPlus.getModelHole(block,true,PlaceStyle.BACK);
    // for(const mHole of modelHoles)
    // {
    //     let polyLine : Polyline|Circle ;
    //     polyLine =  PolylineHelper.createByCurve2d(mHole,true);
    //     //合并圆弧
    //     polyLine = PolylineHelper.mergeCurve(polyLine);

    //     const cureW = new CurveWrap(polyLine, 0, false);
    //     const pts = cureW.GetOrgPoints();
    //     //确保pts2 不重复
    //     removeRepeatPoint(pts2,0.01);
    //     if(pts2)
    //     {
    //         const pps = new Path(pts2);
    //         if (pps.Area > 15000)
    //             part.AppendHole(pps);
    //     }

    // }

    // 造型孔
    for (const m of block.models) {
      if (m.depth < block.thickness - 0.05)
        continue
      if (m.IsDo == false)
        continue
      if (!m.HasContour)
        continue
      if (m.OriginModeling.outline.pts.length != m.OriginModeling.outline.buls.length)
        continue
      let pts = m.OriginModeling.outline.pts.map((t) => { return { x: t.x, y: t.y, bul: 0 } })
      const buls = m.OriginModeling.outline.buls
      for (let i = 0; i < pts.length; i++)
        pts[i].bul = buls[i]

      // 如果反面,翻转方向，重设坐标
      if (face == ComposingType.Reverse)
        pts = this.reversePoint(pts, block.CuttingWidth)

      let polyLine
      polyLine = Points2Polyline(pts)
      const cicle = PolylineHelper.ConverPolyLin2Circle(polyLine)
      if (cicle)
        polyLine = cicle
      let off = m.RealKnifeRadius + this.cutGap / 2 // Math.max(m.RealKnifeRadius, this.cutDia / 2) + this.cutGap / 2;
      if (m.RealKnifeRadius < this.cutDia / 2) {
        // 如果造型刀 小于开料刀， 孔洞 按 开料刀 半径来分析
        off = this.cutDia / 2 + this.cutGap / 2
      }
      if (m.RealKnifeRadius > this.cutDia / 2) {
        // 如果造型刀 大于 开料刀， 孔洞 按 造型刀 直径 - 开料刀的半径  来分析
        off = m.RealKnifeRadius + m.RealKnifeRadius - this.cutDia / 2 + this.cutGap / 2
      }
      const cureW2 = new CurveWrap(polyLine, off, false)
      const pts2 = cureW2.GetInsidePoints2(off)

      if (pts2) {
        const pps = new Path(pts2)
        if (pps.Area > 15000)
          part.AppendHole(pps)
      }
    }

    return part
  }

  equalv2(v1, v2, fuzz) {
    return Math.abs(v1.x - v2.x) < fuzz && Math.abs(v1.y - v2.y) < fuzz
  }

  private ToPlaceResult() {
    if (this.result == null)
      return

    for (const part of this.parts) {
      part.Parent = null
      part.HolePosition = null
    }

    if (Number.isNaN(this.result.Fitness))
      this.result.Fitness = 0
    const retData = new MaterialPlaceResult()

    let boardCount = 0
    let remainCount = 0
    const offv = this.border - this.cutDia / 2 - this.cutGap / 2
    // const offv = 0.01;
    for (let i = 0; i < this.result.OddmentsContainers.length + this.result.Containers.length; i++) {
      const isOddment = i < this.result.OddmentsContainers.length
      const cn = isOddment ? this.result.OddmentsContainers[i] : this.result.Containers[i - this.result.OddmentsContainers.length]

      if (cn.PlacedParts.length == 0)
        continue
      boardCount++
      const boardResult = new BoardPlaceResult()
      boardResult.boardId = boardCount
      if (isOddment) {
        boardResult.isRemainBoard = true
        boardResult.isScrap = true
        const oddBoard = this.scrapBoards[i]
        boardResult.width = oddBoard.width
        boardResult.length = oddBoard.length
        boardResult.remainId = oddBoard.id
        boardResult.remainNo = oddBoard.id.toString()
        remainCount++
      }

      let pid = 0
      // 零件内
      for (const part of cn.PlacedParts as Part[]) {
        // if (part.UserData.bno == '2311001545897')
        //   
        pid++
        const br = this.CreateBlockPlaceResult(part, offv, boardCount, pid)
        part.UserData.br = br
        boardResult.blocks.push(br)
        boardResult.area += part.UserData.size
      }
      retData.boards.push(boardResult)
    }

    // 空洞里头的板信息
    const placeResults: BlockPlaceResult[] = []

    // 计算网洞的位置信息
    for (const hole of this.result.HoleContainers) {
      if (!hole.ParentM)
        console.log(hole)

      for (const part of hole.PlacedParts) {
        part.Parent = this.parts[hole.ParentId]
        // 在这里我们求得这个网洞的左下角相对于父零件的左下角的移动距离
        part.HolePosition = { ...hole.ParentM }
        part.HolePosition.x -= part.Parent.State.MinPoint.x
        part.HolePosition.y -= part.Parent.State.MinPoint.y
        // console.log(`block: ${part.UserData.bno}  parent block:${part.Parent.UserData.bno}`);
      }
    }

    // 放置网洞内的板
    for (const hole of this.result.HoleContainers) {
      for (const part of hole.PlacedParts) {
        let p = { ...part.PlacePosition } // part在hole中的放置位置

        p.x += part.HolePosition.x * 1e4 // + hole的在parent中的位置
        p.y += part.HolePosition.y * 1e4

        let parent = part.Parent
        let boardId: number

        console.log(`block: ${part.UserData.bno}`)

        while (parent) {
          if (parent.HolePosition) {
            p.x += parent.HolePosition.x * 1e4
            p.y += parent.HolePosition.y * 1e4
          }

          p.x += parent.PlacePosition.x
          p.y += parent.PlacePosition.y

          boardId = parent.UserData?.br?.BoardID
          // console.log(`find parent block: ${parent.UserData.bno} boardID = ${boardId}`);
          parent = parent.Parent
        }

        // console.log(p);

        [part.PlacePosition, p] = [p, part.PlacePosition]
        const br = this.CreateBlockPlaceResult(part, offv, 0, 0)
        br.BoardID = boardId;
        [part.PlacePosition, p] = [p, part.PlacePosition]

        // console.log(br.PlaceX, br.PlaceY);

        part.UserData.br = br
        // part.UserData.Parent = this.parts[hole.ParentId];
        // part.Parent = this.parts[hole.ParentId];
        placeResults.push(br)
      }
    }

    //  placeResults 归并到  retData.boards 中

    for (const br of placeResults) {
      const board = retData.boards[br.BoardID - 1]
      board.Blocks.push(br)
      br.PlaceID = board.Blocks.length
      board.area += br.size
    }

    retData.threadID = 0
    retData.usedTime = 0
    retData.placeCount = 0

    // 计算优化率
    retData.boardCount = boardCount - remainCount // 大板数
    retData.boardCount_remain = remainCount // 余料板数
    let size_all = 0 // 所有大板上的小板面积
    let size_last = 0// 最后一张大板的小板面积
    retData.boards.forEach(t => size_all += (t.IsOddment ? 0 : t.area))
    size_last = retData.boardCount > 0 ? retData.boards[retData.boards.length - 1].area : 0

    if (retData.boardCount <= 1) // 大板数
    {
      retData.useSize_avg = size_last
      retData.useSize_noLast = 0
      retData.useSize_last = size_last
    }
    else {
      retData.useSize_avg = size_all / (retData.boardCount)
      retData.useSize_noLast = (size_all - size_last) / (retData.boardCount - 1)
      retData.useSize_last = size_last
    }

    this.placeResult = retData
  }

  // 根据Part的PlacePosition来创建一个BlockPlaceResult
  private CreateBlockPlaceResult(part: Part<any, any>, offv: number, bid: number, pid: number): BlockPlaceResult {
    const w = part.UserData.width
    const h = part.UserData.length

    const c = Math.cos(part.State.Rotation)
    const s = Math.sin(part.State.Rotation)

    const RotatePoint = (p: Vector2) => {
      const [x, y] = [p.x, p.y]
      p.x = x * c - y * s
      p.y = x * s + y * c
    }

    const pts = [
      new Vector2(),
      new Vector2(w, 0),
      new Vector2(0, h),
      new Vector2(w, h),
    ]

    for (const p of pts) {
      // Move
      p.x -= part.State.OrigionMinPoint.x
      p.y -= part.State.OrigionMinPoint.y

      // Rotate
      RotatePoint(p)
    }

    const box = new Box2().setFromPoints(pts)
    const min = box.min

    let px = part.PlacePosition.x / 1e4 + min.x + offv
    let py = part.PlacePosition.y / 1e4 + min.y + offv
    const [rw, rl, pl] = this.getPlaceStyle(part)
    const block = this.blockNos[part.UserData.bno]
    if (block) {
      const sizeOff = BlockSizePlus.getOffDis(block, pl)
      px = px - sizeOff.left
      py = py - sizeOff.bottom
    }
    const size = part.UserData.size
    const br = new BlockPlaceResult(part.UserData.bno, bid, pid, px, py, rw, rl, pl, size)
    return br
  }

  /** 获得放置方式 */
  private getPlaceStyle(part: Part): [number, number, PlaceStyle] {
    const ratation = part.State.Rotation
    const isYX = part.UserData.isYX
    const wave = part.UserData.texture as TextureType
    const face = part.UserData.face as PaiKongType
    let width = part.UserData.length
    let length = part.UserData.width

    if (ratation == 0 || ratation == Math.PI)
      [width, length] = [length, width]

    if (face == ComposingType.Positive) {
      if (ratation == 0)
        return [width, length, PlaceStyle.FRONT]
      if (ratation == Math.PI / 2)
        return [width, length, PlaceStyle.FRONT左转]
      if (ratation == Math.PI)
        return [width, length, PlaceStyle.FRONT后转]
      return [width, length, PlaceStyle.FRONT右转]
    }
    else // ComposingType.Reverse
    {
      if (ratation == 0)
        return [width, length, PlaceStyle.BACK]
      if (ratation == Math.PI / 2)
        return [width, length, PlaceStyle.BACK左转]
      if (ratation == Math.PI)
        return [width, length, PlaceStyle.BACK后转]
      return [width, length, PlaceStyle.BACK右转]
    }
  }

  private polar(v, an: number, dis: number): any {
    v.x += Math.cos(an) * dis
    v.y += Math.sin(an) * dis
    return v
  }

  private PathScale(pts, scale: number) {
    for (const p of pts) {
      p.x *= scale
      p.y *= scale
    }
    return pts
  }

  private reversePoint(pts, w): any[] {
    const newPts = []
    for (let i = pts.length - 1; i >= 0; i--) {
      const p = pts[i]
      const x = w - p.x
      const y = p.y
      const j = i == 0 ? pts.length - 1 : i - 1

      const bul = pts[j].bul
      newPts.push({ x, y, bul })
    }
    return newPts
  }
}
