/** 自动生成余料板空间 */

import { Vector3 } from 'three'
import { Polyline2Points } from 'cadapi'
import type { Polyline } from 'cadapi'
import { BlockSizePlus } from '../../Pack/BlockSizePlus'
import { BlockHelper } from '../BlockHelper'
import { arrayRemoveDuplicateBySort } from '../Nest/Common/ArrayExt'
import { InitClipperCpp } from '../Nest/Common/ClipperCpp'
import type { Vector2 } from '../Nest/Common/Vector2'
import { Container } from '../Nest/Core/Container'
import { NestCache } from '../Nest/Core/NestCache'
import { ParseOddments } from '../Nest/Core/ParseOddments'
import { Part } from '../Nest/Core/Part'
import { Path } from '../Nest/Core/Path'
import { PathGeneratorSingle } from '../Nest/Core/PathGenerator'
import { PlaceStore } from '../Store'
import { CurveWrap, Points2Polyline } from './Curves2Parts'
import { PolylineHelper } from './PolylineHelper'
import { BlockPlus } from '@/Source/Pack/BlockPlus'
import { ScrapBlock } from '@/Source/Models/ScrapBorad'
import type { PlaceBlock, PlaceBoard, PlaceMetrial, SaleBlockModel } from '@/Source/Models/Orders'
import { Arc2d } from '@/Source/Base/CAD'
import { ArrayExt } from '@/Source/Base/ArrayExt'

export class OddmentsHelper {
  squarePath: Path
  canPutPaths: Path[]
  offset = 3.5

  initSquarePath() {
    const seqW = PlaceStore.sysConfig.scrapBlockSquare || 200
    const wmin = PlaceStore.sysConfig.srcapBlockWidthMin || 100
    const wmax = PlaceStore.sysConfig.scrapBlockWidthMax || 600
    this.squarePath = NestCache.CreatePath(60, 60, 0)
    this.canPutPaths = [
      NestCache.CreatePath(seqW, seqW, 0),
      NestCache.CreatePath(wmin, wmax, 0),
      NestCache.CreatePath(wmax, wmin, 0),
    ]
  }

  async analyzeAllScrapSpace(pm1: PlaceMetrial, pb1: PlaceBoard) {
    this.initSquarePath()
    if (pm1 && pb1) // 当前板 先做
    {
      this.offset = (pm1.CutDia) / 2
      await this.analyzeScrapSpace(pm1, pb1)
    }
    const order = PlaceStore.order
    for (const pm of order.MetrialList) {
      this.offset = (pm.CutDia) / 2
      for (const pb of pm.BoardList)
        await this.analyzeScrapSpace(pm, pb)
    }
  }

  async analyzeScrapSpace(pm: PlaceMetrial, board: PlaceBoard) {
    console.log('OddmentsHelper analyzeScrapSpace', pm, board)
    if (board.needToScrapSpace == false)
      return
    await InitClipperCpp()

    const border = 0// PlaceStore.sysConfig.BoardBorder_A;
    let borderB = PlaceStore.sysConfig.BoardBorder_B
    if (board.IsDo2Face == false)
      borderB = 0
    let binPath: Path
    if (board.Points && board.Points.length > 0) // 非矩形板

      binPath = new Path(board.Points)

    else // 矩形板

      binPath = new Path([{ x: border, y: border }, { x: board.Width - borderB, y: border }, { x: board.Width - borderB, y: board.Length - borderB }, { x: border, y: board.Length - borderB }])

    binPath.Id = undefined
    PathGeneratorSingle.RegisterId(binPath)
    // 容器
    const container = new Container(binPath)

    for (let i = 0; i < board.blockList.length; i++) {
      const block = board.blockList[i]
      const part = this.toPart(block, binPath, this.offset * 2, i)
      // 设置位置
      part[0].PlacePosition = { x: (part[1].x + block.PlaceX - border) * 1e4, y: (part[1].y + block.PlaceY - border) * 1e4 }
      container.PlacedParts.push(part[0])
    }
    // for (let i = 0; i < board.blockList.length; i++)
    // {
    //     let block = board.blockList[i];
    //     let part = this.toPart(block, binPath, 0, board.blockList.length + i);
    //     //设置位置
    //     part[0].PlacePosition = { x: (part[1].x + block.PlaceX - border) * 1e4, y: (part[1].y + block.PlaceY - border) * 1e4 };
    //     container.PlacedParts.push(part[0]);
    // }

    // let f = new NestFiler();
    // f.Write(container.PlacedParts.length);
    // for (let part of container.PlacedParts)
    //     f.Write(part.State.Contour.BigIntPoints);
    // container.WriteFile(f);
    // console.log(JSON.stringify(f._datas));

    // FileZip.WriteFile(`${board.BoardID}.container`, JSON.stringify(f._datas));

    board.ScrapBlockList = [] // 清空
    try {
      const spaces = ParseOddments(container, binPath, this.offset, this.squarePath, this.canPutPaths)
      for (const space of spaces) {
        const sb = this.toScrapBlock(space)
        sb.PlaceX = sb.PlaceX + border
        sb.PlaceY = sb.PlaceY + border
        board.ScrapBlockList.push(sb)
      }
    }
    catch (err) {
      // throw new Error(`计算余料空间异常:${pm.FullName} 第${board.BoardID}片`);
      console.log(`计算余料空间异常:${pm.FullName} 第${board.BoardID}片`)
      throw new Error(`计算余料空间异常:${pm.FullName} 第${board.BoardID}片;请保存优化联系售后工程师分析`)
    }

    // 分析小板内 造型孔洞 余料空间
    for (const block of board.blockList) {
      let childBlocks: PlaceBlock[]
      // 造型孔
      for (const m of block.Models) {
        if (m.Depth < block.Thickness - 0.05)
          continue
        if (m.HasContour == false)
          continue
        if (m.IsDo == false)
          continue
        if (!childBlocks) {
          childBlocks = board.blockList.filter(t => t.PlaceX > block.PlaceX
                        && t.PlaceX + t.PlaceWidth < block.PlaceX + block.PlaceWidth
                        && t.PlaceY > block.PlaceY
                        && t.PlaceY + t.PlaceLength < block.PlaceY + block.PlaceLength)
        }
        const spaces = await this.getModelSpace(block, m, childBlocks)
        for (const space of spaces) {
          const sb = this.toScrapBlock(space)
          // sb.PlaceX += border;
          // sb.PlaceY += border;
          sb.PlaceX += block.PlaceX
          sb.PlaceY += block.PlaceY
          board.ScrapBlockList.push(sb)
        }
      }
    }

    board.needToScrapSpace = false
    // console.log(`board id=${board.BoardID} find scrap block count = ${board.ScrapBlockList.length}`);
    // console.log(`${board.BoardID} scraping ok`)
  }

  async getModelSpace(block: PlaceBlock, m: SaleBlockModel, blocks: PlaceBlock[]): Promise<any[]> {
    if (m.OriginModeling == null)
      return []
    const pts = m.OriginModeling.outline.pts
    const buls = m.OriginModeling.outline.buls
    // blocks = [];
    // 起点
    let pts_n = []
    for (let i = 0; i < pts.length; i++) {
      const p0 = BlockHelper.GetPlaceXYInBlock(block, pts[i].x, pts[i].y, false, false)
      pts_n.push({ x: p0.X, y: p0.Y, bul: buls[i] })
    }
    // 如果反面,翻转方向，重设坐标
    if (block.IsTurnOver)
      pts_n = this.reversePoint(pts_n, block.CuttingWidth)

    // pts_n.forEach(t => { t.x += block.PlaceX; t.y += block.PlaceY; });

    let polyLine
    polyLine = Points2Polyline(pts_n)
    const cicle = PolylineHelper.ConverPolyLin2Circle(polyLine)
    if (cicle)
      polyLine = cicle
    const cureW2 = new CurveWrap(polyLine, this.offset, false)
    const pts2 = cureW2.GetInsidePoints2(this.offset)
    if (!pts2)
      return []
    let posx = 10000
    let posy = 10000
    for (const p of pts2) {
      if (p.x < posx)
        posx = p.x
      if (p.y < posy)
        posy = p.y
    }
    for (const p of pts2) {
      p.x -= posx
      p.y -= posy
    }
    const binPath = new Path(pts2)
    if (binPath.Area < 15000)
      return []

    await InitClipperCpp()
    binPath.Id = undefined
    PathGeneratorSingle.RegisterId(binPath)
    // 容器
    const container = new Container(binPath)
    let i = 0
    for (const b of blocks) {
      const part = this.toPart(b, binPath, this.offset, i)
      if (part[0].State.Contour == null)
        continue
      part[0].Id = i++
      // let isin = this.blockInBlock(block, b, polyLine);
      // if (isin == false) continue;
      // let part = this.toPart(b, binPath, this.offset, i++);

      // 设置位置
      part[0].PlacePosition = { x: (b.PlaceX - block.PlaceX - posx) * 1e4, y: (b.PlaceY - block.PlaceY - posy) * 1e4 }
      container.PlacedParts.push(part[0])
    }
    try {
      // let f = new NestFiler();
      // f.Write(container.PlacedParts.length);
      // for (let part of container.PlacedParts)
      //     f.Write(part.State.Contour.BigIntPoints);
      // container.WriteFile(f);
      // console.log(JSON.stringify(f._datas));
      // FileZip.WriteFile(`${board.BoardID}.container`, JSON.stringify(f._datas));

      const spaces = ParseOddments(container, binPath, this.offset, this.squarePath, this.canPutPaths)
      for (const space of spaces)
        space.Points.forEach((t) => { t.x += posx; t.y += posy })

      return spaces
    }
    catch (err) {
      console.log(`板${block.BlockNo}造型孔分析余料空间失败.`)
      throw new Error(`板${block.BlockNo}造型孔分析余料空间失败.请保存优化联系售后工程师分析`)
    }
    // let spaces = ParseOddments(container, binPath, this.offset, this.squarePath, this.canPutPaths);
    // for (let space of spaces)
    // {
    //     space.Points.forEach(t => { t.x += posx; t.y += posy; });
    // }
    // return spaces;
  }

  toPart(block: PlaceBlock, binPath: Path, offset: number, id: number): [Part, Vector2] {
    const w = 10000
    const binPath2 = new Path([{ x: -w, y: -w }, { x: w, y: -w }, { x: w, y: w }, { x: -w, y: w }])
    const part = new Part()
    part.Id = id

    part.UserData = { bno: block.BlockNo, size: block.Size, isYX: block.IsUnRegular, width: block.CuttingWidth, length: block.CuttingLength }

    let path: Path
    let bPoint: Vector2

    const sizeOff = BlockSizePlus.getOffDis(block) // 外扩偏移
    const hasSizeOff = (sizeOff.left + sizeOff.right + sizeOff.top + sizeOff.bottom) > (block.PlaceMetrial.CutDia * 2 + block.PlaceMetrial.CutGap * 2 + 0.01)

    // 矩形 且 带板外不偏移
    if (block.IsUnRegular == false && hasSizeOff == false) {
      path = NestCache.CreatePath(block.PlaceWidth + sizeOff.left + sizeOff.right, block.PlaceLength + sizeOff.top + sizeOff.bottom, offset)
      bPoint = path.OrigionMinPoint
      // 轮廓
      part.Init2(path, binPath2, [0])
    }
    else // 异形
    {
      const pts = []

      const cured = BlockPlus.getBorder(block)
      for (const l of cured) {
        if (l instanceof Arc2d)
          pts.push({ x: l.StartPoint.m_X, y: l.StartPoint.m_Y, bul: l.Bul })

        else
          pts.push({ x: l.StartPoint.m_X, y: l.StartPoint.m_Y, bul: 0 })
      }

      let polyLine
      polyLine = Points2Polyline(pts)
      polyLine.CloseMark = true

      const cureW = new CurveWrap(polyLine, offset, true)
      const pts2 = cureW.GetOutsidePoints()
      arrayRemoveDuplicateBySort(pts, (p1, p2) => (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) < 1e-2)
      path = new Path(pts2)

      const cicle = PolylineHelper.ConverPolyLin2Circle(polyLine)
      if (!cicle) {
        const area = path.BoundingBox.area - path.Area
        if (area < 15000 && pts.length > 6)
          path = NestCache.CreatePath(block.PlaceWidth, block.PlaceLength, offset)
      }
      part.Init2(path, binPath2, [0])

      // 不能放下,那么尝试不简化路径
      if (!path.IsRect && !cicle && part.RotatedStates.length == 0) {
        const pts2 = Polyline2Points(polyLine, true, offset)[1]
        arrayRemoveDuplicateBySort(pts2, (p1, p2) => (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) < 1e-2)
        path = new Path(pts2)
        part.Init2(path, binPath, [0])
      }
      // 轮廓
      bPoint = path.OrigionMinPoint
    }

    if (hasSizeOff) {
      bPoint.x = bPoint.x - sizeOff.left
      bPoint.y = bPoint.y - sizeOff.bottom
    }

    return [part, bPoint]
  }

  toScrapBlock(path: Path): ScrapBlock {
    const bx = path.OrigionMinPoint.x
    const by = path.OrigionMinPoint.y
    const points = path.Points
    const ptsX = points.map((t) => { return t.x })
    const ptsY = points.map((t) => { return t.y })

    const x0 = ArrayExt.min(ptsX, t => t)
    const y0 = ArrayExt.min(ptsY, t => t)
    const x1 = ArrayExt.max(ptsX, t => t)
    const y1 = ArrayExt.max(ptsY, t => t)

    const sp = new ScrapBlock(x0, y0, x1 - x0, y1 - y0)

    const pts = points.map((t) => { return { x: t.x - x0, y: t.y - y0 } })

    sp.PlaceX = bx + x0
    sp.PlaceY = by + y0

    sp.setPoints(pts)
    return sp
  }

  // 判断板是否在造型洞里头
  blockInBlock(block: PlaceBlock, b: PlaceBlock, modlPl: Polyline, posX = 0, posY = 0): boolean {
    const cured = BlockPlus.getBorder(b)
    const pts = []
    for (const l of cured) {
      if (l instanceof Arc2d)
        pts.push({ x: l.StartPoint.m_X, y: l.StartPoint.m_Y, bul: l.Bul })

      else
        pts.push({ x: l.StartPoint.m_X, y: l.StartPoint.m_Y, bul: 0 })
    }
    const polyLine = Points2Polyline(pts)
    const cureW2 = new CurveWrap(polyLine, this.offset, false)
    const pts2 = cureW2.GetInsidePoints2(this.offset)

    // pts2 = pts2.map(t => { return { x: t.x + b.PlaceX - block.PlaceX - posX, y: t.y + b.PlaceY - block.PlaceY - posY }; });

    for (const t of pts2) {
      const x = t.x + b.PlaceX - block.PlaceX - posX
      const y = t.y + b.PlaceY - block.PlaceY - posY
      if (modlPl.PtInCurve(new Vector3(x, y, 0)) == false)
        return false
    }

    return true
  }

  private reversePoint(pts, w): any[] {
    const newPts = []
    for (let i = pts.length - 1; i >= 0; i--) {
      const p = pts[i]
      const x = p.x
      const y = p.y
      const j = i == 0 ? pts.length - 1 : i - 1

      const bul = pts[j].bul
      newPts.push({ x, y, bul })
    }
    return newPts
  }
}
