import { ref } from 'vue'
import { SysConfig } from '../biz/SysConfig'
import type { HoleDetail } from '../vo/hole/HoleDetail'
import { PlaceBorderContour } from '../vo/order/PlaceBorderContour'
import type { contourDetail, sideHoleDetail } from '../vo/product/plateModel'
import { BlockModel } from '../vo/model/BlockModel'
import { equal, getRect } from './base/MathComm'
import { PlaceBlock } from '../vo/order/PlaceBlock'
import { CADExt, Point2d } from './base/CAD'
import { BlockPoint } from '../vo/point/BlockPoint'
// 小板数据转换
export function transformXb(blockList, sysConfig) {
  const transformedData: any = []
  for (let i = 0; i < blockList.length; i++) {
    const plate = blockList[i]
    const transformedPlate = {
      l: plate.length,
      w: plate.width,
      line: plate.texture,
      face: plate.placeHole,
      id: i + 1, // 使用当前索引加1作为id的值
      bno: plate.plateNo || undefined,
      holeFaceCount: 3,
      isRect: !plate.isUnRegular,
      hasHole: plate.hasHole,
      isdtwosided: false, // sysConfig.isDoubleFaceBlockFirst,
    }
    transformedData.push(transformedPlate)
  }
  return transformedData
}

// 获取优化次数-根据小板
export function getOpCs(bans) {
  let xyhcs = 50
  if (bans.length > 1000)
    xyhcs = 40
  if (bans.length > 1500)
    xyhcs = 30
  if (bans.length > 2000)
    xyhcs = 25
  if (bans.length > 4000)
    xyhcs = 20
  if (bans.length > 6000)
    xyhcs = 15
  if (bans.length > 10000)
    xyhcs = 10
  if (bans.length > 15000)
    xyhcs = 5
  if (bans.length > 20000)
    xyhcs = 1
  return xyhcs
}

// 大板数据转换
export function transformDb(materialList, sysConfig) {
  let transbndData: any = []
  const { boardOffsetX, boardOffsetY } = sysConfig
  function addParams(length, width) {
    return { l: length, w: width, x: boardOffsetX, y: boardOffsetY }
  }
  const remainBoardList = materialList[0]?.remainBoardList || []
  remainBoardList.forEach((remainBoard) => {
    transbndData.push(addParams(remainBoard.length, remainBoard.width))
  })
  materialList.forEach((plate) => {
    transbndData.push(addParams(plate.orgLength || plate.length, plate.orgWidth || plate.width))
  })
  return transbndData
}

// 取最厚的一个大板的板厚
function findMaxThickness(data) {
  let maxThickness = 0
  let maxThicknessData = null

  data?.forEach((item) => {
    if (item.thickness > maxThickness) {
      maxThickness = item.thickness
      maxThicknessData = item
    }
  })
  return maxThicknessData
}

// 获取排版间隙
export function transformThickness(materialList, sysConfig) {
  const data = _getDefaultKnife(materialList, sysConfig)
  return data.diameter + sysConfig.blockKnifeLineSpacing // sysConfig.cutKnifeGap
}

// 获取开料刀
export function _getDefaultKnife(materialList, sysConfig) {
  const obj = new SysConfig(sysConfig)
  const target = findMaxThickness(materialList)
  const data = obj.getDefaultKnife(target.thickness)
  return data
}

// 数据转换
// 数据转换前的数据集存一下   转回去的时候估计要用
const beforeConvertData = ref<any>()
export function convertData(newJson) {
  beforeConvertData.value = JSON.parse(JSON.stringify(newJson))
  const { goodsList, machineDTO, optimizeBoardModelDOS, orderList, plan, plateList, plateModels, orderPartsRespVOS, planActualGoodsModelDO } = newJson

  // configList
  let mixGoodsId = '';
  const { setting, label } = machineDTO
  const configRef = typeof (setting) == 'string' ? JSON.parse(setting) : setting
  const sysConfig = new SysConfig(setting)
  const tagConfig = label
  const styleConfig = JSON.stringify(configRef?.styleConfigsRef)
  const configList = [
    {
      Type: 1,
      Setting: JSON.stringify(configRef),
      MachineID: machineDTO.id,
    },
    {
      Type: 2,
      Setting: label,
      MachineID: machineDTO.id,
    },
    {
      Type: 3,
      Setting: JSON.stringify(configRef?.styleConfigsRef),
      MachineID: machineDTO.id,
    },
  ]
  // planOrder

  let blockInfoArr: any = []

  Array.isArray(optimizeBoardModelDOS) && optimizeBoardModelDOS.forEach((item) => {
    if (item) {
      let str = `${item.material}--${item.color}--${item.spec}--${item.plateNum}片--0.251`
      blockInfoArr.push(str)
    }

  })
  //  排单数据
  let planOrder: any = {}
  // 订单流程时  plan（排单数据）是没有的 忽略
  if (plan) {
    planOrder = {
      goodsIdList: null,
      planId: plan.id,
      planCode: plan.id,
      status: plan.status,
      machineId: machineDTO.id,
      createTime: plan.createTime,
      creator: plan.creator,
      planTime: plan.planTime,
      produceTime: plan.produceTime,
      producer: '', //
      remark: plan.remark,
      organId: plan.organId,
      updateTime: plan.updateTime,
      planType: plan.type,
      orderIds: `[${plan.orderNos}]`,
      blockInfo: blockInfoArr.join(';'),
      orderList: null,
      deleted: false,
      sort: plan.sort,
    }
  }

  // materialList  goodsList大板中的 goodid没用
  let materialList: any = []
  let newGoodsList: any = []
  goodsList.forEach((item) => { // 去重的
    let i = newGoodsList.findIndex(e => e.goodsId == item.goodsId)
    if (i == -1) {
      newGoodsList.push(item)
    }
  })
  //  混单的情况 才会走这个
  if(planActualGoodsModelDO != undefined){
    newGoodsList = [planActualGoodsModelDO]
    mixGoodsId = planActualGoodsModelDO.goodsId
  }

  newGoodsList.forEach((item) => {
    if (optimizeBoardModelDOS && optimizeBoardModelDOS.length > 0) {
      let placeItem = optimizeBoardModelDOS.find(e => e?.goodsId == item.goodsId)
      if (placeItem) {
        if (placeItem.usedBoardInfo) {
          let keys = Object.keys(placeItem)
          keys.forEach((k) => {
            if (placeItem[k] != null && placeItem[k] != undefined) {
              item[k] = placeItem[k]
            } else {
              // console.log('数据为空，不做复写操作,', k, placeItem[k], item[k])
            }
          })
        }
      } else {
        // console.error('没找到 异常 排查: 大板的id  和 优化数据中的大板id  对应不上')
      }
    }
    
    const temp = {
      orderId: item.orderId,
      goodsId: item.goodsId,
      goodsName: item.goodsName,
      spec: item.spec,
      material: item.material,
      color: item.color,
      brand: item.brand,
      width: item.width,
      length: item.height,
      isOptimized: item.isOptimized,
      thickness: item.thickness,
      cutBorder: item.cutBorder,
      CutDia: item.CutDia,
      CutGap: item.CutGap,
      boardCount: item.boardCount || 0,
      cutKnifeGap: item.cutKnifeGap,
      minBoardId: item.minBoardId,
      maxBoardId: item.maxBoardId,
      avgUsageRateAll: item.avgUsageRateAll,
      avgUsageRateExcludeLastBoard: item.avgUsageRateExcludeLastBoard,
      usageRateLastBoard: item.usageRateLastBoard,
      usedBoardInfo: item.usedBoardInfo,
      blockPlaceInfo: item.blockPlaceInfo,
      status: item.status,
      hasTexture: item.texture,
      // texture: item.texture || '',
      orgWidth: item.orgWidth || item.width,
      orgLength: item.orgLength || item.height,
      boardCountRemain: item.boardCountRemain,
      remainBoardInfo: item.remainBoardInfo,
      preMillingSize: item.preMillingSize,
      remainBoardList: [],
      isHelpCut: item.isHelpCut,
      helpCutGap: item.helpCutGap,
      cutKnifeId: item.cutKnifeId,
      helpKnifeId: item.helpKnifeId,
      updateDate: item.updateDate,
    }
    materialList.push(temp)
  })
  // orderList
  let areaDeleted = false
  let newOrderList: any = []
  orderList.forEach((item) => {
    let temp = {
      customerId: item.customerId,
      customer: item.customer,
      phoneNumber: item.phoneNumber,
      orderDate: item.orderDate,
      salemanId: item.salemanId,
      dealer: item.dealer,
      dealerPhoneNumber: item.dealerPhoneNumber,
      address: item.address,
      splitter: item.splitter,
      status: item.status,
      amount: 0,
      remark: item.remark,
      customOrderNo: item.customOrderNo,
      deliveryDate: item.deliveryDate,
      pushConfig: item.pushConfig || false,
      offerListStr: item.offerListStr || null,
      cancelStatus: item.cancelStatus || 0,
      itemList: item.itemList || null,
      goodsList: item.goodsList || null,
      offerList: item.offerList || null,
      totalOrderOfferList: item.totalOrderOfferList || null,
      blockList: item.blockList || null,
      dataBlockList: item.dataBlockList || null,
      objectList: item.objectList || null,
      dataObjectList: item.dataObjectList || null,
      goodsInfoList: item.goodsInfoList || null,
      orderProcessList: item.orderProcessList || null,
      editFun: null,
      saleman: item.salesman,
      orderId: item.id,
      createTime: item.createTime,
      organId: item.organId,
      schduleDeliveryDate: null,
      orderType: item.orderType,
      orderSort: item.orderSort,
      dataType: item.dataType,
      deleted: item.deleted,
      processStatus: null,
    }
    areaDeleted = item.deleted
    newOrderList.push(temp)
  })

  // blockList
  let blockList: any = []

  plateList.forEach((item, i) => {
    const temp = {
      area: item.area,
      roomName: item.roomName,
      bodyName: item.bodyName,
      roomId: item.roomId || 0,
      bodyId: item.bodyId || 0,
      processGroupld: item.processGroupld,
      orderId: item.orderId,
      blockId: item.plateId,
      goodsId: mixGoodsId || item.goodsId,
      oldBlockId: item.plateId,
      blockNo: item.plateNo || (i + 1).toString(),
      labelNo: '',
      blockName: item.name,
      width: item.width,
      length: item.height,
      customPlateNo: item.customPlateNo,  // 自定义板编号
      thickness: item.thickness,
      isCurvedGroundLine: false, // item.isCurvedGroundLine, // 是否弧线 不处理异形 异形
      isUnRegular: item.isSpecialShaped,
      sealLeft: item.sealLeft,
      sealRight: item.sealRight,
      sealTop: item.sealUp,
      sealBottom: item.sealDown,
      texture: item.texture,
      placeHole: item.holeFace,
      abnormalOffsetY: item.offsetX,
      abnormalOffsetX: item.offsetY,
      remarkJson: '',
      dataType: 2,
      processGroupName: item.processGroupName,
      type: item.type,
      openDoorType: item.openDoorType,
      extraRemark: {},
      itemId: item.itemId,
      plateRemark: item.remark,
    }
    blockList.push(temp)
  })

  let blockDetailList: any = []
  let holeId = 1
  plateModels.forEach((item) => {
    const fitBlock = plateList.find(e => e.plateId === item.plateId)
  
    if (item.pointInfo && item.boardData && false) {
      // webCAd 拆单数据转换
      let _boardData = JSON.parse(item.boardData)
      let _data = JSON.parse(item.pointInfo)
      // const { holes, outline, modeling, modeling2D,sideModeling,originOutlin} = _data
      const { HoleDetail, OrgPointDetail, PointDetail, SideHoleDetail,SideModelDetail,ModelDetail} = _data
      let holeDetailArr: HoleDetail[] = []

      if (HoleDetail) {
        HoleDetail.forEach(frontBackHoles => {
          let holeDetail: HoleDetail = {
            holeId: frontBackHoles.HoleID,
            holeType: frontBackHoles.HoleType || 0,
            face: frontBackHoles.Face,
            pointX: frontBackHoles.PointX || 0,
            pointY: frontBackHoles.PointY || 0,
            pointZ: frontBackHoles.PointZ || 0,
            radius: frontBackHoles.Radius || 0,
            depth: frontBackHoles.Depth || 0,
            pointX2: 0,
            pointY2: 0,
            pointZ2: 0,
            endPoint: 0,
            angle: frontBackHoles.Angle,
            name:'',
          }
          holeId = holeId + 1
          holeDetailArr.push(holeDetail)
        });
      }
      let sideHoleDetailArr: sideHoleDetail[] = []
      if (SideHoleDetail) {
        SideHoleDetail.forEach(sideHoles => {
          let sideholeDetail: sideHoleDetail = {
            holeId:sideHoles.HoleID || holeId,
            holeType: sideHoles.HoleType,
            face: sideHoles.Face || 0,
            pointX: sideHoles.PointX || 0,
            pointY: sideHoles.PointY || 0,
            pointZ: sideHoles.PointZ || 0,
            radius: sideHoles.Radius || 0,
            depth: sideHoles.Depth || 0,
            pointX2: sideHoles.PointX2 || 0,
            pointY2: sideHoles.PointY2 || 0,
            pointZ2: sideHoles.PointZ || 0,
            endPoint: sideHoles.EndPoint,
            angle: 0,
            direction: null,
          }
          holeId = holeId + 1
          sideHoleDetailArr.push(sideholeDetail)
        });
      }
      let modelDetailArr: contourDetail[] = []
      let sideModelDetailArr: contourDetail[] = []
      // 造型还缺数据  ！！！
      if(Array.isArray(ModelDetail) && ModelDetail.length > 0){
        ModelDetail.forEach(_item => {
          
          const modelDetail: contourDetail = {
            modelId: _item.ModelID || ++holeId,
            lineId: _item.LineID,
            face: _item.Face || 0,
            knifeName: _item.KnifeName,
            knifeRadius: _item.KnifeRadius || 0,
            depth: _item.Depth || 0,
            originModeling: _item.OriginModeling,
            modelPoint: _item.PointList || [],
            modelOffSet: _item.OffSetList || [],
          }
          modelDetailArr.push(modelDetail)
        });
      }

      if(Array.isArray(SideModelDetail) && SideModelDetail.length > 0){
        SideModelDetail.forEach(_item => {
          const sideModelDetail: any = {
            modelId:_item.ModelID,
            lineId: _item.LineID,
            face: _item.Face,
            knifeName: _item.KnifeName,
            knifeRadius: _item.KnifeRadius || 0,
            depth: _item.Depth || 0,
            originModeling: _item.OriginModeling,
            modelPoint: [],
            modelOffSet: [],
          }
          sideModelDetailArr.push(sideModelDetail)
        });
      }
      
      let _PointDetail = [...PointDetail]
      // let p = {...PointDetail[0]}
      // p.PointID = _PointDetail.length +1
      // _PointDetail.push(p)
       
      let _OrgPointDetail = [...OrgPointDetail]
      // let orgP = {...OrgPointDetail[0]}
      // orgP = _OrgPointDetail.length +1
      // _OrgPointDetail.push(orgP)

      // if(fitBlock.isSpecialShaped == false){
      //   _PointDetail = [];
      //   _OrgPointDetail = [];
      // }

      const temp = {
        blockId: item.plateId,
        orderId: item.orderId,
        pointDetail: _PointDetail || PointDetail || [],
        modelDetail: modelDetailArr || [],
        holeDetail: holeDetailArr || [],
        models: modelDetailArr || [],
        remarkParams: {
          sideRemark: item.sideRemark || '',
          extraRemark: item.extraRemark || '',
          specialRemarkDei: item.specialRemarkDei || '',
          extraHandleRemark: item.extraHandleRemark || '',
          drillsRemark: item.drillsRemark || '',
          infoRemark: item.infoRemark || '',
          remarkJsonDei: item.remarkJsonDei || '',
          remarkJson: item.remarkJson || '',
          specialRemark: item.specialRemark || '',
        },
        offSet: {
          x: _boardData.offsetTanslation.x,
          y: _boardData.offsetTanslation.y,
          z: _boardData.offsetTanslation.z || 0,
        },
        orgPointDetail: _OrgPointDetail || OrgPointDetail || [],
        cutSize: {
          width: fitBlock.splitWidth,
          height: fitBlock.splitHeight,
        },
        modelListSide: sideModelDetailArr,//[], //  侧槽
        sideHoleDetail: sideHoleDetailArr || [],
        isUnRegular: fitBlock && fitBlock.isSpecialShaped || false, // item.isSpecialShaped || false,
        plateExtraRemark: item.plateExtraRemark,
        has2DModel: ModelDetail && ModelDetail.filter(e=>e.OffSetList).length > 0,
        has3DModel: _boardData.modeling3D && _boardData.modeling3D.length > 0 //        item.has3DModel
      }
      blockDetailList.push(temp)
    } else {
      // API导入 拆单数据转换
      // 孔 （正面/反面
      let holeDetailArr: HoleDetail[] = []
      if (item.holeDetail) {
        item.holeDetail.forEach((hole) => {
          //  direction 只有侧孔有  angle 生产数据中没有
          const { holeType, faceType, startX, startY, startZ, radius, depth, direction, angle, endX, endY, endZ } = hole
          let holeDetail: HoleDetail = {
            holeId,
            holeType: holeType || 0,
            face: faceType,
            pointX: startX || 0,
            pointY: startY || 0,
            pointZ: startZ || 0,
            radius: radius || 0,
            depth: depth || 0,
            pointX2: endX || 0,
            pointY2: endY || 0,
            pointZ2: endZ || 0,
            endPoint: 0,
            angle: 0,
            name: '',
          }
          holeId = holeId + 1
          holeDetailArr.push(holeDetail)
        })
      }

      // 侧孔
      let sideHoleDetailArr: sideHoleDetail[] = []
      if (item.sideHoleDetail) {
        item.sideHoleDetail.forEach((sideHole) => {
          //  direction 只有侧孔有  angle 生产数据中没有
          const { holeType, faceType, startX, startY, startZ, radius, depth, direction, angle, endX, endY, endZ } = sideHole
          let sideholeDetail: sideHoleDetail = {
            holeId,
            holeType,
            face: faceType || 0,
            pointX: startX || 0,
            pointY: startY || 0,
            pointZ: startZ || 0,
            radius: radius || 0,
            depth: depth || 0,
            pointX2: endX || 0,
            pointY2: endY || 0,
            pointZ2: endZ || 0,
            endPoint: 0,
            angle: 0,
            direction,
          }
          holeId = holeId + 1
          sideHoleDetailArr.push(sideholeDetail)
        })
      }

      let modelDetailArr: contourDetail[] = []
      let sideModelDetailArr: contourDetail[] = []
      if (item.contourDetail) {
        // 垂直造型
        item.contourDetail.forEach((model) => {
          const { modelId, lineID, faceType, knifeName, knifeRadius, depth, originModeling, modelPoint, modelOffSet } = model
          const modelDetail: contourDetail = {
            modelId: model.id || ++holeId,
            lineId: lineID,
            face: faceType,
            knifeName: knifeName || '',
            knifeRadius: knifeRadius || 0,
            depth: depth || 0,
            originModeling,
            modelPoint: model.pointList || [],
            modelOffSet: model.offSetList || [],
          }
          modelDetailArr.push(modelDetail)
          // if(faceType == 0 || faceType == 1){
          // }
        })
      }

      if (item.sideDetail) {
        item.sideDetail.forEach((model) => {
          const { modelId, lineId, faceType, knifeName, knifeRadius, depth, originModeling, modelPoint, modelOffSet } = model

          const sideModelDetail: any = {
            modelId: modelId || ++holeId,
            lineId,
            face: faceType,
            knifeName: knifeName || '',
            knifeRadius: knifeRadius || 0,
            depth: depth || 0,
            originModeling,
            modelPoint: model.pointList || [],
            modelOffSet: model.offSetList || [],
          }
          sideModelDetailArr.push(sideModelDetail)
        })
      }

      const temp = {
        blockId: item.plateId,
        orderId: item.orderId,
        pointDetail: item.pointDetail || [],
        modelDetail: modelDetailArr || [],
        holeDetail: holeDetailArr || [],
        models: modelDetailArr || [],
        remarkParams: {
          sideRemark: item.sideRemark,
          extraRemark: item.extraRemark,
          specialRemarkDei: item.specialRemarkDei,
          extraHandleRemark: item.extraHandleRemark,
          drillsRemark: item.drillsRemark,
          infoRemark: item.infoRemark,
          remarkJsonDei: item.remarkJsonDei,
          remarkJson: item.remarkJson,
          specialRemark: item.specialRemark,
        },
        offSet: {
          x: item.offsetX,
          y: item.offsetY,
          z: item.offsetZ || 0,
        },
        orgPointDetail: item.rawPointDetail || [],
        cutSize: {
          width: item.splitWidth,
          height: item.splitLength,
        },
        modelListSide: sideModelDetailArr,//[], //  侧槽
        sideHoleDetail: sideHoleDetailArr || [],
        isUnRegular: fitBlock && fitBlock.isSpecialShaped || false, // item.isSpecialShaped || false,
        plateExtraRemark: item.plateExtraRemark,
        has2DModel: item.has2DModel,
        has3DModel: item.has3DModel
      }
      /**
       * 异形的情况下  已经作用到  orgPoint 和 points  里面了 
       * plateExtraRemark 数据结构 
       * plateExtraRemark：{
       *       CustomNumber // webCAD 图纸编号
       *       extra ：{
       *        boardType: "背板"
       *        composingFace: "任意面"
                edgeRemarks: ["", "", "", ""]
                has2DModel: false
                has3DModel: false
                processList: []
                rectSealDetail:[
                    {
                      SealColor: "F-A330铁力木",  // 封边颜色
                      SealSize: 1,      // 封边尺寸
                      ReservedEdge: 0,  // 外扩尺寸
                      PointID: 1    // 第几个线条
                    }
                  ]
       *        }
    
       * } 
       */
      blockDetailList.push(temp)
    }




  })

  let resData = {
    machineId: machineDTO.id,
    machineName: machineDTO.name,
    materialList,
    configList,
    sysConfig,
    tagConfig,
    styleConfig,
    planOrder,
    orderList: newOrderList,
    blockList,
    blockDetailList,
    areaDeleted,
    sealInfo: orderPartsRespVOS && orderPartsRespVOS.filter(e => e.type == '封边条'),
  }
   
  // console.log('转换数据:', resData)
  return resData
}

// 数据转换-没有机台
// 数据转换前的数据集存一下   转回去的时候估计要用
export function convertNoMachineData(newJson) {
  beforeConvertData.value = JSON.parse(JSON.stringify(newJson))
  const { goodsList, optimizeBoardModelDOS, orderList, plan, plateList, plateModels, orderPartsRespVOS
  } = newJson


  let blockInfoArr: any = []

  Array.isArray(optimizeBoardModelDOS) && optimizeBoardModelDOS.forEach((item) => {
    let str = `${item.material}--${item.color}--${item.spec}--${item.plateNum}片--0.251`
    blockInfoArr.push(str)
  })
  //  排单数据
  let planOrder: any = {}
  // 订单流程时  plan（排单数据）是没有的 忽略
  if (plan) {
    planOrder = {
      goodsIdList: null,
      planId: plan.id,
      planCode: plan.id,
      status: plan.status,
      // machineId: machineDTO.id,
      createTime: plan.createTime,
      creator: plan.creator,
      planTime: plan.planTime,
      produceTime: plan.produceTime,
      producer: '', //
      remark: plan.remark,
      organId: plan.organId,
      updateTime: plan.updateTime,
      planType: plan.type,
      orderIds: `[${plan.orderNos}]`,
      blockInfo: blockInfoArr.join(';'),
      orderList: null,
      deleted: false,
      sort: plan.sort,
    }
  }

  // materialList  goodsList大板中的 goodid没用
  let materialList: any = []
  let newGoodsList: any = []
  goodsList.forEach((item) => { // 去重的
    let i = newGoodsList.findIndex(e => e.goodsId == item.goodsId)
    if (i == -1) {
      newGoodsList.push(item)
    }
  })
  newGoodsList.forEach((item) => {
    if (optimizeBoardModelDOS && optimizeBoardModelDOS.length > 0) {
      let placeItem = optimizeBoardModelDOS.find(e => e.goodsId == item.goodsId)
      if (placeItem) {
        if (placeItem.usedBoardInfo) {
          let keys = Object.keys(placeItem)
          keys.forEach((k) => {
            if (placeItem[k] != null && placeItem[k] != undefined) {
              item[k] = placeItem[k]
            } else {
              // console.log('数据为空，不做复写操作,', k, placeItem[k], item[k])
            }
          })
        }
      } else {
        console.error('没找到 异常 排查: 大板的id  和 优化数据中的大板id  对应不上')
      }
    }
    const temp = {
      orderId: item.orderId,
      goodsId: item.goodsId,
      goodsName: item.goodsName,
      spec: item.spec,
      color: item.color,
      brand: item.brand,
      width: item.width,
      length: item.height,
      isOptimized: item.isOptimized,
      thickness: item.thickness,
      cutBorder: item.cutBorder,
      CutDia: item.CutDia,
      CutGap: item.CutGap,
      boardCount: item.boardCount || 0,
      cutKnifeGap: item.cutKnifeGap,
      minBoardId: item.minBoardId,
      maxBoardId: item.maxBoardId,
      avgUsageRateAll: item.avgUsageRateAll,
      avgUsageRateExcludeLastBoard: item.avgUsageRateExcludeLastBoard,
      usageRateLastBoard: item.usageRateLastBoard,
      usedBoardInfo: item.usedBoardInfo,
      blockPlaceInfo: item.blockPlaceInfo,
      status: item.status,
      hasTexture: item.texture,
      material: item.material,
      // texture: item.texture || '',
      orgWidth: item.orgWidth,
      orgLength: item.orgLength,
      boardCountRemain: item.boardCountRemain,
      remainBoardInfo: item.remainBoardInfo,
      preMillingSize: item.preMillingSize,
      remainBoardList: [],
      isHelpCut: item.isHelpCut,
      helpCutGap: item.helpCutGap,
      cutKnifeId: item.cutKnifeId,
      helpKnifeId: item.helpKnifeId,
      updateDate: item.updateDate,
    }
    materialList.push(temp)
  })
  // orderList
  let areaDeleted = false
  let newOrderList: any = []
  orderList.forEach((item) => {
    let temp = {
      customerId: item.customerId,
      customer: item.customer,
      phoneNumber: item.phoneNumber,
      orderDate: item.orderDate,
      salemanId: item.salemanId,
      dealer: item.dealer,
      dealerPhoneNumber: item.dealerPhoneNumber,
      address: item.address,
      splitter: item.splitter,
      status: item.status,
      amount: 0,
      remark: item.remark,
      customOrderNo: item.customOrderNo,
      deliveryDate: item.deliveryDate,
      pushConfig: item.pushConfig || false,
      offerListStr: item.offerListStr || null,
      cancelStatus: item.cancelStatus || 0,
      itemList: item.itemList || null,
      goodsList: item.goodsList || null,
      offerList: item.offerList || null,
      totalOrderOfferList: item.totalOrderOfferList || null,
      blockList: item.blockList || null,
      dataBlockList: item.dataBlockList || null,
      objectList: item.objectList || null,
      dataObjectList: item.dataObjectList || null,
      goodsInfoList: item.goodsInfoList || null,
      orderProcessList: item.orderProcessList || null,
      editFun: null,
      saleman: item.salesman,
      orderId: item.id,
      createTime: item.createTime,
      organId: item.organId,
      schduleDeliveryDate: null,
      orderType: item.orderType,
      orderSort: item.orderSort,
      dataType: item.dataType,
      deleted: item.deleted,
      processStatus: null,
    }
    areaDeleted = item.deleted
    newOrderList.push(temp)
  })

  // blockList
  let blockList: any = []

  plateList.forEach((item, i) => {
    const temp = {
      roomName: item.roomName,
      bodyName: item.bodyName,
      roomId: item.roomId || 0,
      bodyId: item.bodyId || 0,
      processGroupld: item.processGroupld,
      orderId: item.orderId,
      blockId: item.plateId,
      goodsId: item.goodsId,
      oldBlockId: item.plateId,
      blockNo: item.plateNo || (i + 1).toString(),
      labelNo: '',
      blockName: item.name,
      width: item.width,
      length: item.height,
      thickness: item.thickness,
      isCurvedGroundLine: false, // item.isCurvedGroundLine, // 是否弧线 不处理异形 异形
      isUnRegular: item.isSpecialShaped,
      isSpecialShaped: item.isSpecialShaped,
      isSculpt: item.isSculpt,
      isRowHole: item.isRowHole,
      sealLeft: item.sealLeft,
      sealRight: item.sealRight,
      sealTop: item.sealUp,
      sealBottom: item.sealDown,
      texture: item.texture,
      placeHole: item.holeFace,
      abnormalOffsetY: item.offsetX,
      abnormalOffsetX: item.offsetY,
      remarkJson: '',
      dataType: 2,
      processGroupName: item.processGroupName,
      type: item.type,
      openDoorType: item.openDoorType,
      extraRemark: {},
      itemId: item.itemId,
      plateRemark: item.remark,
      area: item.area,
      color: item.color,
      hasHole: item.hasHole,
      material: item.material,
      blockMaterial: item.goodsName,
      customPlateNo: item.customPlateNo
    }
    blockList.push(temp)
  })

  let blockDetailList: any = []
  let holeId = 1
  plateModels.forEach((item) => {
    const fitBlock = plateList.find(e => e.id === item.plateId)

    // 孔 （正面/反面）
    let holeDetailArr: HoleDetail[] = []
    if (item.holeDetail) {
      item.holeDetail.forEach((hole) => {
        //  direction 只有侧孔有  angle 生产数据中没有
        const { holeType, faceType, startX, startY, startZ, radius, depth, direction, angle, endX, endY, endZ } = hole
        let holeDetail: HoleDetail = {
          holeId,
          holeType: holeType || 0,
          face: faceType,
          pointX: startX || 0,
          pointY: startY || 0,
          pointZ: startZ || 0,
          radius: radius || 0,
          depth: depth || 0,
          pointX2: endX || 0,
          pointY2: endY || 0,
          pointZ2: endZ || 0,
          endPoint: 0,
          angle: 0,
        }
        holeId = holeId + 1
        holeDetailArr.push(holeDetail)
      })
    }

    // 侧孔
    let sideHoleDetailArr: sideHoleDetail[] = []
    if (item.sideHoleDetail) {
      item.sideHoleDetail.forEach((sideHole) => {
        //  direction 只有侧孔有  angle 生产数据中没有
        const { holeType, faceType, startX, startY, startZ, radius, depth, direction, angle, endX, endY, endZ } = sideHole
        let sideholeDetail: sideHoleDetail = {
          holeId,
          holeType,
          face: faceType || 0,
          pointX: startX || 0,
          pointY: startY || 0,
          pointZ: startZ || 0,
          radius: radius || 0,
          depth: depth || 0,
          pointX2: endX || 0,
          pointY2: endY || 0,
          pointZ2: endZ || 0,
          endPoint: 0,
          angle: 0,
          direction,
        }
        holeId = holeId + 1
        sideHoleDetailArr.push(sideholeDetail)
      })
    }

    let modelDetailArr: contourDetail[] = []
    let sideModelDetailArr: contourDetail[] = []
    if (item.contourDetail) {
      // 垂直造型
      item.contourDetail.forEach((model) => {
        const { modelId, lineID, faceType, knifeName, knifeRadius, depth, originModeling } = model
        const modelDetail: contourDetail = {
          modelId: model.id || ++holeId,
          lineId: lineID,
          face: faceType,
          knifeName: knifeName || '',
          knifeRadius: knifeRadius || 0,
          depth: depth || 0,
          originModeling,
          modelPoint: model.pointList || [],
          modelOffSet: model.offSetList || [],
        }
        modelDetailArr.push(modelDetail)
      })
    }

    if (item.sideDetail) {
      item.sideDetail.forEach((model) => {
        const { modelId, lineId, faceType, knifeName, knifeRadius, depth, originModeling } = model

        const sideModelDetail: any = {
          modelId: modelId || ++holeId,
          lineId,
          face: faceType,
          knifeName: knifeName || '',
          knifeRadius: knifeRadius || 0,
          depth: depth || 0,
          originModeling,
          modelPoint: model.pointList || [],
          modelOffSet: model.offSetList || [],
        }
        sideModelDetailArr.push(sideModelDetail)
      })
    }

    const temp = {
      blockId: item.plateId,
      orderId: item.orderId,
      pointDetail: item.pointDetail || [],
      modelDetail: modelDetailArr || [],
      holeDetail: holeDetailArr || [],
      models: modelDetailArr || [],
      remarkParams: {
        sideRemark: item.sideRemark,
        extraRemark: item.extraRemark,
        specialRemarkDei: item.specialRemarkDei,
        extraHandleRemark: item.extraHandleRemark,
        drillsRemark: item.drillsRemark,
        infoRemark: item.infoRemark,
        remarkJsonDei: item.remarkJsonDei,
        remarkJson: item.remarkJson,
        specialRemark: item.specialRemark,
      },
      offSet: {
        x: item.offsetX,
        y: item.offsetY,
        z: item.offsetZ || 0,
      },
      orgPointDetail: item.rawPointDetail || [],
      cutSize: {
        width: item.splitWidth,
        height: item.splitLength,
      },
      modelListSide: sideModelDetailArr,//[], //  侧槽
      sideHoleDetail: sideHoleDetailArr || [],
      isUnRegular: fitBlock && fitBlock.isSpecialShaped || false, // item.isSpecialShaped || false,
      plateExtraRemark: item.plateExtraRemark,
      has2DModel: item.has2DModel,
      has3DModel: item.has3DModel
    }
    blockDetailList.push(temp)
  })

  let resData = {
    planOrder,
    orderList: newOrderList,
    blockList,
    blockDetailList,
    areaDeleted,
    newGoodsList,
    sealInfo: orderPartsRespVOS && orderPartsRespVOS.filter(e => e.type == '封边条'),
  }
  return resData
}


export function convertDataBack(placeData) {
  console.log('placeData', placeData, beforeConvertData)
  const { materialList, blockList, orderList, planId, planCode } = placeData

  let goodsList = []

  let optimizeRemainPlates: any = []
  let optimizeBoardModelDOS: any = []
  materialList.forEach((material) => {
    if (material.isOptimized === false) {
      console.error('板材没有优化，异常！排查')
    }
    const temp = {
      id: material.id,
      orderId: material.orderId,
      goodsId: material.goodsId,
      goodsName: material.goodsName,
      planId: planCode,
      spec: material.spec,
      material: material.material,
      color: material.color,
      brand: material.brand,
      width: material.width,
      length: material.length,
      thickness: material.thickness,
      cutBorder: material.cutBorder,
      diameter: material.diameter,
      isOptimized: material.isOptimized,
      CutDia: material.CutDia,
      CutGap: material.CutGap,
      boardCount: material.boardCount || 0,
      cutKnifeGap: material.cutKnifeGap,
      minBoardId: material.minBoardId,
      maxBoardId: material.maxBoardId,
      avgUsageRateAll: material.avgUsageRateAll,
      avgUsageRateExcludeLastBoard: material.avgUsageRateExcludeLastBoard,
      usageRateLastBoard: material.usageRateLastBoard,
      usedBoardInfo: material.usedBoardInfo,
      blockPlaceInfo: material.blockPlaceInfo,
      status: material.status,
      hasTexture: material.texture,
      // texture: material.texture,
      orgWidth: material.orgWidth,
      orgLength: material.orgLength,
      boardCountRemain: material.remainBoardCount,
      remainBoardInfo: material.remainBoardInfo,
      preMillingSize: material.preMillingSize,
      remainBoardList: material.remainBoardList,
      isHelpCut: material.isHelpCut,
      helpCutGap: material.helpCutGap,
      cutKnifeId: material.cutKnifeId,
      helpKnifeId: material.helpKnifeId,
      // updateDate: material.updateDate,
    }

    optimizeBoardModelDOS.push(temp)
    if (material.remainBoardList.length != 0) {
      material.remainBoardList.forEach((remainBoard) => {
        let tempRemainBoard: any = {

          goodsName: material.goodsName,
          material: material.material,
          color: material.color,
          width: remainBoard.width,
          length: remainBoard.length,
          count: remainBoard.count,
          thickness: material.thickness,
          remark: remainBoard.remark,
          placeStyle: material.placeStyle,
        }
        // 余料这里的ID  对应的是 源数据中的大板的ID

        let item = beforeConvertData.value.goodsList.find(e => e.goodsId == material.goodsId)

        if (item && item.id) {
          tempRemainBoard = { ...tempRemainBoard, goodsId: item.id }
        }

        if (remainBoard.id && !remainBoard.id.includes('_')) {
          tempRemainBoard = { ...tempRemainBoard, id: remainBoard.id }
        }

        if (beforeConvertData.value.plan && beforeConvertData.value.plan.id) {
          tempRemainBoard = { ...tempRemainBoard, planId: beforeConvertData.value.plan.id }
        }

        if (remainBoard.brand) {
          tempRemainBoard = { ...tempRemainBoard, brand: remainBoard.brand }
        }

        optimizeRemainPlates.push(tempRemainBoard)
      })
    }
  })
  // 订单信息  目测一致
  // let orderList_temp = []
  // orderList.forEach(item => {
  //   let temp = {
  //     customerId: item.customerId,
  //     customer: item.customer,
  //     phoneNumber: item.phoneNumber,
  //     orderDate: item.orderDate,
  //     salemanId: item.salemanId,
  //     dealer: item.dealer,
  //     dealerPhoneNumber: item.dealerPhoneNumber,
  //     address: item.dealerAddress,
  //     status: item.status,
  //     amount: 0,
  //     remark: item.remark,
  //     customOrderNo: item.customOrderNo,
  //     deliveryDate: item.deliveryDate,
  //     pushConfig: item.pushConfig || false,
  //     offerListStr: item.offerListStr || null,
  //     cancelStatus: item.cancelStatus || 0,
  //     itemList: item.itemList || null,
  //     goodsList: item.goodsList || null,
  //     offerList: item.offerList || null,
  //     totalOrderOfferList: item.totalOrderOfferList || null,
  //     blockList: item.blockList || null,
  //     dataBlockList: item.dataBlockList || null,
  //     objectList: item.objectList || null,
  //     dataObjectList: item.dataObjectList || null,
  //     goodsInfoList: item.goodsInfoList || null,
  //     orderProcessList: item.orderProcessList || null,
  //     editFun: null,
  //     saleman: item.salesman,
  //     orderId: item.id,
  //     createTime: item.createTime,
  //     organId: item.organId,
  //     schduleDeliveryDate: null,
  //     orderType: item.orderType,
  //     orderSort: item.orderSort,
  //     dataType: item.dataType,
  //     deleted: item.deleted,
  //     processStatus: null,
  //   }
  // });
  // let plateList = []
  // blockList.forEach((item) => {
  //   const temp = {
  //     roomName: item.roomName,
  //     bodyName: item.bodyName,
  //     processGroupName: item.processGroupName,
  //     roomId: item.roomId || 0,
  //     bodyId: item.bodyId || 0,
  //     processGroupld: item.processGroupld,
  //     orderId: item.orderId,
  //     blockId: item.blockId,
  //     goodsId: item.goodsId,
  //     oldBlockId: item.blockId,
  //     blockNo: item.blockNo,
  //     labelNo: item.labelNo, //  这个为空  标签的
  //     blockName: item.blockName,
  //     width: item.width,
  //     length: item.length,
  //     thickness: item.thickness,
  //     isCurvedGroundLine: item.isCurvedGroundLine, // item.isCurvedGroundLine, // 是否弧线 不处理异形 异形
  //     isSpecialShaped: item.isUnRegular,
  //     sealLeft: item.sealLeft,
  //     sealRight: item.sealRight,
  //     sealUp: item.sealTop,
  //     sealDown: item.sealBottom,
  //     texture: item.texture,
  //     holeFace: item.placeHole,
  //     offsetX: item.abnormalOffsetY,
  //     offsetY: item.abnormalOffsetX,
  //     remarkJson: '',
  //     dataType: item.dataType,
  //     type: item.type,
  //     openDoorType: item.openDoorType,
  //     extraRemark: item.extraRemark,
  //     itemId: item.itemId,
  //   }
  //   plateList.push(temp)
  // })

  // let addPlateIds: any = []
  // let deletePlateIds: any = []
  let resData = {
    optimizeBoardModelDOS,
    // optimizeRemainPlates,
    // goodsList,
    // orderList,
    // plateList,
    // addPlateIds,
    // deletePlateIds,
  }
  return resData
}

// 先保存配置,
/** 保存 原尺寸，开料刀，预铣，辅助开料 */
export function saveSetting(c) {
  let setting: any = {}
  // 尺寸,
  setting.orgWidth = c.orgWidth
  setting.orgLength = c.orgLength
  setting.width = c.width
  setting.length = c.length
  setting.thickness = c.thickness

  setting.preMillingSize = c.preMillingSize
  setting.isHelpCut = c.isHelpCut
  setting.helpCutGap = c.helpCutGap

  setting.diameter = c.diameter
  setting.cutKnifeId = c.cutKnifeId
  setting.cutKnifeName = c.cutKnifeName
  setting.helpKnifeId = c.helpKnifeId
  setting.helpKnifeName = c.helpKnifeName
  c.savedSetting = setting

  // 小板备份资料
  let createTime = Date.now()
  for (let block of c.blockList) {
    let savedObj = block.blockDetail.savedSetting
    if (savedObj && savedObj.createTime == createTime)
      break // 已备份
    blockDetailSaveSetting(block.blockDetail)
    block.blockDetail.savedSetting.createTime = createTime
  }
  return c
}

// 小板备份
function blockDetailSaveSetting(info) {
  info.savedSetting = {}

  info.savedSetting.offsetKnifeRadius = info.offsetKnifeRadius
  info.savedSetting.preMillingSize = info.preMillingSize
  info.savedSetting.isHelpCut = info.isHelpCut
  info.savedSetting.helpCutGap = info.helpCutGap
  // SizeExpand
  info.savedSetting.preMillingExpandSize = info.preMillingExpandSize
  info.savedSetting.modelExpandSize = info.modelExpandSize
  info.savedSetting.vKnifeModelExpandSize = info.vKnifeModelExpandSize
  info.savedSetting.sameKnfieHelpExpandSize = info.sameKnfieHelpExpandSize
  info.savedSetting.currentExpandSize = info.currentSizeExpand
  // 轮廓,铣刀路径
  info.savedSetting.borderContour = clone(info.borderContour)
}

/** 克隆 */
function clone(obj): PlaceBorderContour {
  let newObj = new PlaceBorderContour(obj.placeStyle, obj.borderFinal, obj.borderOrg)
  newObj.borderPreMilling = obj.borderPreMilling
  newObj.polylineOrg = obj.polylineOrg
  newObj.border = obj.border
  newObj.borderSameKnifeHelpCut = obj.borderSameKnifeHelpCut
  newObj.cutLines = obj.cutLines
  newObj.cutLinesSameKnifeHelpCut = obj.cutLinesSameKnifeHelpCut
  newObj.borderMoving = obj.borderMoving
  // newObj.borderKingOptimize = obj.borderKingOptimize;
  newObj.borderModelThrough = obj.borderModelThrough
  newObj.borderModelThroughR = obj.borderModelThroughR
  newObj.cutLinesModelThrough = obj.cutLinesModelThrough
  newObj.borderInnerPlace = obj.borderInnerPlace
  newObj.blockInnerSpace = obj.blockInnerSpace
  newObj.blockOuterSpace = obj.blockOuterSpace
  newObj.spaces = obj.spaces
  newObj.polylines2vModel = obj.polylines2vModel
  newObj.polylinesOutModel = obj.polylinesOutModel
  newObj.placeContours = []
  for (let pc of obj.placeContours) {
    if (pc) {
      let npc = pc.clone()
      newObj.placeContours.push(npc)
    }
  }
  return newObj
}



export function isRect(outline: any): any {
  let pts :any = [] //outline.pts
  let buls :any=[]  // outline.buls
  if(Array.isArray(outline)){
    pts = outline.map(e => e.pts) 
    buls = outline.map(e => e.buls)
  }else{
    pts = outline.pts
    buls = outline.buls
  }
  if (buls.length != 5)
    return null
  if (buls.some(t => t != 0))
    return null

  if (pts.length != 5)
    return null
  if (pts[4].x != pts[0].x || pts[4].y != pts[0].y)
    return null

  return getRect(pts[0], pts[1], pts[2], pts[3])
}
export function isCircle(outline: any): any {
  // const pts = outline.pts
  // const buls = outline.buls
  const pts = outline.map(e => e.pts) //outline.pts
  const buls = outline.map(e => e.buls) // outline.buls


  if (buls.length != 3)
    return null
  if (equal(buls[0], 1) && equal(buls[1], 1) && equal(buls[2], 0)) {
    const p0 = Array.isArray(outline) ? outline[0].pts : outline?.pts[0]
    const p1 = Array.isArray(outline) ? outline[1].pts : outline?.pts[1]
    let x0 = Math.min(p0.x, p1.x)
    let x1 = Math.max(p0.x, p1.x)
    let y0 = Math.min(p0.y, p1.y)
    let y1 = Math.max(p0.y, p1.y)

    let r
    if (equal(x0, x1)) {
      r = (y1 - y0) / 2
      x0 -= r
      x1 += r
    }
    else {
      r = (x1 - x0) / 2
      y0 -= r
      y1 += r
    }
    const t = r / Math.SQRT2
    const dis = r - t
    return { x: x0 + dis, y: y0 + dis, w: t * 2, l: t * 2 }
  }
}


export function getSize(m: BlockModel): number {
  if (m.pointList.some(t => t.curve != 0))
    return -1
  if (m.pointList.length < 5)
    return -1

  let p1 = m.pointList[0]
  let p2 = m.pointList[1]
  let p3 = m.pointList[2]
  let p4 = m.pointList[3]
  let p5 = m.pointList[4]

  let dis12 = getdis(p1, p2)
  let dis23 = getdis(p2, p3)
  let dis_min = Math.min(dis12, dis23)
  let dis_max = Math.max(dis12, dis23)

  if (getdis(p1, p5) > 0.001)
    return -1 // 1.5 不重叠的不是拉槽
  // 矩形 中点到四个点 距离都是一样的
  let pc = { pointX: (p1.pointX + p2.pointX + p3.pointX + p4.pointX) / 4, pointY: (p1.pointY + p2.pointY + p3.pointY + p4.pointY) / 4 }
  let dis1 = getdis(p1, pc)
  let dis2 = getdis(p2, pc)
  if (!equal(dis1, dis2))
    return -1
  let dis3 = getdis(p3, pc)
  if (!equal(dis1, dis3))
    return -1
  let dis4 = getdis(p4, pc)
  if (!equal(dis1, dis4))
    return -1

  return (dis_min + m.knifeRadius * 2) * (dis_max + m.knifeRadius * 2)
}


export function getdis(p1, p2) {
  return Math.sqrt((p1.pointX - p2.pointX) ** 2 + (p1.pointY - p2.pointY) ** 2)
}

export function getMaxChamferR(b: PlaceBlock): number {
  let maxR = 0;
  let daojiaoCount = 0;

  for (let i = 0; i < b.points.length; i++) {
    let p0 = b.points[i];
    if (p0.curve > 0.4143) continue; //不是倒角的,
    if (p0.curve < 0.4142) continue;

    let j = i + 1;
    if (j == b.points.length) j = 0;
    let p1 = b.points[j];

    let r = CADExt.GetRadius(new Point2d(p0.pointX, p0.pointY), new Point2d(p1.pointX, p1.pointY), p0.curve);
    if (r > maxR) maxR = r;
    daojiaoCount++;
  }
  return maxR;
}

/** 获取板件所有缺角 */
export function getInnerCorner(b: PlaceBlock): BlockPoint[][] {
  let startPos = -1;
  let unfilledCorners: BlockPoint[][] = [];   //缺角
  for (let i = 0; i < b.points.length; i++) {
    if (isBorderPoint(b, b.points[i])) {
      startPos = i;
      break;
    }
  }
  if (startPos == -1) return unfilledCorners; //找不到板边缘点

  let insizePs: BlockPoint[] = [];    //缺角轮廓点
  for (let i = startPos; i <= b.points.length + startPos; i++) {
    let p = b.points[i >= b.points.length ? i - b.points.length : i];
    let onBorder = isBorderPoint(b, p);
    if (onBorder) //边缘点
    {
      if (insizePs.length == 0) {
        insizePs.push(p);
      }
      else if (insizePs.length == 1 && isSameBorder(insizePs[0], p)) //已有一个边缘点且同边, 替换它
      {
        insizePs.splice(0, 1, p);
      }
      else {
        insizePs.push(p); // 获得一个缺角轮廓
        unfilledCorners.push(insizePs);
        insizePs = [];
        insizePs.push(p);
      }
    }
    else //板内点
    {
      if (insizePs.length > 0) insizePs.push(p);//
    }
  }
  return unfilledCorners;
}

/** 获取板件所有缺角row */
export function getRawInnerCorner(b): BlockPoint[][] {
  let startPos = -1;
  let unfilledCorners: BlockPoint[][] = [];   //缺角
  for (let i = 0; i < b.rawPointDetail.length; i++) {
    if (isBorderPoint(b, b.rawPointDetail[i])) {
      startPos = i;
      break;
    }
  }
  if (startPos == -1) return unfilledCorners; //找不到板边缘点

  let insizePs: BlockPoint[] = [];    //缺角轮廓点
  for (let i = startPos; i <= b.rawPointDetail.length + startPos; i++) {
    let p = b.rawPointDetail[i >= b.rawPointDetail.length ? i - b.rawPointDetail.length : i];
    let onBorder = isAfrerBorderPoint(b, p);
    if (onBorder) //边缘点
    {
      if (insizePs.length == 0) {
        insizePs.push(p);
      }
      else if (insizePs.length == 1 && isSameBorder(insizePs[0], p)) //已有一个边缘点且同边, 替换它
      {
        insizePs.splice(0, 1, p);
      }
      else {
        insizePs.push(p); // 获得一个缺角轮廓
        unfilledCorners.push(insizePs);
        insizePs = [];
        insizePs.push(p);
      }
    }
    else //板内点
    {
      if (insizePs.length > 0) insizePs.push(p);//
    }
  }
  return unfilledCorners;
}

/**是否边缘点 */
export function isBorderPoint(b: PlaceBlock, p: BlockPoint): boolean {
  if (equal(p.pointX, 0)) return true;
  if (equal(p.pointY, 0)) return true;
  if (equal(p.pointX, b.cutWidth)) return true;
  if (equal(p.pointY, b.cutLength)) return true;
  return false;
}

/**是否边缘点2 */
export function isAfrerBorderPoint(b, p): boolean {
  if (equal(p.pointX, 0)) return true;
  if (equal(p.pointY, 0)) return true;
  if (equal(p.pointX, b.splitWidth)) return true;
  if (equal(p.pointY, b.splitHeight)) return true;
  return false;
}

/**是否同边 */
export function isSameBorder(p1: BlockPoint, p2: BlockPoint): boolean {
  if (equal(p1.pointX, p2.pointX)) return true;
  if (equal(p1.pointY, p2.pointY)) return true;
  return false;
}