
/**配置项 */
export interface configItem {
    /** 配置字段 */
    key: String,
    /** 默认值 */
    defaultValue?: String | Number | Array<any> | boolean
    /** 文字说明 */
    label: String,
    /** 文字的class */
    labelClass?: String
    /** 文字样式 */
    labelStyle?: any
    /** 生效的值 */
    value?: String | Number | Array<any> | boolean
    /** 分组 */
    groupBy: String
    /** 辅助说明 */
    helpMessage?: String
    //** 是否在UI界面显示 */
    isShow?: boolean
    /** 是否在UI界面可编辑 */
    disable?: boolean
    /** 组件类型 标识*/
    component?: string
    /** 附带的其它组件信息 */
    componentList?: configItem[]
    /** 数据选项 */
    options?: any[]
    /** 样式 */
    style?: any
}
/**G代码指令
 * 目前 只需要做 G0 - G3
 */
export enum GCode {
    /**快速定位 空程移动 范例：G00 X100 Y50（刀具快速移动至X100、Y50位置） */
    G0 = 'G0',
    /**快速定位 空程移动 范例：G00 X100 Y50（刀具快速移动至X100、Y50位置）*/
    G00 = 'G00',
    /**直线 直线插补 范例：G01 X200 Y300 F150（刀具以150mm/min的进给速度直线移动至X200、Y300）*/
    G1 = 'G1',
    /**直线 范例：G01 X200 Y300 F150（刀具以150mm/min的进给速度直线移动至X200、Y300）*/
    G01 = 'G01',
    /** 顺时针 范例：G17 G02 X150 Y50 I50 J0 F100（在XY平面，以半径50mm顺时针圆弧插补至X150、Y50）*/
    G2 = 'G2',
    /** 顺时针 范例：G17 G02 X150 Y50 I50 J0 F100（在XY平面，以半径50mm顺时针圆弧插补至X150、Y50）*/
    G02 = 'G02',
    /** 逆时针 范例：G17 G03 X150 Y50 I50 J0 F100（在XY平面，以半径50mm逆时针圆弧插补至X150、Y50）*/
    G3 = 'G3',
    /** 逆时针 范例：G17 G03 X150 Y50 I50 J0 F100（在XY平面，以半径50mm逆时针圆弧插补至X150、Y50）*/
    G03 = 'G03',
    /**暂停 范例：G04 X2（暂停2秒）*/
    G4 = 'G4',
    /**暂停 范例：G04 X2（暂停2秒）*/
    G04 = 'G04',
    /** 通过中间点圆弧插补 
     * 1. 通过中间点圆弧插补
     * 示例：G05 X60 Z50 IX50 IZ60 F120 刀具从当前位置出发，经过中间点 (50, 60)，最终到达终点 (60, 50)，形成圆弧轨迹。
     * 2. 高精轨迹控制（HPCC）模式
     * 功能：G05 可启动高精轨迹控制模式（HPCC），通过曲线拟合和参数优化，实现高精度、高速度的轨迹加工，适用于复杂曲面或高精度零件。
     * 指令格式：
     * 启动：G05 P10000（P值设为10000）
     * 关闭：G05 P0（P值设为0）
     * 3、注意事项：
     * 轴类型限制：建议线性轴参与HPCC模式，旋转轴需谨慎设置。
        与其他功能冲突：
        在HPCC模式下，G61（准确停止检查）、G63（攻螺纹）等功能可能失效，需退出HPCC模式后恢复。
        启用 RTCP（旋转刀具中心点）或 STCP 模式时，禁止同时启用HPCC功能。
        不支持的功能：
        不支持单节停止（C40/M00）、反向手轮模拟、图形模拟等。
        剩余距离显示可能不准确，因显示的是离曲线终点的距离，而非单节终点。
     * */
    G5 = 'G5',
    /** 通过中间点圆弧插补 */
    G05 = 'G05',
    /** 抛物线插补 */
    G6 = 'G6',
    /** 抛物线插补 */
    G06 = 'G06',
    /** z样条曲线插补 */
    G7 = 'G7',
    /** z样条曲线插补 */
    G07 = 'G07',
    /** 进给加速 */
    G8 = 'G8',
    /** 进给加速 */
    G08 = 'G08',
    /** 进给减速 */
    G9 = 'G9',
    /** 进给减速 */
    G09 = 'G09',
    /** 参数写入方式有效 */
    G10 = 'G10',
    /** 参数写入方式取消 */
    G11 = 'G11',
    /** 极坐标变成 */
    G16 = 'G16',
    /** XY平面选择 */
    G17 = 'G17',
    /** XZ平面选择 */
    G18 = 'G18',
    /** YZ平面选择 */
    G19 = 'G19',
    /** 英制输入 */
    G20 = 'G20',
    /** 公制输入 */
    G21 = 'G21',
    /** 半径尺寸编程方式  */
    G22 = 'G22',
    /** 系统操作界面上使用 */
    G220 = 'G220',
    /** 直径尺寸编程方式 */
    G23 = 'G23',
    /** 系统操作界面上使用  */
    G230 = 'G230',
    /** 子程序结束  */
    G24 = 'G24',
    /** 跳转加工  */
    G25 = 'G25',
    /** 循环加工 */
    G26 = 'G26',
    /** 参考点返回 */
    G28 = 'G28',
    /** 倍率注销 */
    G30 = 'G30',
    /** 倍率定义 */
    G31 = 'G31',
    /** 等螺距螺纹切削，英制 */
    G32 = 'G32',
    /** 等螺距螺纹切削，公制 */
    G33 = 'G33',
    /** 增螺距螺纹切削 */
    G34 = 'G34',
    /** 减螺距螺纹切削 */
    G35 = 'G35',
    //** 刀具半径补正取消 */
    G40 = 'G40',
    //** 刀具半径补正 左*/
    G41 = 'G41',
    //** 刀具半径补正 右*/
    G42 = 'G42',
    //** 刀具长度补正+ */
    G43 = 'G43',
    //** 刀具长度补正- */
    G44 = 'G44',
    //** 道具偏置+/+ */
    G45 = 'G45',
    //** 道具偏置+/- */
    G46 = 'G46',
    //** 道具偏置-/- */
    G47 = 'G47',
    //** 道具偏置-/+ */
    G48 = 'G48',
    //** 刀具长度 补正取消 */
    G49 = 'G49',
    //** 局部坐标系设定 */
    G52 = 'G52',
    //** 机床坐标系选择 */
    G53 = 'G53',
    //** 工件坐标系选择1 */
    G54 = 'G54',
    //** 工件坐标系选择2 */
    G55 = 'G55',
    //** 工件坐标系选择3 */
    G56 = 'G56',
    //** 工件坐标系选择4 */
    G57 = 'G57',
    //** 工件坐标系选择5 */
    G58 = 'G58',
    //** 工件坐标系选择6 */
    G59 = 'G59',
    //** 坐标系旋转有效 */
    G68 = 'G68',
    //** 坐标系旋转取消 */
    G69 = 'G69',
    /** 高速深孔钻 */
    G73 = 'G73',
    /** 精搪孔 */
    G76 = 'G76',
    /** 固定循环取消/取消循环指令 */
    G80 = 'G80',
    /**程序停止 */
    M00 = 'M00',
    /**选择性停止 */
    M01 = 'M01',
    /** 程序结束 */
    M02 = 'M02',
    /**主轴正转 */
    M03 = 'M03',
    /**主轴反转 */
    M04 = 'M04',
    /**主轴停止 */
    M05 = 'M05',
    /**自动换刀 */
    M06 = 'M06',
}

/** 内部指令 接收 */
export class CodeAction {
    code?: String
    order?: String
    codeParams?: any
}
/**内部响应类 */
export class GCodeResType {
    code: number = -1
    data: any = null
    msg: string = ''
    logError(msg) {
        console.error(msg)
    }
}

/**G代码基类 */
export class GCodeAction {

    /** 空程直线 */
    G0(params: CodeParams) {
        const { x, y, z, f, xKey, yKey, zKey, fKey, codeKey } = params
        let val: string = GCode.G0
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 空程直线 */
    G00(params: CodeParams) {
        const { x, y, z, f, xKey, yKey, zKey, fKey, codeKey } = params
        let val: string = GCode.G00
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 直线 */
    G1(params: CodeParams) {
        const { x, y, z, f, xKey, yKey, zKey, fKey, codeKey } = params
        let val: string = GCode.G1
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'


        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }



        return val
    }
    /** 直线 */
    G01(params: CodeParams) {
        const { x, y, z, f, xKey, yKey, zKey, fKey, codeKey } = params
        let val: string = GCode.G01
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'


        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 顺时针 弧线 
     * @param x  x坐标
     * @param y  y坐标
     * @param z  z坐标
     * @param i  圆弧 对于起点的偏移量 x
     * @param y  圆弧 对于起点的偏移量 Y
     * @param z  圆弧 对于起点的偏移量 Y
     * @param f  速度
    */
    G2(params: CodeParams) {
        const { x, y, z, i, j, k, r, f, xKey, yKey, zKey, fKey, iKey, jKey, kKey, codeKey } = params
        let val: string = GCode.G2
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        let _iKey = iKey || 'I'
        let _jKey = jKey || 'J'
        let _kKey = kKey || 'K'

        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }
        if (checkVal(r)) {
            val += ` R${r}`
        } else {
            if (checkVal(i)) {
                val += ` ${_iKey}${i}`
            }
            if (checkVal(j)) {
                val += ` ${_jKey}${j}`
            }
            if (checkVal(k)) {
                val += ` ${_kKey}${k}`
            }
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 顺时针 弧线 
     * @param x  x坐标
     * @param y  y坐标
     * @param z  z坐标
     * @param i  圆弧 对于起点的偏移量 x
     * @param y  圆弧 对于起点的偏移量 Y
     * @param z  圆弧 对于起点的偏移量 Y
     * @param f  速度
    */
    G02(params: CodeParams) {
        const { x, y, z, i, j, k, r, f, xKey, yKey, zKey, fKey, iKey, jKey, kKey, codeKey } = params
        let val: string = GCode.G02
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        let _iKey = iKey || 'I'
        let _jKey = jKey || 'J'
        let _kKey = kKey || 'K'

        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }

        if (checkVal(r)) {
            val += ` R${r}`
        } else {
            if (checkVal(i)) {
                val += ` ${_iKey}${i}`
            }
            if (checkVal(j)) {
                val += ` ${_jKey}${j}`
            }
            if (checkVal(k)) {
                val += ` ${_kKey}${k}`
            }
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 逆时针 弧线 
     * @param x  x坐标
     * @param y  y坐标
     * @param z  z坐标
     * @param i  圆弧 对于起点的偏移量 x
     * @param y  圆弧 对于起点的偏移量 Y
     * @param z  圆弧 对于起点的偏移量 Y
     * @param f  速度
    */
    G3(params: CodeParams) {
        const { x, y, z, i, j, k, r, f, xKey, yKey, zKey, fKey, iKey, jKey, kKey, codeKey } = params
        let val: string = GCode.G3
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        let _iKey = iKey || 'I'
        let _jKey = jKey || 'J'
        let _kKey = kKey || 'K'

        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }

        if (checkVal(r)) {
            val += ` R${r}`
        } else {
            if (checkVal(i)) {
                val += ` ${_iKey}${i}`
            }
            if (checkVal(j)) {
                val += ` ${_jKey}${j}`
            }
            if (checkVal(k)) {
                val += ` ${_kKey}${k}`
            }
        }
        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 逆时针 弧线 
     * @param x  x坐标
     * @param y  y坐标
     * @param z  z坐标
     * @param i  圆弧 对于起点的偏移量 x
     * @param y  圆弧 对于起点的偏移量 Y
     * @param z  圆弧 对于起点的偏移量 Y
     * @param f  速度
    */
    G03(params: CodeParams) {
        const { x, y, z, i, j, k, r, f, xKey, yKey, zKey, fKey, iKey, jKey, kKey, codeKey } = params
        let val: string = GCode.G03
        if (typeof (codeKey) == 'string' && codeKey != '') {
            val = codeKey
        }
        let _xkey = xKey || 'X'
        let _yKey = yKey || 'Y'
        let _zKey = zKey || 'Z'
        let _fKey = fKey || 'F'
        let _iKey = iKey || 'I'
        let _jKey = jKey || 'J'
        let _kKey = kKey || 'K'

        if (checkVal(x)) {
            val += ` ${_xkey}${x}`
        }
        if (checkVal(y)) {
            val += ` ${_yKey}${y}`
        }
        if (checkVal(z)) {
            val += ` ${_zKey}${z}`
        }

        if (checkVal(r)) {
            val += ` R${r}`
        } else {
            if (checkVal(i)) {
                val += ` ${_iKey}${i}`
            }
            if (checkVal(j)) {
                val += ` ${_jKey}${j}`
            }
            if (checkVal(k)) {
                val += ` ${_kKey}${k}`
            }
        }

        if (checkVal(f)) {
            val += ` ${_fKey}${f}`
        }
        return val
    }
    /** 主轴正转 
     *  
     * @param s 转速
    */
    M03(s: any) {
        let val: string = GCode.M03
        if (checkVal(s)) {
            val += ` S${s}`
        }
        return val
    }
    /**主轴反向转 
     * @param s 转速
    */
    M04(s: any) {
        let val: string = GCode.M04
        if (checkVal(s)) {
            val += ` S${s}`
        }
        return val
    }

    /**主轴停止 */
    M05() {
        let val: string = GCode.M05

        return val
    }
    /**换刀指令 */
    M06(t: String) {
        let val: string = GCode.M06
        if (checkVal(t)) {
            val += ` T${t}`
        }
        return val
    }

}

/** 行为基类 */
export class processServeBase extends GCodeResType {
    /**G代码基类 */
    _Action = new GCodeAction()
    /** 刀库 */
    knifeList?: Array<Knife | _knifeType>
    /** 当前使用的刀具 */
    usedKnife?: Knife | _knifeType
    constructor() {
        super()
    }
    //** 使用G0代码 移动刀具 */
    KnifeMove(params: CodeParams) {
        let res = this._Action.G0(params)
        return res + '\n'
    }
    /**
     *  根据传入的数据 生成 G0 || G1 || G2 || G3 代码 
     */
    KnifeMove_Arc(params: CodeParams) {
        let res: any = ''

        let dir = params.dir || 0
        if (typeof (dir) == 'string') {
            dir = parseInt(dir)
        }
        switch (dir) {
            case 0:
                res = this._Action.G0(params)
                break;
            case 1:
                res = this._Action.G1(params)
                break;
            case 2:
                res = this._Action.G2(params)
                break;
            case 3:
                res = this._Action.G3(params)
                break;
            default:
                res = ''
                break;
        }

        return res
    }

    /** 换刀指令  --
     * 
     * 因为设计的问题   
     * 现在配置界面会直接配置换刀代码
     *  
     * 
     * 配置数据读取
     * @param str: 刀具名 
     */
    KnifeChange(str: String) {
        let res: any = null
        if (str != '') {
            res = this._Action.M06(str)
        }
        return res
    }
    /**  开始指令 
     * 因为设计的问题
     * 相关配置从传入配置中读取
    */
    startCode() {
        let res: any = null
        /** 文件头 */
        res = 'codeName\n'
        /** 轴启动 */
        res += this._Action.M03(18000)

        /** 刀启动 */
        res = this._Action.M06('T1')

        /** */
        return res
    }
    /**  结束指令 
    * 因为设计的问题
    * 相关配置从传入配置中读取
   */
    endCode() {
        let res: any = null

        /**刀停止 */

        /** 轴停止 */
        res = this._Action.M05()

        return res
    }

}

/** 业务类 */
export class processServe {
    /** 安全距离 */
    safeZ = 30
    /** 注释标识 */
    leaderChar: string = '//'
    serveBase = new processServeBase()
    _GCodeResType = new GCodeResType()
    /**G代码基类 */
    _Action = new GCodeAction()
    /**文件头 */
    boardFileHead = 'G54 G90'
    /** 文件尾 */
    boardFileEnd = 'M30'
    /** 测试用 */
    checkParams(code) {
        let res = new GCodeResType()
        let handle = true
        let temp = {
            /** x坐标 */
            x: false,
            /** y坐标 */
            y: false,
            /** z坐标 */
            z: false,
            /** */
            dir: false,
            /** 圆弧半径 */
            r: false,
            /** 速度  */
            f: false,
            /** IJK 模式的i */
            i: false,
            /** IJK 模式的j */
            j: false,
            /** IJK 模式的k */
            k: false,
        }
        switch (code) {
            case 'G0':
                temp = { ...temp, x: true, y: true, z: true, f: true }
                break;
            case 'G00':
                temp = { ...temp, x: true, y: true, z: true, f: true }
                break;
            case 'G1':
                temp = { ...temp, x: true, y: true, z: true, f: true }
                break;
            case 'G01':
                temp = { ...temp, x: true, y: true, z: true, f: true }
                break;
            case 'G2':
                temp = { ...temp, x: true, y: true, z: true, i: true, j: true, k: true, dir: true, r: true, f: true }
                break;
            case 'G02':
                temp = { ...temp, x: true, y: true, z: true, i: true, j: true, k: true, dir: true, r: true, f: true }
                break;
            case 'G3':
                temp = { ...temp, x: true, y: true, z: true, i: true, j: true, k: true, dir: true, r: true, f: true }
                break;
            case 'G03':
                temp = { ...temp, x: true, y: true, z: true, i: true, j: true, k: true, dir: true, r: true, f: true }
                break;
            default:
                handle = false
                break;
        }

        if (handle == false) {
            res.code = -1
            res.data = null
            res.msg = `${code}命令开发中`
        } else {
            res.code = 100
            res.data = temp
            res.msg = 'sucess'
        }
        return res
    }

    //** 生成单个加工项的代码 -- 测试用 */
    doProcessItem(processItem: CodeParamsObj) {
        let code = ''
        // 文件头
        code += this.boardFileHead + '\n'
        if (processItem.list.length > 0) {
            if (processItem.processItemInfo?.knife) {
                code += this.startCode(processItem.processItemInfo?.knife) + '\n'
            }

            code += this.addAnnotation('process Remark') + '\n'
            for (const item of processItem.list) {
                code += `${this.serveBase.KnifeMove_Arc(item)} \n`
            }
            if (processItem.processItemInfo?.knife) {
                code += this.endCode(processItem.processItemInfo?.knife) + '\n'
            }
        } else {
            this._GCodeResType.logError('doProcessItem fail,no processItem Data')
        }
        // 文件尾
        code += this.boardFileEnd + '\n'
        return code
    }
    /** 生成多个加工项代码 */
    doProcessList(data: CodeParamsList) {
        let code = ''

        if (data.processList.length > 0) {
            for (const item of data.processList) {
                code += `${this.doProcessItem(item)} \n`
            }
        } else {
            this._GCodeResType.logError('doProcessList fail,no processItem Data')
        }
        return code
    }
    /** 添加注释 */
    addAnnotation(str) {
        let res = `${this.leaderChar} ${str}`
        return res
    }


    startCode(knife: _knifeType) {
        let res: any = ''

        /** 轴启动 */
        res += knife.axisStartCode + '\n'
        /** 刀启动 */
        res = knife.knifeStartCode

        /** */
        return res
    }
    /**  结束指令 
    * 因为设计的问题
    * 相关配置从传入配置中读取
   */
    endCode(knife: _knifeType) {
        let res: any = ''
        /**刀停止 */
        res += knife.knifeStopCode + '\n'
        /** 轴停止 */
        res += knife.axisStopCode

        return res
    }
}

function checkVal(val) {
    let r = true
    if ((val == undefined || val == '')) {
        r = false
    }
    return r
}



/** 加工项的类别
 * 用于区分加工项的类别，不同的类别  在文件生成的时候  可能需要对应某些独立的节点
 */
export enum processItemType {
    /**排钻 */
    Hole = 'hole',
    /** 铣孔 */
    MillingHole = 'millingHole',
    /** 造型 非槽加工 铣型 */
    Model = 'model',
    /** 造型 槽-- 拉槽 */
    Grooves = 'grooves',
    /** 造型 铣槽  */
    MillingGrooves = 'millingGrooves',
    /** 侧面造型 */
    SideModel = 'SideModel',
    /** 侧面槽 - 拉槽 */
    SideGrooves = 'sideGrooves',
    /** 侧面槽 - 拉槽 */
    SideMillingGrooves = 'SideMillingGrooves',
    /** 侧面孔 排钻 */
    SideHole = 'sideHole',
    /** 侧面孔 铣孔 */
    MillingSideHole = 'MillingSideHole',
    /** 开料 */
    CutBlock = 'cutBlock',
    /** 修边 */
    MillingBlockBoard = 'MillingBlockBoard'
}


// 加工项 点数据 
export class CodeParams {
    /** x坐标 */
    x?: Number | String
    /** y坐标 */
    y?: Number | String
    /** z坐标 */
    z?: Number | String
    /** 调用的代码编号 */
    dir?: Number | String
    /** 圆弧半径 */
    r?: Number | String
    /** 速度  */
    f?: Number | String
    /** IJK 模式的i */
    i?: Number | String
    /** IJK 模式的j */
    j?: Number | String
    /** IJK 模式的k */
    k?: Number | String

    /** 代码标识 */
    codeKey?: String
    /** x坐标 */
    xKey?: String
    /** y坐标 */
    yKey?: String
    /** z坐标 */
    zKey?: String
    /** 圆弧半径 */
    rKey?: String
    /** 速度  */
    fKey?: String
    /** IJK 模式的i */
    iKey?: String
    /** IJK 模式的j */
    jKey?: String
    /** IJK 模式的k */
    kKey?: String
}


/** 加工项对应的信息 */
export class ProcessInfo extends CodeAction {
    /**当前加工项的下标*/
    i?: Number
    /** 加工项 对应刀具的数据 */
    knife?: _knifeType
    /** 加工项的类型 */
    type?: processItemType
    /** 加工项所在的 文件名 */
    belong?: string
    /** 该加工项基于哪个加工面  传入数据 主要用于 加工项坐标转换   感觉可能没用 */
    belongFace?: FaceType
    /** 板件信息 */
    block?: any
    // /** 垂直基准点 */
    // originZ0Position?: OriginZPosition
    // /** 水平基准点 */
    // originPointPosition?: BoardPosition
    // /** 大板定位 */
    // boardLocation?: BoardPosition
    // /** 加工项的宽方向 x */
    // widthSideAxis?: AxisType
    // /** 加工项的长方向 y */
    // lengthSideAxis ?: AxisType
    // /** 加工项的高方向 */
    // heightSideAxis ?: AxisType

}


/**导出文件的 业务信息 */
export class ProcessFileInfo extends CodeAction {
    // 订单数据
    order?: any = []
    /** 排单信息 */
    planOrderInfo?: any = {}
    // 商品信息
    materialInfo?: any[]
    /** 所在板件信息 */
    block?: any[]
    /** 压缩包 文件名  */
    zipFileName?: any
    /** 文件名数据集 */
    fileNameList?: string[]
    /** 文件名 */
    fileName?: string | FileNameType
    /**  */
}

export class FileNameType {
    name: String = ''
    Suffix: String = ''
}

/**加工项数据 */
export class CodeParamsObj {
    //**加工项 点阵  */     
    list: CodeParams[] = []
    /** 加工项 的其他附带信息 */
    processItemInfo?: ProcessInfo = {}
}

/** 代码指令数据集 加工项数据集 */
export class CodeParamsList {
    processListInfo?: ProcessFileInfo
    processList: CodeParamsObj[] = []
}


export class _knifeType {
    [key: string]: any
    /** 开料刀ID号 */
    knifeId?: number | string
    /** 轴号 */
    axleId?: number | string
    /** 是否启用 */
    isEnabled?: boolean = true;

    /** 刀直径mm */
    diameter?: number
    /** 刀长(最大深度) */
    length?: number
    /**进给速度, 0取系统默认 */
    speed?: number

    /** 偏置短边偏移(X轴)-相对于工件原点位置XY坐标的短边方向偏移量 */
    offsetX?: number
    /** 偏置长边偏移(Y轴)-相对于工件原点位置XY坐标的长边方向偏移量*/
    offsetY?: number
    /** 偏置垂直偏移(Z轴)-相对于工件原点位置XY坐标的垂直方向偏移量*/
    offsetZ?: number
    /** 基准坐标-短边(X轴)-钻头在钻包中短边方向(纵向)的相对位置 */
    baseX?: number
    /** 基准坐标-长边(Y轴)-钻头在钻包中长边方向(横向)的相对位置 */
    baseY?: number
    /** 轴启动指令后置 */
    isAxisStartCodePostpost?: boolean = false;
    /** 轴启动指令前置 */
    isAxisStartCodePrepost?: boolean = false;


    /** 刀名称 */
    knifeName?: String
    /** 刀类型 */
    knifeType?: Number | String
    /** 功能属性(数组): 1开料/切割 2拉槽 3铣型 4铣孔 5钻孔 6拉米诺 7乐扣 8T型 */
    ability?: any = []
    /** 支持的加工面 */
    face?: FaceType = 0
    /**刀启动指令 */
    knifeStartCode?: string = ''
    /**刀停止指令 */
    knifeStopCode?: string = ''
    /**轴开始指令 */
    axisStartCode?: string = ''
    /** 轴停止指令 */
    axisStopCode?: string = ''
}


/** 让对象能够支持动态属性 */
export class _type {
    [key: string]: any;
}

/**
 * 板面类型
 * 0正面 1反面 2侧面 21左侧面 22右侧面 23上侧面 24下侧面 29 弧形侧面 30异形侧面
 */
export enum FaceType {
    /** 正面 */
    FRONT = 0,
    /** 反面 */
    BACK = 1,
    /** 侧面 */
    SIDE = 2,
    /** 左侧面 */
    LEFT_SIDE = 21,
    /** 右侧面 */
    RIGHT_SIDE = 22,
    /** 上侧面 */
    TOP_SIDE = 23,
    /** 下侧面 */
    BOTTOM_SIDE = 24,
    /** 弧形侧面 */
    CURVED_SIDE = 29,
    /** 异形侧面 */
    SPECIAL_SHAPED_SIDE = 30,
}

/** 开料刀 */
export class Knife {
    /** 是否启用 */
    isEnabled = true;
    /** 轴号 */
    axleId = 0;
    /** 开料刀ID号 */
    knifeId = 1;
    /** 加工面(0正面 1反面 2左侧面 3右侧面 4上侧面 5下侧面 6任意) */
    processFace = 0;
    /** 刀名称 */
    knifeName = '';
    /** 刀具类型(1铣刀 2成型刀 3钻头 4锯 5刀片) */
    knifeType = KnifeType.MILLING_CUTTER;
    /** 功能属性(数组): 1开料/切割 2拉槽 3铣型 4铣孔 5钻孔 6拉米诺 7乐扣 8T型 */
    ability: any = [];

    // /** 默认开料刀 */
    // isDefaultCutKnife = false;
    // /** 是否可用于开料切割 */
    // isCuttingKnife = false;
    // /** 是否允许铣孔(knifeType为铣刀生效) */
    // isMillingAllowed = false;
    /** 刀直径mm */
    diameter = 6;
    /** 刀长(最大深度) */
    length = 20;
    /** 锯齿厚度，单次加工切缝宽度 */
    sawThiness = 0;
    /**锯向: 0横向（或长边） 1纵向（短边） 2自由角度*/
    sawDirection = 0;
    /**切向加工方向: 0横向左往右 2横向右往左 3纵向上往下 4纵向下往上 5随意 */
    processDirection = 0;
    /**进给速度, 0取系统默认 */
    speed = 0;
    /** 进给深度 */
    stepDepth = 0;
    // /** 组号 */
    // groupNo = 0;
    // /** 主刀 */
    // isMainKnife = false;
    // /** 是否高级加工 */
    // isAdvancedProcessEnabled = false;
    // /** 是否集合加工 */
    // isBatchProcessEnabled = false;
    // /** 副刀偏置长边偏移(Y轴) */
    // auxKnifeOffsetY = 0;
    // /** 副刀偏置短边偏移(X轴) */
    // auxKnifeOffsetX = 0;
    /** 偏置短边偏移(X轴)-相对于工件原点位置XY坐标的短边方向偏移量 */
    offsetX = 0;
    /** 偏置长边偏移(Y轴) -相对于工件原点位置XY坐标的长边方向偏移量*/
    offsetY = 0;
    /** 偏置垂直偏移(Z轴) -相对于工件原点位置XY坐标的垂直方向偏移量*/
    offsetZ = 0;
    /** 基准坐标-短边(X轴) -钻头在钻包中短边方向(纵向)的相对位置 */
    baseX = 0;
    /** 基准坐标-长边(Y轴) -钻头在钻包中长边方向(横向)的相对位置 */
    baseY = 0;
    /**组合钻 */
    isModularDrill = false;
    /** 是否预启动 */
    isPreStartEnabled = false;
    /** 预启动提前动作数 */
    preStartAheadActionCount = 5;
    /** 预启动延迟换刀 */
    isPreStartToolChangeDelay = false;
    /** 预启动延迟换刀指令 */
    preStartToolChangeDelayCode = '';
    /** 轴启动指令后置 */
    isAxisStartCodePostpost = false;
    /** 轴停止指令前置 */
    isAxisStopCodePrepose = false;
    /** 钻组独立指令（启用后，刀起始指令、刀结束指令使用钻组起始指令、钻组结束指令替换） */
    drillGroupCode = '';

    /** 轴启动代码*/
    axisStartCode = "";
    /** 刀启动代码 */
    knifeStartCode = '';
    /** 钻组起始指令 */
    drillGroupStartCode = '';
    /** 钻组结束指令 */
    drillGroupEndCode = '';
    /** 刀停止代码 */
    knifeStopCode = '';
    /** 轴停止指令 */
    axisStopCode = '';
    // /** 高级加工指令 */
    // advancedCode = '';

    /** 开料刀 */
    get isCuttingKnife() {
        return this.isEnabled && this.ability.includes(AbilityType.CUT);
    }

    /** 造型刀 */
    get isModelingKnife() {
        return this.isEnabled && this.ability.includes(AbilityType.MILLING_MODEL);
    }

    /** 钻刀 */
    get isDrillingKnife() {
        return this.isEnabled && this.ability.includes(AbilityType.DRILL_HOLE);
    }

    /** 铣孔 */
    get isCutting4HoleKnife() {
        return this.isEnabled && this.ability.includes(AbilityType.MILLING_HOLE);
    }

    constructor(data: any = null) {
        if (data != null) {
            for (const key in data) {
                try {
                    this[key] = data[key]
                    // if (Reflect.has(this, key)) {
                    //   this[key] = data[key]
                    // }
                } catch (error) {
                    console.log('load knife parameter error', key, error)
                }
            }
        }
    }

    /** 设置刀具(轴号, 刀名, 刀直径, 是否主刀, 是否铣孔, 是否开料刀) */
    // set(axleId: number, name: string, diameter: number, isMainKnife: boolean, isMillingAllowed: boolean, isCuttingKnife: boolean)
    /** 设置刀具(轴号, 刀名, 刀类型, 刀直径, 刀长, 是否默认刀) */
    set(axleId: number, name: string, knifeType: KnifeType, diameter: number, length: number, isEnabled: boolean) {
        this.axleId = axleId;
        this.knifeName = name;
        this.knifeType = knifeType;
        this.diameter = diameter;
        this.length = length;
        this.isEnabled = isEnabled;
    }
}


/** 刀类型: MILLING_CUTTER铣刀 FORMING_CUTTER成型刀 DRILL钻头 SAW锯 BLADE刀片 */
export enum KnifeType {
    /** 铣刀 */
    MILLING_CUTTER = 1,
    /** 成型刀 */
    FORMING_CUTTER = 2,
    /** 钻头 */
    DRILL = 3,
    /** 锯 */
    SAW = 4,
    /** 刀片 */
    BLADE = 5
}

/** 刀功能: 
 ** 1-CUT开料/切割 2-PULLING_GROOVE拉槽 3-MILLING_MODEL铣型 4-MILLING_HOLE铣孔 
 ** 5-DRILL_HOLE钻孔 6-RAMINO拉米诺 7-EASY_FASTEN乐扣 8-T_TYPE T型 */
export enum AbilityType {
    /** 1开料/切割 */
    CUT = 1,
    /** 2拉槽 */
    PULLING_GROOVE = 2,
    /** 3铣型 */
    MILLING_MODEL = 3,
    /** 4铣孔 */
    MILLING_HOLE = 4,
    /** 5钻孔 */
    DRILL_HOLE = 5,
    /** 6拉米诺 */
    RAMINO = 6,
    /** 7乐扣 */
    EASY_FASTEN = 7,
    /** 8T型 */
    T_TYPE = 8
}

/** 枚举 坐标轴类型 */
export enum AxisType {
    /** X轴正 */
    X_POS = 0,
    /** X轴负 */
    X_NEG = 1,
    /** Y轴正 */
    Y_POS = 2,
    /** Y轴负 */
    Y_NEG = 3,
    /** 向上Z轴正 */
    Z_POS = 4,
    /** 向下Z轴负 */
    Z_NEG = 5,
}

/** 枚举 坐标轴类型 */
export enum OriginZPosition {
    /** 台面向上Z轴正 */
    WorkTop = 0,
    /** 板面向上Z轴正 */
    BoardFace = 1,
}

// nc文件编码
export const ncEncodeMap = {
    'UTF-8': 'UTF-8',
    'GB2312': 'GB2312',
    'UTF-8-BOM': 'UTF-8-BOM',
    'GBK': 'GBK',
    'Big-5': 'Big-5',
    'GB18030': 'GB18030',
    'Unicode': 'Unicode'
}

// 小板文件名选项
export const smallPlateOptionsMap = {
    板宽: '{0}',
    板长: '{1}',
    颜色: '{2}',
    材质: '{3}',
    大板号: '{4}',
    工位号: '{5}',
    板厚: '{6}',
    开料顺序: '{7}',
    批次识别: '{8}',
    小板编号: '{9}',
    全局大板号: '{10}',
    品牌: '{11}',
}

// zip压缩包文件名选项
export const zipFileOptionsMap = {
    '板宽': '{0}',
    '板长': '{1}',
    '颜色': '{2}',
    '材质': '{3}',
    '大板号': '{4}',
    '工位号': '{5}',
    '板厚': '{6}',
    '开料顺序': '{7}',
    '批次识别': '{8}',
    '小板编号': '{9}',
    '全局大板号': '{10}',
    '品牌': '11',
}

/** 枚举 大板边角位置 */
export enum BoardPosition {
    /** 左上角 */
    // LEFT_TOP = 0,
    // /** 左下角 */
    // LEFT_BOTTOM = 1,
    // /** 右下角 */
    // RIGHT_BOTTOM = 2,
    // /** 右上角 */
    // RIGHT_TOP = 3,
    /** 左上角 */
    LEFT_TOP = 3,
    /** 左下角 */
    LEFT_BOTTOM = 0,
    /** 右下角 */
    RIGHT_BOTTOM = 1,
    /** 右上角 */
    RIGHT_TOP = 2,
    /**
     *     左下角 = 0,
      右下角 = 1,
      右上角 = 2,
      左上角 = 3
     */
}

/** 排版样式 */
export enum PlaceStyle {
    /** 正面 */
    FRONT = 0,
    /** 正面右转 */
    FRONT_TURN_RIGHT = 1,
    /** 正面后转 */
    FRONT_TURN_BACK = 2,
    /** 正面左转 */
    FRONT_TURN_LEFT = 3,
    /** 反面 */
    BACK = 4,
    /** 反面右转 */
    BACK_TURN_RIGHT = 5,
    /** 反面后转 */
    BACK_TURN_BACK = 6,
    /** 反面左转 */
    BACK_TURN_LEFT = 7,
}

/** 小板边的位置类型 */
export enum EdgeType
  {
  /** 下=0 */
  BOTTOM = 0,
  /** 右=1 */
  RIGHT = 1,
  /** 上=2 */
  TOP = 2,
  /** 左=3 */
  LEFT = 3,
}
