import { Knife } from "./confClass"  
import { FileZip } from "./common/base/ZipFile"  
import { _knifeType, _type, CodeParamsList } from "./confClass"
import { device1 } from "./processConfig/device1/parser1"
import { device2 } from "./processConfig/device2/parser2"

export enum CodeType {
    文件头 = 'FSTART',
    调刀 = 'T',
    主程序 = 'C',
    主程序结束 = 'CE',
    文件尾 = 'FEND',
    备注 = 'MARK',
    FileStart = 'FSTART',
    UseKnife = 'T',
    MainCode = 'C',
    MainCodeEnd = 'CE',
    FileEnd = 'FEND',
    REMARK = 'MARK',
}

export class ParserMain {
    /**解析器  解析器容器 */
    _device
    deviceList = [
        {
            label: '解析器1',
            value: 'device1',
            groupLabel: '通用',
            groupKey: 'parser1',
            device: device1
        },
        {
            label: '解析器2',
            value: 'device2',
            groupKey: 'parser1',
            groupLabel: '通用',
            device: device2
        }
    ]
    zipFile: FileZip
    processListData?: CodeParamsList

    constructor(device?) {
        if (device) {
            this._device = device
        }
        this.zipFile = new FileZip()
    }
    //** 文件头 */
    FSTART(val) {
        let str = this.handleCode('FSTART', val)
        return str
    }
    /** 换刀 */
    T(val, params?) {
        let str = this.handleCode('T', val, params)
        return str
    }
    /** 主程序 加工数据 */
    C(val, params) {
        let str = this.handleCode('C', val, params)
        return str
    }
    /** 主程序结束 */
    CE(val) {
        let str = this.handleCode('CE', val)
        return str
    }
    /** 文件结束 */
    FEND(val) {
        let str = this.handleCode('FEND', val)
        return str
    }
    /** 备注 */
    MARK(val, params) {
        let str = this.handleCode('MARK', val, params)
        return str
    }
    //  { 
    //     "value": "1",
    //     "label": "铣刀",
    // },
    // {
    //     "value": "2",
    //     "label": "成型刀",
    // },
    // {
    //     "value": "3",
    //     "label": "钻头"
    // },
    // {
    //     "dictType": "work_area_knif_type",
    //     "value": "4",
    //     "label": "锯片",
    // }
    getAbilityOptions(i) {
        let knife = this._device.knifeList[i]
        const knifeTypeOptions = this.getKnifeTypeOptions() || []
        let label = knifeTypeOptions.find(e => e.value == knife.knifeType).label || ''
        const allowedAbilities = knifeTypeAbilityMap[label] || []
        const filteredAbilities = orginAbilitys.filter(ability => allowedAbilities.includes(ability.label)) || []
        return filteredAbilities
    }
    setDefaultKnife(i) {
        console.log('setDefaultKnife', i);
        this._device.knifeList.forEach((knife, index) => {
            if (i == index) {
                knife.isDefaultCutKnife = true
            } else {
                knife.isDefaultCutKnife = false
            }
        });
    }
    getKnifeTypeOptions() {
        return knifeTypeOptions
    }
    /** 设置使用的解析器 */
    setDevice(device) {
        this._device = device
    }
    /** 根据解析器名称 设置解析器 */
    setDeviceByName(deviceName) {
        let deviceOption = this.deviceList.find(e => e.label == deviceName)
        if (deviceOption) {
            let device = deviceOption.device
            this.setDevice(new device())
        }
    }
    /** 根据解析器key 设置解析器 */
    setDeviceByKey(deviceKey) {
        let deviceOption = this.deviceList.find(e => e.value == deviceKey)
        if (deviceOption) {
            let device = deviceOption.device
            this.setDevice(new device())
        }
    }

    /** 处理指令代码 */
    handleCode(codeType, codeKey, params?) {
        let str = ''
        if (typeof (this._device[codeType][codeKey]) == 'function') {
            str += this._device[codeType][codeKey](params)
        } else {
            str += this._device[codeType][codeKey]
        }
        if (!codeType) {
            str = '// 未输入代码，请重新输入\n'
        } else if (!codeKey) {
            str = '// 未输入指令，请重新输入\n'
            let k = Object.keys(this._device[codeType])
            str += `// 代码【${codeType}】支持的指令为：${k.join(',')}`
        } else if (str == 'undefined') {
            str = `// 代码【${codeType}】指令【${codeKey}】解析失败，请检查代码指令配置\n`
        }
        // console.log('handleCode parserMain', str)
        /** 空行处理 */
        if (this._device.isFilterEmptyLine == true) {
            str = this._device.handleEmptyLine(str)
        }

        // 空行前插入
        if (this._device.isNcLinePrefixEnabled == true) {
            str = this._device.handleEmptyLinePreFix(str)
        }
        // 空行后插入
        if (this._device.isNcLineSuffixEnabled == true) {
            str = this._device.handleEmptyLineSuffix(str)
        }

        if (codeType == 'FEND') {
            this._device.lastCodeParams = undefined
        }
        return str
    }

    /**
     * 加载 解析器数据
     * 
    */
    loadProcessListData(data: CodeParamsList) {
        this.processListData = data
    }

    checkProcessDataPosition(data: CodeParamsList): CodeParamsList {
        let newData = { ...data }

        // 根据解析器的放置方式 转换数据
        for (let processList of newData.processList) {
            if (Array.isArray(processList.list) && processList.list.length > 0) {
                let newList = this._device.getRealPoints(processList.list)
                processList.list = newList
            }
        }
        return newData
    }

    /**根据 解析器收到的数据  生成 加工代码 */
    getCode(data: CodeParamsList) {
        let str = '';
        if (!data || !Array.isArray(data?.processList)) {
          console.error('无效数据: data.processList 必须是数组');
          return str;
        }
      
        for (const item of data.processList) {
          if (!item?.processItemInfo) continue;
          const { code, order, codeParams } = item.processItemInfo;
          str += this.handleCode(code, order, code === 'C' ? item : codeParams);
        }
      
        if (this._device?.isUseSimpleCode) {
          str = this._device.handleSimpleCode(str);
        }
        return str;
      }
    /** 导出文件 */
    async exportFile(data: CodeParamsList) {
        let zipFileName = data.processListInfo?.zipFileName || 'test.zip'
        this.zipFile.SetZipFileName(zipFileName)

        let fileNameInfo = data.processListInfo?.fileName
        let fileName = ''
        if (typeof (fileNameInfo) == 'string') {
            fileName = fileNameInfo
        } else {
            fileName = `${fileNameInfo?.name || 'empty'}.${fileNameInfo?.Suffix || '.txt'}`
        }
        let str = '';
        for (const item of data.processList) {
            if (!item.processItemInfo) {
                continue
            }
            let code = item.processItemInfo.code
            let order = item.processItemInfo.order
            let params = item.processItemInfo.codeParams

            if (code == 'C') {
                params = item
            }
            str += await this.handleCode(code, order, params)
        }

        await this.zipFile.PushFile(fileName, str)


        await this.zipFile.Download()
        //
    }


    setZipFileName(fileName: string) {
        this.zipFile.SetZipFileName(fileName)
    }
    /** 设置刀库 */
    setKnifeList(knifeList: Array<Knife | _knifeType>) {
        this._device.knifeList = knifeList
    }
    /** 获取解析器列表 */
    getDeviceList() {

        return this.deviceList
    }
}


/** demo  刀具数据 */
export const knifeData = {
    "isEnabled": true,
    "axleId": 2,
    "knifeId": 2,
    "processFace": "",
    "knifeName": "T1",
    "knifeType": 3,
    "ability": [
        5
    ],
    "diameter": 5,
    "length": 20,
    "sawThiness": 7,
    "sawDirection": 2,
    "processDirection": 4,
    "speed": 0,
    "stepDepth": 0,
    "offsetX": 0,
    "offsetY": 0,
    "offsetZ": 0,
    "baseX": 0,
    "baseY": 0,
    "isModularDrill": false,
    "isPreStartEnabled": false,
    "preStartAheadActionCount": 5,
    "isPreStartToolChangeDelay": false,
    "preStartToolChangeDelayCode": "",
    "isAxisStartCodePostpost": false,
    "isAxisStopCodePrepose": false,
    "drillGroupCode": "",
    "axisStartCode": "M03 S18000\n",
    "knifeStartCode": `M06 T1\nG43 H1\n`,
    "drillGroupStartCode": "T1",
    "drillGroupEndCode": "",
    "knifeStopCode": "",
    "axisStopCode": "M05\n",
    "preStartActionDeferCode": "",
    "useHolesGroupKnife": false,
    "preStartActionStepsLimit": "",
    "knifeNo": "",
    "editable": true,
    "isDefaultCutKnife": false,
    "isPreStartChangeKnifeDefer": false
}
export const knifeData1 = {
    "isEnabled": true,
    "axleId": 2,
    "knifeId": 2,
    "processFace": "",
    "knifeName": "T2",
    "knifeType": 3,
    "ability": [
        5
    ],
    "diameter": 6,
    "length": 20,
    "sawThiness": 7,
    "sawDirection": 2,
    "processDirection": 4,
    "speed": 0,
    "stepDepth": 0,
    "offsetX": 0,
    "offsetY": 0,
    "offsetZ": 0,
    "baseX": 0,
    "baseY": 0,
    "isModularDrill": false,
    "isPreStartEnabled": false,
    "preStartAheadActionCount": 5,
    "isPreStartToolChangeDelay": false,
    "preStartToolChangeDelayCode": "",
    "isAxisStartCodePostpost": false,
    "isAxisStopCodePrepose": false,
    "drillGroupCode": "",
    "axisStartCode": "M03 S18000\n",
    "knifeStartCode": `M06 T2\nG43 H2\n`,
    "drillGroupStartCode": "T2",
    "drillGroupEndCode": "",
    "knifeStopCode": "",
    "axisStopCode": "M05\n",
    "preStartActionDeferCode": "",
    "useHolesGroupKnife": false,
    "preStartActionStepsLimit": "",
    "knifeNo": "",
    "editable": true,
    "isDefaultCutKnife": false,
    "isPreStartChangeKnifeDefer": false
}
export const knifeData2 = {
    "isEnabled": true,
    "axleId": 2,
    "knifeId": 2,
    "processFace": "",
    "knifeName": "T3",
    "knifeType": 3,
    "ability": [
        5
    ],
    "diameter": 6,
    "length": 20,
    "sawThiness": 7,
    "sawDirection": 2,
    "processDirection": 4,
    "speed": 0,
    "stepDepth": 0,
    "offsetX": 0,
    "offsetY": 0,
    "offsetZ": 0,
    "baseX": 0,
    "baseY": 0,
    "isModularDrill": false,
    "isPreStartEnabled": false,
    "preStartAheadActionCount": 5,
    "isPreStartToolChangeDelay": false,
    "preStartToolChangeDelayCode": "",
    "isAxisStartCodePostpost": false,
    "isAxisStopCodePrepose": false,
    "drillGroupCode": "",

    "axisStartCode": "M03 S18000\n",
    "knifeStartCode": `M06 T2\nG43 H2\n`,
    "drillGroupStartCode": "T3",
    "drillGroupEndCode": "",
    "knifeStopCode": "",
    "axisStopCode": "M05\n",

    "preStartActionDeferCode": "",
    "useHolesGroupKnife": false,
    "preStartActionStepsLimit": "",
    "knifeNo": "",
    "editable": true,
    "isDefaultCutKnife": false,
    "isPreStartChangeKnifeDefer": false
}

export const knifeTypeOptions = [
    {
        "value": "1",
        "label": "铣刀",
    },
    {
        "value": "2",
        "label": "成型刀",
    },
    {
        "value": "3",
        "label": "钻头",
    },
    {
        "value": "4",
        "label": "锯片",
    },
]

export const orginAbilitys = [
    {
        "value": "1",
        "label": "切割",
    },
    {
        "value": "2",
        "label": "拉槽",
    },
    {
        "value": "3",
        "label": "铣型",
    },
    {
        "value": "4",
        "label": "铣孔",
    },
    {
        "value": "5",
        "label": "钻孔",
    },
    {
        "value": "6",
        "label": "拉米诺",
    },
    {
        "value": "7",
        "label": "乐扣/U形件",
    },
    {
        "value": "8",
        "label": "T形/燕尾槽刀",
    },
]

export const knifeTypeAbilityMap = {
    钻头: ['钻孔'], // 3
    铣刀: ['切割', '拉槽', '铣型', '铣孔', '钻孔'], // 1
    成型刀: ['铣型', '拉米诺', '乐扣/U形件', 'T形/燕尾槽刀'], // 2
    锯片: ['拉槽', '拉米诺'],//4 '切割',下个版本支持 
  }