/** 解析器1 */
import { AxisType } from "../../confClass";
import { _knifeType, _type, CodeParams, CodeParamsList, CodeParamsObj, configItem, GCodeAction, ProcessInfo,OriginZPosition } from "../../confClass";
import { BoardPosition } from "../../confClass";
import { FileZip } from "../../common/base/ZipFile";
import { confList } from "./parser1Data";
import { knifeData, knifeData1, knifeData2 } from "../../parseMain";

/**
 * 备注：
 * 文件导出模式（大板模式 || 小板模式）---作用于 加工项数据  和加工项的点阵数据
 * 应该在数据处理 生效
 * 解析器 只根据 板件的放置方式进行转换
 */
export class device1 {
    [key: string]: any;

    /**G代码基类 */
    _Action = new GCodeAction()
    /** 刀库 */
    knifeList?: Array<_knifeType>
    /** 当前使用的刀具 */
    usedKnife?: _knifeType

    /** 暴露给外部的参数 */
    config: configItem[] = confList
    /** 板厚 -- 读数据处理的数据 */
    thickness: number = 18
    /** 文件导出工具类 */
    zipFile?: FileZip


    /**是否执行换刀后的第一行精简指令 */
    doSimpleFirstCode: boolean = false

    lastCodeParams?: CodeParams

    /** 为了开发 方便  把一些配置的字段写在这里  */

    /** ZIP压缩包文件名 */
    exportOrderPathName: string = '{2}.zip'
    /** NC文件编码 */
    ncFileEncoding: string = 'UTF-8'
    /** 是否添加注释信息 */
    isNcFileComment: boolean = true
    /** 是否空行插入前缀 */
    isNcLinePrefixEnabled: boolean = false
    /** 空行插入前缀  前缀内容*/
    ncLinePrefix: string = ''
    /** 文件名 */
    blockFileName: string = '{9}.ban'

    /** 使用精简指令 */
    isUseSimpleCode: boolean = false
    /** 精简换刀后第一行指令 */
    isSimpleFirstCode: boolean = false
    /** 反转圆弧指令 */
    reverseArcCode: boolean = false
    /** 空程移动指令 */
    NcCodeFreeMove: string = 'G0'
    /** 直线插补标识 */
    NcCodeLineInterpolation: string = 'G1'
    /** 顺时针圆弧插补标识 */
    NcCodeClockwiseArcInterpolation: string = 'G2'
    /** 逆时针圆弧插补标识 */
    NcCodeAnticlockwiseArcInterpolation: string = 'G3'
    /** 水平坐标横轴标识 */
    NcCodeAxisX: string = 'Y'
    /** 水平坐标纵轴标识 */
    NcCodeAxisY: string = 'X'
    /** 垂直坐标轴标识 */
    NcCodeAxisZ: string = 'Z'
    /** 速度标识 */
    NcCodeSpeed: string = 'F'
    /** 水平坐标横轴增量标识 */
    NcCodeIncrementAxisX: string = 'I'
    /** 水平坐标纵轴增量标识 */
    NcCodeIncrementAxisY: string = 'J'
    /** 垂直坐标轴增量标识 */
    NcCodeIncrementAxisZ: string = 'K'
    /** 注释标识符 */
    leaderChar: string = '//'
    /** 解析器（输出机台）的坐标系基准 x */
    boardLength: number = 2440
    /** 解析器（输出机台）的坐标系基准 y */
    boardWidth: number = 1220
    /** 解析器（输出机台）的坐标系基准 z */
    boardHeight: number = 50

    /** 水平基准 */
    originPointPosition: BoardPosition = BoardPosition.LEFT_TOP
    /** 垂直基准 */
    originZ0Position: OriginZPosition = OriginZPosition.BoardFace
    /** 水平纵轴坐标轴向 */
    widthSideAxis: AxisType = AxisType.X_POS
    /** 水平横轴坐标轴向 */
    lengthSideAxis: AxisType = AxisType.Y_POS
    /** 垂直轴坐标轴向 */
    heightAxis: AxisType = AxisType.Z_POS
    /** 大板定位 */
    boardLocation: BoardPosition = BoardPosition.LEFT_BOTTOM
    /** 大板长轴偏置 */
    boardOffsetY: number = 0
    /** 大板短轴偏置 */
    boardOffsetX: number = 0
    /** 导出文件 保留小数点位数 */
    decimalPointPrecision: number = 3
    /** 末尾补零 */
    fixFloatNumberEndZero: boolean = true
    /** 整数值末尾加小数点 */
    intNumberAddDecimalPoint: boolean = true
    /** 辅助开料指令 */
    helpCutKnifeCode: string = ''
    /** 辅助开料指令2 */
    helpCutKnifeCode2: string = ''

    constructor() {
        this.knifeList = []
        this.knifeList.push(knifeData)
        this.knifeList.push(knifeData1)
        this.knifeList.push(knifeData2)
    }
    /**文件头 */
    FSTART: _type = {
        FSTART: `G54 G90\nM03\n`,
        // F1: `G54 G90\n`,
        // F2: `G53 G90\nM03\n`,
    }
    /** 换刀 */
    T: _type = {
        // T6: `T6 \nG43 H6\n`,
        // Ts6: `T6\n`,
        // T7: `T7\nG43 H7\n`,
        // ** 默认都走这个 调刀指令  若无需调刀则返回 '' 要调刀 则返回刀具的调刀代码 */
        // 20250508  将TD 定义为 
        /**根据刀直径（半径）找刀 */

        TD: (diameter, params?: any) => {
            let code = ''
            let _knife: _knifeType | undefined = undefined

            if (diameter) {
                _knife = this.knifeList?.find(e => e.diameter == diameter)
            }


            if (_knife != undefined) {
                code += this.checkChangeKnifeEndCode(_knife)
                this.usedKnife = _knife
                // code += _knife.axisStartCode
                code += _knife.knifeStartCode
            }
            if (this.isSimpleFirstCode == true && this.lastCodeParams) {
                this.doSimpleFirstCode = true
            }
            return code
        },
        /**根据刀名称找刀 换刀 */
        TN: (knifeName, params?: any) => {
            let code = ''
            let _knife: any = null
            if (knifeName) {
                _knife = this.knifeList?.find(e => e.knifeName == knifeName)
            }

            if (_knife != undefined) {
                code += this.checkChangeKnifeEndCode(_knife)
                this.usedKnife = _knife
                // code += _knife.axisStartCode
                code += _knife.knifeStartCode
            }
            if (this.isSimpleFirstCode == true && this.lastCodeParams) {
                this.doSimpleFirstCode = true
            }
            return code
        },
        // 停刀 + 停轴
        TE: () => {
            let code = ''
            if (this.usedKnife) {
                code += this.usedKnife.knifeStopCode + '\n'
                // code += this.usedKnife.axisStopCode + '\n'
            }

            return code
        }

    }
    /**主程序 G0 - G3  辅助开料 */
    C: _type = {
        KnifeMove: (processItem: CodeParamsObj) => {
            let code = ''
            /** 获取调刀代码   */
            if (processItem.processItemInfo?.knife) {
                let changeKnifeCode = ''
                if (processItem.processItemInfo?.knife.diameter) {
                    changeKnifeCode = this.T?.TD(processItem.processItemInfo?.knife.diameter)
                } else if (processItem.processItemInfo?.knife.knifeName) {
                    changeKnifeCode = this.T?.TN(processItem.processItemInfo?.knife.knifeName)
                }

                code += changeKnifeCode
            }

            if (processItem.list.length > 0) {
                for (const item of processItem.list) {
                    let point = item
                    if (processItem.processItemInfo) {
                        point = this.getXYZ(item, processItem.processItemInfo);
                    }
                    if (this.reverseArcCode == true) {
                        if (point.dir == 2) {
                            point.dir = 3
                        } else if (point.dir == 3) {
                            point.dir = 2
                        }
                    }
                    code += `${this.KnifeMove_Arc(point)} \n`
                }
            } else {
                this.logError('doProcessItem fail,no processItem Data')
            }
            return code
        },
        HelpCutCode1: this.helpCutKnifeCode,
        HelpCutCode2: this.helpCutKnifeCode2
    }
    /**主程序结束 */
    CE: _type = {
        CE1: `M05\n`
    }
    /**文件尾 */
    FEND: _type = {
        FEND: `M30\n`,
        // FEND1: `M30\n`
    }
    /** 备注 */
    MARK: _type = {
        Mark: (str) => {
            let markContent = str + '\n'
            let isShowMark = this.isNcFileComment || false
            if (isShowMark) {
                let leaderChar = this.leaderChar || ''
                markContent = `${leaderChar} ${markContent}`
            } else {
                markContent = ''
            }
            return markContent + '\n'
        },
        BLOCKREMARK1: `//Cut Block`,
    }
    /** 文件操作指令 */
    FILE: _type = {
        START: (zipName) => {
            this.initZipFile()
            this.setZipFileName(zipName)
        },
        ADDFILE: (data) => {
            this.setFile(data)
        },
        END: () => {
            this.fileDownLoad()
        }
    }
    /** 根据分组key 返回配置项 */
    getConfigListByGroupKey(groupKey: string, groupCompment?: string[][]) {
        let list = this.config.filter(e => e.groupBy == groupKey)
        if (groupCompment) {
            for (const groupItem of groupCompment) {
                if (groupItem.length > 1) {
                    let itemFirstOne = groupItem[0]
                    let concatItem: configItem[] = [];
                    let itemWithOutFirst = groupItem.filter((e, x) => x != 0)
                    itemWithOutFirst.forEach(keyStr => {
                        concatItem = list.filter(e => e.key == keyStr)
                        list = list.filter(e => e.key != keyStr)
                    })

                    let i = list.findIndex(e => e.key == itemFirstOne)

                    list[i].componentList = concatItem
                } else {
                    // 分组的 配置项 如果少于2个 就不需要分组
                    continue
                }
            }
        }
        console.log('getConfigListByGroupKey',groupKey,groupCompment,list);
        return list
    }
    /** 根据代码列表 获取指令列表 */
    getCodeListByOrderKeys(codeKeys: string[]) {
        let list = []
        codeKeys.forEach(code => {
            let orderList = this.getCodeByOrderKeys(code)
            list = list.concat(orderList)
        })
        return list
    }
    /** 根据代码 获取指令列表 */
    getCodeByOrderKeys(codeKey: string) {
        let list: any = []

        let arr = Object.keys(this[codeKey])
        arr.forEach(str => {
            let temp = {
                code: codeKey,
                order: str
            }
            list.push(temp)
        })
        return list
    }
    //** 使用G0代码 移动刀具 */
    KnifeMove(params: CodeParams) {
        let res = this._Action.G0(params)
        return res + '\n'
    }
    /**
     *  根据传入的数据 生成 G0 || G1 || G2 || G3 代码 
     */
    KnifeMove_Arc(params: CodeParams) {
        let res: any = ''

        let dir = params.dir || 0
        if (typeof (dir) == 'string') {
            dir = parseInt(dir)
        }

        let usedParams: CodeParams = { ...params }
        if (this.doSimpleFirstCode == true && this.lastCodeParams) {
            for (const key in usedParams) {
                if (['x', 'y', 'z', 'f'].includes(key)) {
                    if (usedParams[key] == this.lastCodeParams[key]) {
                        Reflect.deleteProperty(usedParams, key)
                    }
                }
            }
            this.doSimpleFirstCode = false
        }

        switch (dir) {
            case 0:

                if (typeof (this.NcCodeFreeMove) == 'string' && this.NcCodeFreeMove != '') {
                    usedParams.codeKey = this.NcCodeFreeMove
                }
                if (typeof (this.NcCodeAxisX) == 'string' && this.NcCodeAxisX != '') {
                    usedParams.xKey = this.NcCodeAxisX
                }
                if (typeof (this.NcCodeAxisY) == 'string' && this.NcCodeAxisY != '') {
                    usedParams.yKey = this.NcCodeAxisY
                }
                if (typeof (this.NcCodeAxisZ) == 'string' && this.NcCodeAxisZ != '') {
                    usedParams.zKey = this.NcCodeAxisZ
                }
                if (typeof (this.NcCodeSpeed) == 'string' && this.NcCodeSpeed != '') {
                    usedParams.fKey = this.NcCodeSpeed
                }

                res = this._Action.G0(usedParams)
                break;
            case 1:
                if (typeof (this.NcCodeLineInterpolation) == 'string' && this.NcCodeLineInterpolation != '') {
                    usedParams.codeKey = this.NcCodeLineInterpolation
                }
                if (typeof (this.NcCodeAxisX) == 'string' && this.NcCodeAxisX != '') {
                    usedParams.xKey = this.NcCodeAxisX
                }
                if (typeof (this.NcCodeAxisY) == 'string' && this.NcCodeAxisY != '') {
                    usedParams.yKey = this.NcCodeAxisY
                }
                if (typeof (this.NcCodeAxisZ) == 'string' && this.NcCodeAxisZ != '') {
                    usedParams.zKey = this.NcCodeAxisZ
                }
                if (typeof (this.NcCodeSpeed) == 'string' && this.NcCodeSpeed != '') {
                    usedParams.fKey = this.NcCodeSpeed
                }
                res = this._Action.G1(usedParams)
                break;
            case 2:
                if (typeof (this.NcCodeClockwiseArcInterpolation) == 'string' && this.NcCodeClockwiseArcInterpolation != '') {
                    usedParams.codeKey = this.NcCodeClockwiseArcInterpolation
                }
                if (typeof (this.NcCodeAxisX) == 'string' && this.NcCodeAxisX != '') {
                    usedParams.xKey = this.NcCodeAxisX
                }
                if (typeof (this.NcCodeAxisY) == 'string' && this.NcCodeAxisY != '') {
                    usedParams.yKey = this.NcCodeAxisY
                }
                if (typeof (this.NcCodeAxisZ) == 'string' && this.NcCodeAxisZ != '') {
                    usedParams.zKey = this.NcCodeAxisZ
                }
                if (typeof (this.NcCodeSpeed) == 'string' && this.NcCodeSpeed != '') {
                    usedParams.fKey = this.NcCodeSpeed
                }

                if (typeof (this.NcCodeIncrementAxisX) == 'string' && this.NcCodeIncrementAxisX != '') {
                    usedParams.iKey = this.NcCodeIncrementAxisX
                }
                if (typeof (this.NcCodeIncrementAxisY) == 'string' && this.NcCodeIncrementAxisY != '') {
                    usedParams.jKey = this.NcCodeIncrementAxisY
                }
                if (typeof (this.NcCodeIncrementAxisZ) == 'string' && this.NcCodeIncrementAxisZ != '') {
                    usedParams.kKey = this.NcCodeIncrementAxisZ
                }
                res = this._Action.G2(usedParams)
                break;
            case 3:
                if (typeof (this.NcCodeAnticlockwiseArcInterpolation) == 'string' && this.NcCodeAnticlockwiseArcInterpolation != '') {
                    usedParams.codeKey = this.NcCodeAnticlockwiseArcInterpolation
                }
                if (typeof (this.NcCodeAxisX) == 'string' && this.NcCodeAxisX != '') {
                    usedParams.xKey = this.NcCodeAxisX
                }
                if (typeof (this.NcCodeAxisY) == 'string' && this.NcCodeAxisY != '') {
                    usedParams.yKey = this.NcCodeAxisY
                }
                if (typeof (this.NcCodeAxisZ) == 'string' && this.NcCodeAxisZ != '') {
                    usedParams.zKey = this.NcCodeAxisZ
                }
                if (typeof (this.NcCodeSpeed) == 'string' && this.NcCodeSpeed != '') {
                    usedParams.fKey = this.NcCodeSpeed
                }

                if (typeof (this.NcCodeIncrementAxisX) == 'string' && this.NcCodeIncrementAxisX != '') {
                    usedParams.iKey = this.NcCodeIncrementAxisX
                }
                if (typeof (this.NcCodeIncrementAxisY) == 'string' && this.NcCodeIncrementAxisY != '') {
                    usedParams.jKey = this.NcCodeIncrementAxisY
                }
                if (typeof (this.NcCodeIncrementAxisZ) == 'string' && this.NcCodeIncrementAxisZ != '') {
                    usedParams.kKey = this.NcCodeIncrementAxisZ
                }
                res = this._Action.G3(usedParams)
                break;
            default:
                res = ''
                break;
        }
        this.lastCodeParams = params
        return res
    }
    /** 换刀指令  --
     * 
     * 因为设计的问题   
     * 现在配置界面会直接配置换刀代码
     *  
     * 
     * 配置数据读取
     * @param str: 刀具名 
     */
    KnifeChange(str: String) {
        let res: any = null
        if (str != '') {
            res = this._Action.M06(str)
        }
        return res
    }
    /**  开始指令 
     * 因为设计的问题
     * 相关配置从传入配置中读取
    */
    startCode() {
        let res: any = null
        /** 文件头 */
        res = 'codeName\n'
        /** 轴启动 */
        res += this._Action.M03(18000)

        /** 刀启动 */
        res = this._Action.M06('T1')

        /** */
        return res
    }
    /**  结束指令 
    * 因为设计的问题
    * 相关配置从传入配置中读取
   */
    endCode() {
        let res: any = null

        /**刀停止 */

        /** 轴停止 */
        res = this._Action.M05()

        return res
    }

    /** 处理空行过滤 */
    handleEmptyLine(code) {
        let arr = code.split('\n')
        let newArr = arr.filter(e => e != '')
        newArr = newArr.map(e => {
            return e + '\n'
        })

        let newCode = newArr.join('')
        return newCode
    }
    /** 空行插入前缀 */
    handleEmptyLinePreFix(code) {
        let arr = code.split('\n')
        let newArr = arr.map(e => {
            if (e != '') {
                return e
            } else {
                return `${this.ncLinePrefix} ${e}\n`
            }
        })

        let newCode = newArr.join('\n')
        return newCode
    }
    /** 空行插入后缀 */
    handleEmptyLineSuffix(code) {
        let arr = code.split('\n')
        let newArr = arr.map(e => {
            if (e != '') {
                return e
            } else {
                return `${e} ${this.ncLineSuffix}\n`
            }
        })

        let newCode = newArr.join('\n')
        return newCode
    }
    /** 精简指令 */
    handleSimpleCode(code) {
        let arr = code.split('\n')
        let objArr = arr.map((e, i) => {
            let type = (!e.includes('//') && !e.includes('/**') || e.includes('"//')) ? 'code' : 'remark'
            let temp = {
                code: e,
                index: i,
                regList: e.match(/[XYZF]([-+]?\d*\.?\d+)/g) || [],
                type
            }
            return temp
        })
        let checkLines = objArr.filter(e => e.type == 'code')

        // 校验对象  存键值
        let targetObject = {}
        for (const key in checkLines) {
            let line = checkLines[key]
            if (line.regList.length > 0) {
                for (const regVal of line.regList) {
                    let reg = regVal.match(/([XYZF])([-+]?\d*\.?\d+)/)
                    let FullVal = reg[0]
                    let valKey = reg[1]
                    let val = reg[2]
                    if (Reflect.has(targetObject, valKey)) {
                        if (targetObject[valKey] == val) {
                            // 相同  需要精简操作
                            objArr[line.index].code = objArr[line.index].code.replace(FullVal, ``)
                        } else {
                            // 不同 更新校验的值
                            targetObject[valKey] = val
                        }
                    } else {
                        // 键值 没有  赋给 校验对象
                        Reflect.set(targetObject, valKey, val)
                    }
                }
            }
        }
        let newCode = objArr.map(e => e.code).join('\n')
        // 这里 数据中莫名其妙多了个空格 处理下
        let list = newCode.split(' ').filter(e => e != '')
        newCode = list.join(' ')
        return newCode
    }

    /** 获取配置列表 */
    getConfigList() {
        return this.config
    }
    //** 设置配置列表 */
    setConfigList(List: configItem[]) {
        this.config = [...List]
    }
    //** 装载配置列表 */
    updateConfigList(List: configItem[]) {
        this.config = [...this.config, ...List]
    }
    /**根据数据导出文件 */
    async exportFile(data: CodeParamsList) {
        if (this.zipFile == undefined) {
            this.zipFile = new FileZip()
        }

        let zipFileName = data.processListInfo?.zipFileName || 'test.zip'
        this.zipFile.SetZipFileName(zipFileName)

        let fileNameInfo = data.processListInfo?.fileName
        let fileName = ''
        if (typeof (fileNameInfo) == 'string') {
            fileName = fileNameInfo
        } else {
            fileName = `${fileNameInfo?.name || 'empty'}.${fileNameInfo?.Suffix || '.txt'}`
        }
        let str = '';
        for (const item of data.processList) {
            if (!item.processItemInfo) {
                continue
            }
            let code = item.processItemInfo.code
            let order = item.processItemInfo.order
            let params = item.processItemInfo.codeParams

            if (code == 'C') {
                params = item
            }
            str += await this.handleCode(code, order, params)
        }

        await this.zipFile.PushFile(fileName, str)


        await this.zipFile.Download()
    }
    initZipFile() {
        this.zipFile = new FileZip()
    }
    /** 设置文件名 */
    async setZipFileName(zipName, fileZipObj?: FileZip) {
        if (this.zipFile == undefined) {
            return
        }
        let zipFileName = zipName || 'empty.zip'
        if (fileZipObj != undefined) {
            this.fileZipObj.SetZipFileName(zipFileName)
        } else {
            this.zipFile.SetZipFileName(zipFileName)
        }

    }
    /** 给zip文件内添加文件 */
    // async setFile(fileName, content, fileZipObj?: FileZip) {
    async setFile(data) {
        const { fileName, content, fileZipObj, isBase64, ncFileEncoding } = data
        if (this.zipFile == undefined) {
            return
        }
        let _isBase64 = isBase64 || false
        let fileCode = ncFileEncoding || 'UTF-8'
        if (fileZipObj != undefined) {
            await this.fileZipObj.PushFile(fileName, content, _isBase64, fileCode)
        } else {
            await this.zipFile.PushFile(fileName, content, _isBase64, fileCode)
        }

    }
    /** 下载zip */
    async fileDownLoad(fileZipObj?: FileZip) {
        if (this.zipFile == undefined) {
            return
        }
        if (fileZipObj != undefined) {
            await this.fileZipObj.Download()
        } else {
            await this.zipFile.Download()
        }

    }

    /** 处理C代码 */
    handleCode(codeType, codeKey, params?) {
        let str = ''
        if (typeof (this[codeType][codeKey]) == 'function') {
            str += this[codeType][codeKey](params)
        } else {
            str += this[codeType][codeKey]
        }
        if (!codeType) {
            str = '// 未输入代码，请重新输入\n'
        } else if (!codeKey) {
            str = '// 未输入指令，请重新输入\n'
            let k = Object.keys(this[codeType])
            str += `// 代码【${codeType}】支持的指令为：${k.join(',')}`
        } else if (str == 'undefined') {
            str = `// 代码【${codeType}】指令【${codeKey}】解析失败，请检查代码指令配置\n`
        }

        console.log('handleCode', str)

        // 输出文件未 清除上一条的刀路数据
        if (codeType == 'FEND') {
            this.lastCodeParams = undefined
        }

        return str
    }
    /** 更新代码 */
    updateCode(data) {
        for (const codeItem of data) {
            this.updateOrder(codeItem.value, codeItem.orderArr)
        }
    }
    /** 更新指令 */
    updateOrder(code, orderArr) {
        // 更新前 指令的数据集
        let keys = Object.keys(this[code])

        //  更新指令
        for (const orderItem of orderArr) {
            let key = orderItem.name
            let value = orderItem.value
            Reflect.set(this[code], key, value)
        }

        // 移除指令
        let orderKeys = orderArr.map(e => e.name)

        for (const key of keys) {
            if (!orderKeys.includes(key)) {
                if (typeof (this[code][key]) != 'function') {
                    Reflect.deleteProperty(this[code], key)
                }
            }
        }
    }
    /** 添加备注 */
    blockRemark(params?) {
        let str = `//Cut Block Id=${params.boardId} No=${params.blockNo} W=${params.width} L=${params.length}\n`
        return str
    }
    /** */
    getFileHead(params?) {
        let str = `//根据参数生成文件头\n ${params}\n`
        return str
    }
    getRealPoints(list: CodeParams[]) {
        let newList = [...list]
        for (let point of newList) {
            point = this.getRealPoint(point)
        }
    }
    getRealPoint(point: CodeParams) {
        let newPoint = { ...point }

        return newPoint
    }

    /** 内部功能逻辑  换刀时判断是否需要 上一把刀的停止代码 */
    checkChangeKnifeEndCode(knife: _knifeType) {
        let code = '';
        /** true 需要 输出停刀代码  false 不需要 */
        let flag = false
        // 必须要有刀
        if (this.usedKnife) {
            let keys = Object.keys(knife)
            for (const key of keys) {
                if (knife[key] != this.usedKnife[key]) {
                    flag = true
                    continue
                }
            }
        }
        if (flag) {
            code += this.T.TE()
        }
        return code;
    }
    /**
     * 
     * @param point 加工项的点
     * @param processItemInfo 加工项的信息 水平基准、垂直基准、轴方向（x、y、z），板件方向（x、y、z）
     * // 这里加工项的点数据  都是经过数据处理的  假定这里拿到的数据都是基于左上角 台面
     * 
     * @returns 实际加工项的点
     */
    getXYZ(point: CodeParams, processItemInfo: ProcessInfo): CodeParams {
        let newPoint: any = {}
        for (const key in point) {
            if (point[key] != undefined) {
                Reflect.set(newPoint, key, parseFloat(point[key]))
            }
        }
        /** 有2个部分  
         * 一个是基于机台和板件的转换  依据板件定位
         * 另外一个是基于板件和加工项的转换 依据板件长高方向*/
        switch (this.originZ0Position) {
            case 0:
                // 台面 不操作
                break;
            case 1:
                // 板面 Z坐标需要转换
                newPoint.z = newPoint.z - this.thickness
                break;
            default:
                break;
        }

        /** step  先转换板件的位置 */
        // 大板定位 不同 根据不同的定位点修改

        // processItemInfo
        switch (this.originPointPosition) {
            case BoardPosition.LEFT_TOP:
                // 不操作
                newPoint = this.changeXYZAxiosSide(newPoint)
                break;
            case BoardPosition.LEFT_BOTTOM:
                // 左下角 x坐标要转换
                newPoint.x = newPoint.x + this.boardWidth - processItemInfo.block.cutWidth //400
                newPoint = this.changeXYZAxiosSide(newPoint)
                break;
            case BoardPosition.RIGHT_TOP:
                // 右上角 y坐标要转换
                newPoint.y = newPoint.y + this.boardLength - processItemInfo.block.cutLength // 600
                newPoint = this.changeXYZAxiosSide(newPoint)
                break;
            case BoardPosition.RIGHT_BOTTOM:
                // 右下角 xy 坐标要转换
                newPoint.x = newPoint.x + this.boardWidth - processItemInfo.block.cutWidth
                newPoint.y = newPoint.y + this.boardLength - processItemInfo.block.cutLength
                newPoint = this.changeXYZAxiosSide(newPoint)
                break;
            default:
                break;
        }




        // 这里做 数值的小数点处理 
        for (const key in newPoint) {
            if (['x', 'y', 'z', 'r', 'i', 'j', 'k'].includes(key)) {
                let isTofix = false
                if (this.fixFloatNumberEndZero == true) {
                    // 末尾补零
                    isTofix = true
                } else if (this.intNumberAddDecimalPoint == true) {
                    // 整数值末尾加小数点
                    if (Number.isInteger(newPoint[key])) {
                        isTofix = true
                    }
                }
                if (isTofix) {
                    newPoint[key] = parseFloat(newPoint[key]).toFixed(this.decimalPointPrecision)
                }
            }
        }


        return newPoint
    }
    /** 根据 轴向变更坐标 */
    changeXYZAxiosSide(point: CodeParams) {
        let newPoint: any = {}
        for (const key in point) {
            if (point[key] != undefined) {
                Reflect.set(newPoint, key, parseFloat(point[key]))
            }
        }
        let width = this.boardWidth
        let length = this.boardLength
        let height = this.boardHeight


        if (this.widthSideAxis == AxisType.X_POS && this.lengthSideAxis == AxisType.Y_POS) {
            // 默认 为 X = x 正 Y = y 正  不操作
        } else if (this.widthSideAxis == AxisType.Y_POS && this.lengthSideAxis == AxisType.X_POS) {
            // x = y正   y = x正   X Y坐标 倒转
            newPoint = { ...newPoint, x: newPoint.y, y: newPoint.x }
        }

        else if (this.widthSideAxis == AxisType.X_NEG && this.lengthSideAxis == AxisType.Y_POS) {
            // x = x负  y = y正     
            newPoint = { ...newPoint, x: newPoint.x - width, y: newPoint.y }
        } else if (this.widthSideAxis == AxisType.Y_POS && this.lengthSideAxis == AxisType.X_NEG) {
            // x = y正  y = x负   
            newPoint = { ...newPoint, x: newPoint.y - width, y: newPoint.x }
        }

        else if (this.widthSideAxis == AxisType.X_NEG && this.lengthSideAxis == AxisType.Y_NEG) {
            // x = x负  y = y负  
            newPoint = { ...newPoint, x: newPoint.x - width, y: newPoint.y - length }
        } else if (this.widthSideAxis == AxisType.Y_NEG && this.lengthSideAxis == AxisType.X_NEG) {
            // x = y负  y = x负   
            newPoint = { ...newPoint, x: newPoint.y - width, y: newPoint.x - length }
        }

        else if (this.widthSideAxis == AxisType.X_POS && this.lengthSideAxis == AxisType.Y_NEG) {
            // x = x正  y = y负  
            newPoint = { ...newPoint, x: newPoint.x, y: newPoint.y - length }
        } else if (this.widthSideAxis == AxisType.Y_NEG && this.lengthSideAxis == AxisType.X_POS) {
            // x = y负  y = x正  
            newPoint = { ...newPoint, x: newPoint.y, y: newPoint.x - length }
        }

        if (this.heightAxis == AxisType.Z_NEG) {
            // Z轴负
            newPoint = { ...newPoint, z: newPoint.z - height }
        }

        return newPoint
    }
}

