
import { configItem, ncEncodeMap, smallPlateOptionsMap, zipFileOptionsMap, AxisType } from "../../confClass";
/** 解析器1 的配置参数 */
export const confList: configItem[] = [
    {
        key: 'boardWidth',
        defaultValue: 1220,
        label: '机台宽度',
        groupBy: 'base',
        component: 'InputNumber',
        style: {
            width: '120px'
        }
    },
    {
        key: 'boardLength',
        defaultValue: 2440,
        label: '机台长度',
        groupBy: 'base',
        component: 'InputNumber',
        style: {
            width: '120px'
        }
    },
    {
        key: 'boardHeight',
        defaultValue: 100,
        label: '机台高度',
        groupBy: 'base',
        component: 'InputNumber',
        style: {
            width: '120px',
        }
    },
    {
        key: 'originPointPosition',
        defaultValue: 3,
        label: '原点位置',
        groupBy: 'baseAxios',
        helpMessage: `提示：加工台面工件原点，坐标XY轴零点位置`,
        component: 'Select',
        options: [
            {
                value: 3,
                label: '左上角',
            },
            {
                value: 0,
                label: '左下角',
            },
            {
                value: 1,
                label: '右下角',
            },
            {
                value: 2,
                label: '右上角',
            },
        ],
        style: {
            width: '120px'
        }
    },
    {
        // 注： isUseBoardFaceZ
        key: 'originZ0Position',
        defaultValue: 0,
        label: '垂直原点',
        groupBy: 'baseAxios',
        component: 'Select',
        options: [
            {
                value: 0,
                label: '台面',
            },
            {
                value: 1,
                label: '板面',
            }
        ],
        style: {
            width: '120px'
        }
    },
    {
        key: 'widthSideAxis',
        defaultValue: AxisType.X_POS,
        label: '水平纵轴坐标轴向',
        groupBy: 'axios',
        component: 'Select',
        options: [
            {
                value: 0,
                label: 'X轴正',
            },
            {
                value: 1,
                label: 'X轴负',
            },
            {
                value: 2,
                label: 'Y轴正',
            },
            {
                value: 3,
                label: 'Y轴负',
            },
        ],
        helpMessage: `提示：屏幕显示的上下方向。`,
    },
    {
        key: 'lengthSideAxis',
        defaultValue: AxisType.Y_POS,
        label: '水平横轴坐标轴向',
        groupBy: 'axios',
        helpMessage: `提示：屏幕显示的上下方向。`,
        component: 'Select',
        options: [
            {
                value: 0,
                label: 'X轴正',
            },
            {
                value: 1,
                label: 'X轴负',
            },
            {
                value: 2,
                label: 'Y轴正',
            },
            {
                value: 3,
                label: 'Y轴负',
            },
        ],
    },
    {
        key: 'heightAxis',
        defaultValue: AxisType.Z_POS,
        label: '垂直轴坐标轴向',
        groupBy: 'axios',
        helpMessage: `提示：向上，默认Z+。`,
        component: 'Select',
        options: [
            {
                value: 4,
                label: 'Z轴正',
            },
            {
                value: 5,
                label: 'Z轴负',
            }
        ],
    },
    {
        key: 'boardLocation',
        defaultValue: 0,
        label: '大板',
        groupBy: 'position',
        helpMessage: `提示：常规开料加工靠档位置。`,
        component: 'Select',
        options: [
            {
                value: 3,
                label: '左上角',
            },
            {
                value: 0,
                label: '左下角',
            },
            {
                value: 1,
                label: '右下角',
            },
            {
                value: 2,
                label: '右上角',
            },
        ],
    },
    {
        key: 'boardOffsetY',
        defaultValue: 0,
        label: '长边偏置',
        groupBy: 'position',
        helpMessage: `提示：大板工件原点长边方向偏置，用于校准机台坐标，支持正负值，单位mm毫米。`,
        component: 'InputNumber',
        style: {
            width: '120px'
        }
    },
    {
        key: 'boardOffsetX',
        defaultValue: 0,
        label: '短边偏置',
        groupBy: 'position',
        helpMessage: `提示：大板工件短边方向原点偏置，用于校准机台坐标，支持正负值，单位mm毫米。`,
        component: 'InputNumber',
        style: {
            width: '120px'
        }
    },
    {
        key: 'exportOrderPathName',
        label: 'ZIP压缩包文件名',
        groupBy: 'general',
        helpMessage: `支持路径(文件夹/目录)，如：D:/晨丰开料加工/{0}-{1}-{2}.zip`,
        component: 'MultiSelect',
        options: convertToLabelValue(zipFileOptionsMap), // getDictOptions(DICT_TYPE.WORK_AREA_COMMON_FILE_TYPE, 'string').reverse(),
        style: {
            width: '80px'
        }
    },
    {
        key: 'ncFileEncoding',
        label: 'NC文件编码',
        groupBy: 'general',
        helpMessage: `文件编码格式，后续可能会有扩展需要`,
        component: 'Select',
        options: convertToLabelValue(ncEncodeMap),
        style: {
            width: '120px'
        }
    },
    {
        key: 'isFilterEmptyLine',
        defaultValue: false,
        label: '空行过滤',
        groupBy: 'NcRowHandle',
        helpMessage: '过滤文件头尾、刀具启停等指令多余的空行，默认开启',
        component: 'CheckBox',
    },
    {
        key: 'isNcFileComment',
        defaultValue: '',
        label: '添加注释信息',
        groupBy: 'NcRowHandle',
        helpMessage: 'nc添加文件说明、加工说明的注释内容',
        component: 'CheckBox',
    },
    {
        key: 'leaderChar',
        defaultValue: '',
        label: '注释标识符',
        groupBy: 'NcRowHandle',
        helpMessage: 'nc指令行，指定起始符号+注释内容，如：“//开料开始，加工刀直径6mm”',
        component: 'Input',
    },
    {
        key: 'isNcLinePrefixEnabled',
        defaultValue: false,
        label: '空行插入前缀',
        groupBy: 'NcRowHandle',
        helpMessage: '开启NC每行前缀生效',
        component: 'CheckBox',
    },
    {
        key: 'ncLinePrefix',
        defaultValue: '',
        label: '前缀', //
        groupBy: 'NcRowHandle',
        helpMessage: '用于添加NC指令行起始符，支持类似格式N',
        component: 'Input',
    },
    {
        key: 'isNcLineSuffixEnabled',
        defaultValue: false,
        label: '空行插入后缀',
        groupBy: 'NcRowHandle',
        helpMessage: '开启NC每行后缀生效',
        component: 'CheckBox',
    },
    {
        key: 'ncLineSuffix',
        defaultValue: '',
        label: '后缀', // 
        groupBy: 'NcRowHandle',
        helpMessage: '用于添加NC指令行结束符',
        component: 'Input',
    },
    {
        key: 'blockFileName',
        defaultValue: '',
        label: '文件名',
        groupBy: 'fileNameConfig1',
        helpMessage: '支持路径(文件夹/目录)，如：NC/{10,#3}.nc',
        component: 'MultiSelect',
        options: convertToLabelValue(smallPlateOptionsMap),
        style: {
            width: '420px',
            marginRight: '50%'
        }
    },
    {
        key: 'blockFileAB',
        defaultValue: '',
        label: '合并正反面文件名',
        groupBy: 'fileNameConfig',
        helpMessage: '支持路径(文件夹/目录)，如：NC/{10,#3}.nc',
        component: 'MultiSelect',
        options: convertToLabelValue(smallPlateOptionsMap),
        style: {
            width: '420px',
            marginRight: '50%'
        }
    },
    // blockFileA
    {
        key: 'blockFileA',
        defaultValue: '',
        label: '正面文件名',
        groupBy: 'fileNameConfig',
        helpMessage: '支持路径(文件夹/目录)，如：NC/{10,#3}_Z.nc',
        component: 'MultiSelect',
        options: convertToLabelValue(smallPlateOptionsMap),
        style: {
            width: '420px',
            marginRight: '50%'
        }
    },
    {
        key: 'blockFileB',
        defaultValue: '',
        label: '反面文件名',
        groupBy: 'fileNameConfig',
        helpMessage: '支持路径(文件夹/目录)，如：NC/{10,#3}_F.nc',
        component: 'MultiSelect',
        options: convertToLabelValue(smallPlateOptionsMap),
        style: {
            width: '420px',
            marginRight: '50%'
        }
    },
    {
        key: 'isUseSimpleCode',
        defaultValue: false,
        label: '使用精简指令',
        groupBy: 'orderCode',
        helpMessage: `使用精简指令，默认关闭，开启用于支持精简XYZ和F指令`,
        component: 'CheckBox',
    },
    {
        key: 'isSimpleFirstCode',
        defaultValue: false,
        label: '精简换刀后第一行指令',
        groupBy: 'orderCode',
        helpMessage: `使用精简指令，默认关闭，开启用于支持精简XYZ和F指令(一般是配合嵌入式计算机控制系统的机台使用，因为存储空间和读取速度限制)。`,
        component: 'CheckBox',
    },
    {
        key: 'reverseArcCode',
        defaultValue: false,
        label: '反转圆弧指令',
        groupBy: 'orderCode',
        helpMessage: `提示：(配合坐标系设置项——顺时针圆弧指令G2变为G3，逆时针圆弧指令G3变为G2。)`,
        component: 'CheckBox',
    },
    {
        key: 'isNCPreCode',
        defaultValue: false,
        label: '代码前置',
        groupBy: 'cutCode',
        helpMessage: `提示：(插入到NC文件头前。)`,
        component: 'CheckBox',
    },

    {
        key: 'ncFileHead',
        defaultValue: '',
        label: '文件头',
        groupBy: 'NcOrderAndCode',
        helpMessage: `提示：(插入到NC文件头前。)`,
        component: 'Textarea',
        style: {
            width: '220px',
        }
    },
    {
        key: 'ncFileEnd',
        defaultValue: '',
        label: '文件尾',
        groupBy: 'NcOrderAndCode',
        helpMessage: `提示：(插入到NC文件尾后。)`,
        component: 'Textarea',
        style: {
            width: '220px',
        }
    },
    {
        key: 'ncMergeConnection',
        defaultValue: '',
        label: '合并连接',
        groupBy: 'NcOrderAndCode',
        helpMessage: `提示：(插入到NC文件尾后。)`,
        component: 'Textarea',
        style: {
            width: '220px',
        }
    },
    {
        key: 'NcCodeFreeMove',
        defaultValue: 'G0',
        label: '空程移动指令',
        labelClass: 'NcCodeKeysConfigLabel',
        groupBy: 'NcCodeKeysConfig',
        helpMessage: `提示：G0或G00指令`,
        component: 'Input',
        style: {
            width: '220px',
        },

    },
    {
        key: 'NcCodeLineInterpolation',
        defaultValue: 'G1',
        label: '直线插补标识',
        labelClass: 'NcCodeKeysConfigLabel',
        groupBy: 'NcCodeKeysConfig',
        helpMessage: `提示：文本，默认“G1”`,
        component: 'Input',
        style: {
            width: '220px',
        }
    },
    {
        key: 'NcCodeClockwiseArcInterpolation',
        defaultValue: 'G2',
        label: '顺时针圆弧插补标识',
        labelClass: 'NcCodeKeysConfigLabel',
        groupBy: 'NcCodeKeysConfig',
        helpMessage: `提示：文本，默认“G2”`,
        component: 'Input',
        style: {
            width: '220px',
        }
    },
    {
        key: 'NcCodeAnticlockwiseArcInterpolation',
        defaultValue: 'G3',
        label: '逆时针圆弧插补标识',
        groupBy: 'NcCodeKeysConfig',
        labelClass: 'NcCodeKeysConfigLabel',
        helpMessage: `提示：文本，默认“G3”`,
        component: 'Input',
        style: {
            width: '220px',
        }
    },
    // {
    //     key: 'NcCodeAxisX',
    //     defaultValue: 'Y',
    //     label: '水平坐标横轴标识',
    //     groupBy: 'NcCodeKeysConfig',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     helpMessage: `提示：默认“Y”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeAxisY',
    //     defaultValue: 'X',
    //     label: '水平坐标纵轴标识',
    //     groupBy: 'NcCodeKeysConfig',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     helpMessage: `提示：默认“X”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeAxisZ',
    //     defaultValue: 'Z',
    //     label: '垂直坐标轴标识',
    //     groupBy: 'NcCodeKeysConfig',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     helpMessage: `提示：默认“Z”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeSpeed',
    //     defaultValue: 'F',
    //     label: '速度标识',
    //     groupBy: 'NcCodeKeysConfig',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     helpMessage: `提示：默认“F”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeIncrementAxisX',
    //     defaultValue: 'J',
    //     label: '水平坐标横轴增量标识',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     groupBy: 'NcCodeKeysConfig',
    //     helpMessage: `提示：默认“J”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeIncrementAxisY',
    //     defaultValue: 'I',
    //     label: '水平坐标纵轴增量标识',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     groupBy: 'NcCodeKeysConfig',
    //     helpMessage: `提示：默认“I”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    // {
    //     key: 'NcCodeIncrementAxisZ',
    //     defaultValue: 'K',
    //     label: '垂直坐标轴增量标识',
    //     labelClass: 'NcCodeKeysConfigLabel',
    //     groupBy: 'NcCodeKeysConfig',
    //     helpMessage: `提示：默认“K”`,
    //     component: 'Input',
    //     style: {
    //         width: '220px',
    //     }
    // },
    {
        key: 'decimalPointPrecision',
        defaultValue: 0,
        label: 'NC小数点位数',
        groupBy: '',
        helpMessage: `NC文件中数值生成指定位数的小数点`,
    },
    {
        key: 'fixFloatNumberEndZero',
        defaultValue: true,
        label: '末尾补零',
        groupBy: '',
        helpMessage: ``,
    },
    {
        key: 'intNumberAddDecimalPoint',
        defaultValue: true,
        label: '整数值末尾加小数点',
        groupBy: '',
        helpMessage: ``,
    },
    {
        key: 'helpCutKnifeCode',
        defaultValue: '',
        label: '辅助开料指令',
        groupBy: 'helpCut',
        helpMessage: `提示：(辅助开料与正常开料加工指令之间插入的指定指令(关真空吸附、长延时、开真空吸附、短延时))。`,
        component: 'Textarea',
        style: {
            width: '240px',
        }
    },
    {
        key: 'helpCutKnifeCode2',
        defaultValue: '',
        label: '辅助开料指令2',
        groupBy: 'helpCut',
        helpMessage: `提示：(辅助开料与正常开料加工指令之间插入的指定指令(关真空吸附、长延时、开真空吸附、短延时))。
                  限制：启用双工位后生效。`,
        component: 'Textarea',
        style: {
            width: '240px',
        }
    },




    //     {
    //         key: 'boardFileHeadA',
    //         defaultValue: '',
    //         label: '大板NC正(A)面文件头',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'boardFileEndA',
    //         defaultValue: '',
    //         label: '大板NC正(A)面文件尾',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'boardFileHeadB',
    //         defaultValue: '',
    //         label: '反面文件头',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'boardFileEndB',
    //         defaultValue: '',
    //         label: '反面文件尾',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'isExportBlockFileB',
    //         defaultValue: '',
    //         label: '导出反面文件',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'blockNcFileHead',
    //         defaultValue: '',
    //         label: '文件头(合并)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'blockNcFileEnd',
    //         defaultValue: '',
    //         label: '文件尾(合并)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'isDualStationEnabled',
    //         defaultValue: false,
    //         label: '启用双工位',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'station2NcFileHeadA',
    //         defaultValue: '',
    //         label: '正面文件头(双工位)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'station2NcFileEndA',
    //         defaultValue: '',
    //         label: '正面文件尾(双工位)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'station2NcFileHeadB',
    //         defaultValue: '',
    //         label: '反面文件头(双工位)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'station2NcFileEndB',
    //         defaultValue: '',
    //         label: '反面文件尾(双工位)',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'isHelpCut',
    //         defaultValue: false,
    //         label: '辅助开料-启用',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'loadBoardCode',
    //         defaultValue: '',
    //         label: '上料代码',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'isLoadBoardBeforeFileHead',
    //         defaultValue: '',
    //         label: '上料代码插入到NC文件头前',
    //         groupBy: '',
    //     },
    //     {
    //         key: 'leaderChar',
    //         defaultValue: '',
    //         label: '注释标识符',
    //         groupBy: '',
    //         helpMessage: 'nc指令行，指定起始符号+注释内容，如：“//开料开始，加工刀直径6mm”',
    //     },
    //     {
    //         key: 'isNcFileComment',
    //         defaultValue: '',
    //         label: '添加注释信息',
    //         groupBy: '',
    //         helpMessage: 'nc添加文件说明、加工说明的注释内容',
    //     },

    //     {
    //         key: 'toolChangeLowerLimitActionCount',
    //         defaultValue: '',
    //         label: '延迟换刀下限动作数',
    //         groupBy: '',
    //         helpMessage: '',
    //     },
    //     {
    //         key: 'toolChangeDelayCode',
    //         defaultValue: '',
    //         label: '延迟换刀指令',
    //         groupBy: '',
    //         helpMessage: '',
    //     },
    //     {
    //         key: 'isFirstCutBorderInFaceB',
    //         defaultValue: '',
    //         label: '先修边',
    //         groupBy: '',
    //         helpMessage: '',
    //     },
    //     {
    //         key: 'isAllowDoubleSplit',
    //         defaultValue: '',
    //         label: '开料分刀',
    //         groupBy: '',
    //         helpMessage: '开料切割加工，分层切割，步进加工',
    //     },
    //     {
    //         key: 'isSplitCondition',
    //         defaultValue: '',
    //         label: '启用',
    //         groupBy: '',
    //         helpMessage: '提示：(启用钻孔分刀加工(啄钻))。',
    //     },
    //     {
    //         key: 'knifeSplitType',
    //         defaultValue: '',
    //         label: '分刀方式',
    //         groupBy: '',
    //         helpMessage: '加工的分拆方式',
    //     },
    //     {
    //         key: 'splitCount',
    //         defaultValue: 2,
    //         label: '分刀次数',
    //         groupBy: '',
    //         helpMessage: '按指定层数分刀，单位层',
    //     },
    //     {
    //         key: 'splitDepth',
    //         defaultValue: 15,
    //         label: '步进深度',
    //         groupBy: '',
    //         helpMessage: '分层切割的每层进刀量，最后一刀小于等于设定值，单位mm毫米',
    //     },
    //     {
    //         key: 'isModelSplitCondition',
    //         defaultValue: 15,
    //         label: '启用',
    //         groupBy: '',
    //         helpMessage: '提示：(启用钻孔分刀加工)。',
    //     },
    //     {
    //         key: 'modelKnifeSplitType',
    //         defaultValue: 15,
    //         label: '分刀方式',
    //         groupBy: '',
    //         helpMessage: '提示：(按深度——分刀加工时按指定的步进深度值，由浅到深依次生成多个造型刀路；按层数——加工深度按指定层数，等分步进深度生成多个造型刀路。)。',
    //     },
    //     {
    //         key: 'modelKnifeSplitType',
    //         defaultValue: 15,
    //         label: '分刀方式',
    //         groupBy: '',
    //         helpMessage: '提示：(按深度——分刀加工时按指定的步进深度值，由浅到深依次生成多个造型刀路；按层数——加工深度按指定层数，等分步进深度生成多个造型刀路。)。',
    //     },
    //     {
    //         key: 'modelSplitCount',
    //         defaultValue: 2,
    //         label: '分刀次数',
    //         groupBy: '',
    //         helpMessage: '按指定层数分刀，单位层',
    //     },
    //     {
    //         key: 'modelSplitDepth',
    //         defaultValue: 15,
    //         label: '步进深度',
    //         groupBy: '',
    //         helpMessage: '分层切割的每层进刀量，最后一刀小于等于设定值，单位mm毫米',
    //     },




    //     {
    //         key: 'isHoleByPosition',
    //         defaultValue: false,
    //         label: '排钻按位置加工',
    //         groupBy: '',
    //         helpMessage: '用于添加NC指令行结束符',
    //     },
    //     {
    //         key: 'holeSplitProcessType',
    //         defaultValue: 0,
    //         label: '加工方式',
    //         groupBy: '',
    //         helpMessage: '回安全点——同一位置相同直径的孔，打完一个深度后，回退到安全距离，再进行下一次打孔动作；退刀刀指定距离——同一位置相同直径的孔，打完一个深度后，回退指定距离后，再进行下一次打孔动作。',
    //     },
    //     {
    //         key: 'knifeUpSize',
    //         defaultValue: 2,
    //         label: '退刀指定距离',
    //         groupBy: '',
    //         helpMessage: '退刀指定距离',
    //     },
    //     {
    //         key: 'holeKnifeSplitType',
    //         defaultValue: 2,
    //         label: '分刀方式',
    //         groupBy: '',
    //         helpMessage: '提示：(按深度——分刀加工时按指定的步进深度值，由浅到深依次生成多个孔；按层数——加工深度按指定层数，等分步进深度生成多个孔。)。',
    //     },
    //     {
    //         key: 'throughHoleOneTime',
    //         defaultValue: true,
    //         label: '通孔一刀加工',
    //         groupBy: '',
    //         helpMessage: '含穿孔(深度大于等于板厚的孔)；否，避免刀具对刀误差无法完整打通相关的孔；是，减少反面加工概率',
    //     },
    //     {
    //         key: 'holeFreeSpeed',
    //         defaultValue: 0,
    //         label: '空程速度',
    //         groupBy: '',
    //         helpMessage: '提示：(为0时不插入空程动作(G00到板面)，直接从安全高度到孔底或预钻孔深度；非0则按指定速度加入空程动作。)。',
    //     },
    //     {
    //         key: 'preProcessHoleDepth',
    //         defaultValue: 0,
    //         label: '预钻深度',
    //         groupBy: '',
    //         helpMessage: `提示：(预钻孔深度，单位mm毫米)。
    //                   限制：为0时，置灰“预钻孔速度”、“预钻孔延时”、“预钻孔延时指令”。`,
    //     },
    //     {
    //         key: 'preProcessHoleSpeed',
    //         defaultValue: 0,
    //         label: '预钻速度',
    //         groupBy: '',
    //         helpMessage: `提示：(预钻孔加工速度，单位mm/min毫米每分钟)。
    //                   限制：“初始加工深度”大于0时有效。`,
    //     },
    //     {
    //         key: 'isCuttingSplitCondition',
    //         defaultValue: false,
    //         label: '启用',
    //         groupBy: '',
    //         helpMessage: '提示：(启用钻孔分刀加工)。',
    //     },
    //     {
    //         key: 'cuttingKnifeSplitType',
    //         defaultValue: 1,
    //         label: '分刀方式',
    //         groupBy: '',
    //         helpMessage: '加工的分拆方式',
    //     },
    //     {
    //         key: 'cuttingSplitCount',
    //         defaultValue: 2,
    //         label: '分刀次数',
    //         groupBy: '',
    //         helpMessage: '按指定层数分刀，单位层',
    //     },
    //     {
    //         key: 'cuttingSplitDepth',
    //         defaultValue: 15,
    //         label: '步进深度',
    //         groupBy: '',
    //         helpMessage: '分层切割的每层进刀量，最后一刀小于等于设定值，单位mm毫米',
    //     },
    //     {
    //         key: 'helpCutKnifeDepth',
    //         defaultValue: 0,
    //         label: '留底',
    //         groupBy: '',
    //         helpMessage: '辅助开料刀路加工深度，是否切透，单位mm毫米',
    //     },
    //     {
    //         key: 'workStartDistance',
    //         defaultValue: 30,
    //         label: '下刀长度',
    //         groupBy: '',
    //         helpMessage: `提示：(开料起始进刀距离(板面到台面水平方向距离)，单位mm毫米)。
    //                   限制：关联校验收刀提前长度。`,
    //     },
    //     {
    //         key: 'cuttingSplitProcessType',
    //         defaultValue: 0,
    //         label: '加工方式',
    //         groupBy: '',
    //         helpMessage: `提示：(按板件——逐片板件分层加工；按层——按拆分好的深度由浅到深的先后顺序加工。)。`,
    //     },
    //     {
    //         key: 'workSpeed',
    //         defaultValue: 8000,
    //         label: '速度',
    //         groupBy: '',
    //         helpMessage: `G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     },
    //     {
    //         key: 'workPreDistance',
    //         defaultValue: 8000,
    //         label: '提前长度',
    //         groupBy: '',
    //         helpMessage: `提示：(下刀点到斜向下刀底部水平距离，单位mm毫米)。
    //                   限制：关联校验收刀提前长度。`,
    //     },
    //     {
    //         key: 'holeSpeed',
    //         defaultValue: 5000,
    //         label: '加工速度',
    //         groupBy: '',
    //         helpMessage: ``,
    //     },
    //     {
    //         key: 'workStartSpeed',
    //         defaultValue: 3000,
    //         label: '下刀速度',
    //         groupBy: '',
    //         helpMessage: `提示：(G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟)。`,
    //     },
    //     {
    //         key: 'innerCornerDistence',
    //         defaultValue: 0,
    //         label: '内转角进弯长度',
    //         groupBy: '',
    //         helpMessage: `用于解决内转角进弯速度过快，变向后减速出现进弯与出弯圆弧角加工不到位问题，为0不启用，单位mm毫米`,
    //     },
    //     {
    //         key: 'innerCornerSpeed',
    //         defaultValue: 6000,
    //         label: '内转角速度',
    //         groupBy: '',
    //         helpMessage: `内转角减速段加工速度，G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     },
    //     {
    //         key: 'outerCornerSpeed',
    //         defaultValue: 6000,
    //         label: '外转角速度',
    //         groupBy: '',
    //         helpMessage: `内转角减速段加工速度，G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     },
    //     {
    //         key: 'sameBorderSpeed',
    //         defaultValue: 0,
    //         label: '共边速度',
    //         groupBy: '',
    //         helpMessage: `内转角减速段加工速度，G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     },
    //     {
    //         key: 'workEndDistace',
    //         defaultValue: 0,
    //         label: '共边速度',
    //         groupBy: '',
    //         helpMessage: `内转角减速段加工速度，G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     }, {
    //         key: 'workEndSpeed',
    //         defaultValue: 0,
    //         label: '收刀速度',
    //         groupBy: '',
    //         helpMessage: `G1/G2/G3或G01/G02/G03指令对应F速度值，单位mm/min毫米每分钟`,
    //     },

    //     {
    //         key: 'arcTurnPolylineLen',
    //         defaultValue: 0,
    //         label: '圆弧转多段线',
    //         groupBy: '',
    //         helpMessage: `圆弧转多段线刀路最大直线段长度值，用于对接不支持圆弧加工项的协议或机台；0圆弧描述，不转多段线刀路。`,
    //     },

    //     {
    //         key: 'arcCode',
    //         defaultValue: 0,
    //         label: '圆弧加工指令',
    //         groupBy: '',
    //         helpMessage: `提示：(配合坐标系设置项——顺时针圆弧指令G2变为G3，逆时针圆弧指令G3变为G2。)`,
    //     },
    //     {
    //         key: 'prestartAheadActionCount',
    //         defaultValue: 0,
    //         label: '预启动提前动作数',
    //         groupBy: '',
    //         helpMessage: `用于多主轴电机(多工序)固定刀具机台换刀加工，提前启动主轴，减少等待时间。提前动作计数说明：1. 空程移动，安全位置水平一个坐标点移动到另一个坐标点，记为一个动作；2. 打孔，安全位置向下-到板面-再到孔底-退刀到安全位置，记为一个动作；3. 拉槽/造型加工，安全位置向下-到凹槽深度-板内加工-对刀到安全位置，记为一个动作。`,
    //     },
    //     {
    //         key: 'cutBoardBorderB',
    //         defaultValue: 2,
    //         label: '反面修边值',
    //         groupBy: '',
    //         helpMessage: `修改变更后要求重新优化排版`,
    //     },
    //     {
    //         key: 'isSameOriginPointPosition',
    //         defaultValue: false,
    //         label: '同工位1设置',
    //         groupBy: '',
    //         helpMessage: `提示：(“是”——与工位1同样配置(使用工位1配置的水平基准(零点)、垂直基准(零点)、水平横轴坐标轴向、水平纵轴坐标轴向、垂直轴坐标轴向、*多坐标系、*多坐标系配置))。
    //                       限制：“是”则置灰工单坐标相关设置项。`,
    //     },
    //     {
    //         key: 'boardLocation2',
    //         defaultValue: 0,
    //         label: '大板定位',
    //         groupBy: '',
    //         helpMessage: `提示：常规开料加工靠档位置。`,
    //     },


    //     {
    //         key: 'offsetX_station2',
    //         defaultValue: 0,
    //         label: '短边偏移',
    //         groupBy: '',
    //         helpMessage: `人工偏移-短边偏移`,
    //     },
    //     {
    //         key: 'offsetY_station2',
    //         defaultValue: 0,
    //         label: '长边偏移',
    //         groupBy: '',
    //         helpMessage: `人工偏移-长边偏移`,
    //     },
    //     {
    //         key: 'offsetX_fromStation1',
    //         defaultValue: 0,
    //         label: '短边方向',
    //         groupBy: '',
    //         helpMessage: `工位2工件原点-短边方向`,
    //     },
    //     {
    //         key: 'offsetY_fromStation1',
    //         defaultValue: 0,
    //         label: '长边方向',
    //         groupBy: '',
    //         helpMessage: `工位2工件原点-长边方向`,
    //     },
    //     {
    //         key: 'blockOffsetX',
    //         defaultValue: 0,
    //         label: '小板短轴偏置',
    //         groupBy: '',
    //         helpMessage: `提示：小板工件原点短边方向偏置，用于校准机台坐标，支持正负值，单位mm毫米。`,
    //     },
    //     {
    //         key: 'blockOffsetY',
    //         defaultValue: 0,
    //         label: '小板长轴偏置',
    //         groupBy: '',
    //         helpMessage: `提示：小板工件原点长边方向偏置，用于校准机台坐标，支持正负值，单位mm毫米。`,
    //     },
    //     {
    //         key: 'originPointPosition2',
    //         defaultValue: 0,
    //         label: '水平基准(零点)',
    //         groupBy: '',
    //         helpMessage: `提示：(加工台面工件原点，坐标XY轴零点位置(X0Y0点))。`,
    //     },
    //     {
    //         key: 'widthSideAxis2',
    //         defaultValue: 0,
    //         label: '水平纵轴坐标轴向',
    //         groupBy: '',
    //         helpMessage: `提示：(加工台面工件原点，坐标Z轴零点位置(Z0平面)，可选台面、板面)。`,
    //     },
    //     {
    //         key: 'lengthSideAxis2',
    //         defaultValue: 0,
    //         label: '水平横轴坐标轴向',
    //         groupBy: '',
    //         helpMessage: `提示：(加工台面工件原点，坐标Z轴零点位置(Z0平面)，可选台面、板面)。`,
    //     },



    //     {
    //         key: 'blockKnifeLineSpacing',
    //         defaultValue: 0,
    //         label: '刀路间距',
    //         groupBy: '',
    //         helpMessage: ``,
    //     },
    //     {
    //         key: 'extensionBeforeCutBoardBorder',
    //         defaultValue: 0,
    //         label: '刀路前延伸',
    //         groupBy: '',
    //         helpMessage: `修边刀路起始前延伸量，用于解决修边不充分问题。`,
    //     },
    //     {
    //         key: 'extensionAfterCutBoardBorder',
    //         defaultValue: 0,
    //         label: '刀路后延伸',
    //         groupBy: '',
    //         helpMessage: `修边刀路终止后延伸量，用于解决修边不充分问题。`,
    //     },
    //     {
    //         key: 'modelSplitProcessType',
    //         defaultValue: 0,
    //         label: '加工方式',
    //         groupBy: '',
    //         helpMessage: `方式一(按开料顺序)——按开料顺序逐片加工/开料完；
    //                     方式二(按层加工)——按深度由浅到深逐层加工；
    //                     方式三(未挖穿项就近加工，挖穿项按开料顺序)——未挖穿层按加工项加工至挖穿前，挖穿的层按开料顺序逐片加工。`,
    //     },
    //     {
    //         key: 'cutBoardBorderFaceA',
    //         defaultValue: 0,
    //         label: '正面修边',
    //         groupBy: '',
    //         helpMessage: `正面加工开始前先修边，用于减少大板边缘板件开料阻力造成的跑板问题。`,
    //     },
    //     {
    //         key: 'processType',
    //         defaultValue: 0,
    //         label: '加工方式',
    //         groupBy: '',
    //         helpMessage: `
    // 方式一(按开料顺序)——按开料顺序逐片加工/开料完；
    // 方式二(按层加工)——按深度由浅到深逐层加工；
    // 方式三(未挖穿项就近加工，挖穿项按开料顺序)——未挖穿层按加工项加工至挖穿前，挖穿的层按开料顺序逐片加工。`,
    //     },
    //     {
    //         key: 'helpCutStyle',
    //         defaultValue: 0,
    //         label: '方式',
    //         groupBy: '',
    //         helpMessage: `提示：(同刀辅助开料，使用与板材开料刀一样的刀具；小刀辅助开料，使用比正常开料刀直径小的铣刀进行辅助开料)`,
    //     },
    //     {
    //         key: 'helpCutGap',
    //         defaultValue: 0,
    //         label: '外扩',
    //         groupBy: '',
    //         helpMessage: `提示：(第一次开料刀路外扩偏移距离，单位mm毫米)`,
    //     },
    //     {
    //         key: 'helpCutKnifeDiameterMin',
    //         defaultValue: 0,
    //         label: '辅助开料刀直径范围（下限）',
    //         groupBy: '',
    //         helpMessage: ``,
    //     },
    //     {
    //         key: 'helpCutKnifeDiameterMax',
    //         defaultValue: 0,
    //         label: '辅助开料刀直径范围（上限）',
    //         groupBy: '',
    //         helpMessage: ``,
    //     },
    //     {
    //         key: 'isCutBlockInModelFirst',
    //         defaultValue: false,
    //         label: '挖穿板优先开料',
    //         groupBy: '',
    //         helpMessage: ``,
    //     },


    //     {
    //         key: 'freeHeight',
    //         defaultValue: 20,
    //         label: '垂直安全距离',
    //         groupBy: '',
    //         helpMessage: `提示：(空程移动时或停刀位置，Z坐标点到板面距离，单位mm毫米)。`,
    //     },
    //     {
    //         key: 'freeSpeed',
    //         defaultValue: 10000,
    //         label: '空程速度',
    //         groupBy: '',
    //         helpMessage: `G0或G00指令对应速度，单位mm/min毫米每分钟`,
    //     },




    //     {
    //         key: 'ncLineSuffix',
    //         defaultValue: 0,
    //         label: 'NC每行后缀',
    //         groupBy: '',
    //         helpMessage: `用于添加NC指令行结束符,如：星辉雕刻机，NC指令行需要以分号“;”结束`,
    //     },
    //     {
    //         key: 'preCuttingDepth',
    //         defaultValue: 0,
    //         label: '预切入深度',
    //         groupBy: '',
    //         helpMessage: `提示：(常规造型加工，下刀预切入深度)。`,
    //     },
]




export function convertToLabelValue(obj) {
    if (typeof obj !== 'object' || obj === null)
        throw new Error('参数必须是对象')

    const entries = Object.entries(obj)
    if (entries.length === 0)
        return []

    return entries.map(([label, value]) => ({ label, value }))
}