import { ref } from "vue";
import { FileZip } from "./base/ZipFile";
import { BlockHelper } from "../vo/order/BlockHelper";
import { PlacePositionHelper } from "../vo/order/PlacePostionHelper";
import { PlaceBlock } from "../vo/order/PlaceBlock";
import { BlockSizePlus } from "../vo/order/BlockSizePlus";
import { PlaceMaterial } from "../vo/order/PlaceMaterial";
import { PlaceStore } from "../vo/order/PlaceStore";
import { BlockPlus } from "../vo/order/BlockPlus";
import { cloneDeep } from "lodash-es";
import { PlaceStyle } from "../vo/enums/PlaceStyle";
import { formatToDate } from '@/utils/dateUtil'
import { useMessage } from '@/hooks/web/useMessage'

const { createMessage } = useMessage()
/**输出到文本文件的内容  正面 （A面） */
const outPutString = ref<any>('');
/**输出到文本文件的内容  反面 （B面） */

const outPutStringB = ref<any>('');

//**机台配置的  CNC文件结构 */
const TreeData = ref<any>({})

//** 优化数据 */
const placeData = ref<any>(null);
/** 
 * 记录下所有的加工项
 * 注：不同的加工
 */
const allProcessList = ref<any>([])

//** 机台配置 */
const drillConfig = ref<any>(null);
export async function readyToCreateCNCFile(treeData: any, planOrder: any, zipFile: FileZip, sysConfigDrill?: any) {

  TreeData.value = treeData
  placeData.value = planOrder
  drillConfig.value = sysConfigDrill

  /**
   * 梳理下流程--  关于坐标
   * 1、 小板模式  不考虑大板
   * 那么计算坐标的时候 只要考虑 机台 坐标参数  (靠档和长高方向【送板方向？】)   和 小板的排版方式 placeStyle
   * 
   * 2、大板模式  需要考虑大板
   * 那么计算坐标的时候  要考虑 虑 机台 坐标参数  (靠档和长高方向【送板方向？】) 、 大板的坐标、小板在大板中的坐标   板件是否翻面
   */

  allProcessList.value = [];
  await createCNCFile(zipFile);

  console.log('allProcessItem', allProcessList.value)
  // 测试
  // testToGetDicValue(DataTags_all)
}


function getFileName(val, exportFileType?: any, source?: any, noHandleSource?: any) {
  let valueStr = val.slice(3, -3)
  let res = ''

  let blockList: any = [];
  if (noHandleSource == true) {
    if (exportFileType == 0 || exportFileType == 1) {
      blockList = source
    } else {
      source.forEach(m => {
        m.blockList.forEach(block => {
          blockList.push(block);
        });
      });
    }
  } else {
    if (exportFileType == 0) {
      blockList = [source]
    } else if (exportFileType == 1) {
      blockList = source
    } else if (exportFileType == 2) {
      source.blockList.forEach(block => {
        blockList.push(block);
      });
    } else if (exportFileType == 3) {
      source.forEach(m => {
        m.blockList.forEach(block => {
          blockList.push(block);
        });
      });
    }
  }


  switch (valueStr) {
    case 'organName':

      res = 'organName' // zipFileNameStr.replace(val, placeData.value.source.planOrder.organName)
      break;

    case 'dealer':

      res = placeData.value.source.orderList.map(e => e.dealer)
      break;
    case 'customer':

      res = placeData.value.source.orderList.map(e => e.customer).join('、')
      break;
    case 'address':

      res = placeData.value.source.orderList.map(e => e.address).join('、')
      break;
    case 'orderId':

      res = placeData.value.source.orderList.map(e => e.orderId).join('、')
      break;
    case 'customOrderNo':

      res = placeData.value.source.orderList.map(e => e.customOrderNo).join('、')
      break;
    case 'planCode':

      res = placeData.value.source.planOrder.planCode
      break;
    case 'goodsName':

      res = blockList.map(e => e.goodsName).join('、')
      break;
    case 'material':

      res = blockList.map(e => e.materialName).join('、')
      break;
    case 'color':

      res = blockList.map(e => e.color).join('、')
      break;
    case 'spec':

      res = blockList.map(e => e.spec).join('、')
      break;
    case 'blockNo':

      res = blockList.map(e => e.blockNo).join('、')
      break;
    case 'blockName':

      res = blockList.map(e => e.bodyName).join('、')
      break;
    default:
      break;
  }

  return res
}
function truncateString(str, maxLength, useEllipsis = true) {
  if (str.length <= maxLength) {
    return str;
  } else {
    return str.slice(0, maxLength) + (useEllipsis ? '...' : '');
  }
}
async function createCNCFile(zipFile: FileZip) {


  /**
   * 生成CNC 文件有4种情况 
   * 小板独立文件、小板合并文件、大板独立文件、大板合并文件  0 || 1 || 2 || 3
   * blockAlone 、 blockMerge 、 boardAlone 、 boardMerge
   * 默认 小板独立文件
   *   merge alone
   *  */

  // 测试

  let exportFileType = drillConfig.value.fileExportType || drillConfig.value?.machineSettingDO?.ncExportConfigsRef?.fileExportType || 0


  // 测试用 读取所有字典的字段看看返回的内容
  // testToGetDicValue(DataTags_all)
  let fileRes: any = []
  /**  是否分面 */
  const isFileSort = drillConfig.value?.isFileSort



  /** 数据预处理---把每个孔 || 造型的刀 解析下做个关联 后面数据解析的时候减少计算量*/

  // 已经这之前已经处理 孔和 造型的数据内添加了 knife
  let blockList: PlaceBlock[] = []
  placeData.value.materialList.forEach(material => {
    material.boardList.forEach(board => {
      board.blockList.forEach(block => {
        blockList.push(block)
      });
    });
  })

  if (blockList.length == 0 && placeData.value.blockList.length != 0) {
    placeData.value.materialList.forEach(material => {
      material.blockList.forEach(block => {
        blockList.push(block)
      });
    })
  }

  /** 加工方案的需求下  没有用 */
  //**导出空单(无板) */
  let isExportOrderNoBoard = drillConfig.value?.isExportOrderNoBoard || false
  /** 导出空单(无加工） */
  let isExportOrderNoProcess = drillConfig.value?.isExportOrderNoProcess || false
  /** 导出已加工 */
  let isExportOrderProcess = drillConfig.value?.isExportOrderProcess || false
  switch (exportFileType) {
    case 0:
      // 小板独立文件  生成文件的 基准点为 具体的某个小板
      for (let index = 0; index < blockList.length; index++) {
        const block = blockList[index];
        // 是否需要双面加工

        let isTwoFace = block.blockDetail.isTwoFaceProcess
        let hasFaceA = block.blockDetail.holeListFaceA.length > 0 || block.blockDetail.modelListFaceA.length > 0
        let hasFaceB = block.blockDetail.holeListFaceB.length > 0 || block.blockDetail.modelListFaceB.length > 0

        // 若AB 面 都没有加工 且 有侧面加工 的情况下  都在A面加工
        if (!hasFaceA && !hasFaceB && (block.blockDetail.holeListSide.length > 0 || block.blockDetail.modelListSide.length > 0)) {
          hasFaceA = true
        }

        outPutString.value = ''
        outPutStringB.value = ''

        await _setDataSource(block, exportFileType)
        console.log('板', block.blockNo, '已设置数据源', fileDataAB.value)
        await createCNC(TreeData.value, 0, block, exportFileType, -1, 'start');


        // 设置 zip压缩包名称 start
        var zipFileNameStr = drillConfig.value?.zipPathName || drillConfig.value?.machineSettingDO?.ncExportConfigsRef.zipPathName || ''

        var zipFileNameConfigList = zipFileNameStr.match(/{{\<([^>]*)>\}}/g) || []

        zipFileNameConfigList.forEach(val => {

          var newVal = getFileName(val, exportFileType, block);

          zipFileNameStr = zipFileNameStr.replaceAll(val, newVal)
        })

        var zipFileName = zipFileNameStr //`${zipFileNameStr}.zip`
        await zipFile.SetZipFileName(zipFileName)
        // 设置 zip压缩包名称 end 


        const codeStr = drillConfig.value?.ncFileEncoding || ''
        const fileContent = outPutString.value



        if (isFileSort) {

          // 需要双面加工的时候 才导出
          if (hasFaceB && drillConfig.value?.isExportBoardFileB) {
            var fileNameB = drillConfig.value?.fileBackPathName || ''
            var fileNameConfigList_B = fileNameB.match(/{{\<([^>]*)>\}}/g)
            fileNameConfigList_B && fileNameConfigList_B.forEach(val => {

              var newVal = getFileName(val, exportFileType, block);
              fileNameB = fileNameB.replaceAll(val, newVal)
            })

            let nameB = fileNameB

            let fileBContent = outPutStringB.value;
            await zipFile.PushFile(`${nameB}`, fileBContent, false, codeStr)
          } else {
            console.log('不添加文件~！B面')
          }

          if (hasFaceA && drillConfig.value?.isExportBoardFileA) {
            var fileNameA = drillConfig.value?.fileFrontPathName || ''

            var fileNameConfigList = fileNameA.match(/{{\<([^>]*)>\}}/g)

            fileNameConfigList && fileNameConfigList.forEach(val => {
              var newVal = getFileName(val, exportFileType, block);
              fileNameA = fileNameA.replaceAll(val, newVal)
            })

            let nameA = fileNameA
            await zipFile.PushFile(`${nameA}`, fileContent, false, codeStr)
          } else {
            console.log('不添加文件~！A面')
          }


        } else {
          var fileName = drillConfig.value?.filePathName || ''

          var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)

          fileNameConfigList && fileNameConfigList.forEach(val => {
            var newVal = getFileName(val, exportFileType, block);
            fileName = fileName.replaceAll(val, newVal)
          })

          let name = `${fileName}`;
          await zipFile.PushFile(`${name}`, fileContent, false, codeStr)
        }


      }
      break;
    case 1:
      // 小板合并文件  生成文件的 基准点为 小板的数据集
      outPutString.value = ''
      outPutStringB.value = ''

      await _setDataSource(blockList, exportFileType)
      console.log('小板合并文件模式：生成开始', outPutString.value);
      await createCNC(TreeData.value, 0, blockList, exportFileType, -1, 'start');
      console.log('小板合并文件模式：生成结束', outPutString.value);


      // 设置 zip压缩包名称 start
      var zipFileNameStr = drillConfig.value?.zipPathName || drillConfig.value?.machineSettingDO?.ncExportConfigsRef.zipPathName

      var zipFileNameConfigList = zipFileNameStr.match(/{{\<([^>]*)>\}}/g) || []

      zipFileNameConfigList.forEach(val => {

        var newVal = getFileName(val, exportFileType, blockList);
        zipFileNameStr = zipFileNameStr.replaceAll(val, newVal)
      })




      var zipFileName = zipFileNameStr
      await zipFile.SetZipFileName(zipFileName)
      // 设置 zip压缩包名称 end 





      var fileName = drillConfig.value?.fileFrontPathName || ''

      var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)
      fileNameConfigList.forEach(val => {

        var newVal = getFileName(val, exportFileType, placeData.value.blockList);
        fileName = fileName.replaceAll(val, newVal)
      })


      var fileContent = '';

      fileContent = outPutString.value

      if (isFileSort) {
        fileContent += '\n'
        fileContent += '\n'

        fileContent += outPutStringB.value
      }

      var codeStr = drillConfig.value?.ncFileEncoding || ''



      await zipFile.PushFile(`${fileName}`, fileContent, false, codeStr)

      break;
    case 2:
      // 大板独立文件    生成文件的 基准点为 具体的某个大板

      for (let index = 0; index < placeData.value.materialList.length; index++) {
        let material = placeData.value.materialList[index];

        for (let index1 = 0; index1 < material.boardList.length; index1++) {
          const board = material.boardList[index1]
          outPutString.value = ''
          outPutStringB.value = ''
          await _setDataSource(material, exportFileType)
          console.log('大板独立文件模式：生成开始', outPutString.value);
          await createCNC(TreeData.value, 0, material, exportFileType, -1, 'start');
          console.log('大板独立文件模式：生成结束', outPutString.value);


          // 设置 zip压缩包名称 start
          var zipFileNameStr = drillConfig.value?.zipPathName || drillConfig.value?.machineSettingDO?.ncExportConfigsRef.zipPathName

          var zipFileNameConfigList = zipFileNameStr.match(/{{\<([^>]*)>\}}/g) || []

          zipFileNameConfigList.forEach(val => {

            var newVal = getFileName(val, exportFileType, material);
            zipFileNameStr = zipFileNameStr.replaceAll(val, newVal)
          })




          var zipFileName = zipFileNameStr //`${zipFileNameStr}.zip`
          await zipFile.SetZipFileName(zipFileName)
          // 设置 zip压缩包名称 end 
          var fileName = drillConfig.value?.fileFrontPathName || ''

          var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)

          fileNameConfigList && fileNameConfigList.forEach(val => {
            var newVal = getFileName(val, exportFileType, material);
            fileName = fileName.replaceAll(val, newVal)
          })



          let name = `${fileName}`;


          const fileContent = outPutString.value
          const codeStr = drillConfig.value?.ncFileEncoding || ''
          let isTwoFace = board.isTwoFaceProcessing
          fileNameConfigList_B && fileNameConfigList_B.forEach(val => {

            var newVal = getFileName(val, exportFileType, material);
            fileNameB = fileNameB.replaceAll(val, newVal)
          })
          if (isFileSort) {
            var fileNameB = drillConfig.value?.fileBackPathName || ''
            var fileNameConfigList_B = fileNameB.match(/{{\<([^>]*)>\}}/g)
            // 需要双面加工的时候 才导出

            if (isTwoFace) {
              let nameB = fileNameB
              let fileBContent = outPutStringB.value;
              await zipFile.PushFile(`${nameB}`, fileBContent, false, codeStr)
            }

            let nameA = fileName
            await zipFile.PushFile(`${nameA}`, fileContent, false, codeStr)

          } else {
            await zipFile.PushFile(`${name}`, fileContent, false, codeStr)
          }


        }

      }
      break;
    case 3:
      // 大板合并文件  生成文件的 基准点为 大板的数据集

      outPutString.value = ''
      outPutStringB.value = ''
      await _setDataSource(placeData.value.materialList, exportFileType)
      console.log('大板合并文件模式：生成开始', outPutString.value);
      await createCNC(TreeData.value, 0, placeData.value.materialList, exportFileType, -1, 'start');
      console.log('大板合并文件模式：生成结束', outPutString.value);


      // 设置 zip压缩包名称 start
      var zipFileNameStr = drillConfig.value?.zipPathName || drillConfig.value?.machineSettingDO?.ncExportConfigsRef.zipPathName

      var zipFileNameConfigList = zipFileNameStr.match(/{{\<([^>]*)>\}}/g) || []

      zipFileNameConfigList && zipFileNameConfigList.forEach(val => {

        var newVal = getFileName(val, zipFileNameStr, exportFileType, placeData.value.materialList);
        zipFileNameStr = zipFileNameStr.replaceAll(val, newVal)
      })




      var zipFileName: any = `${zipFileNameStr}.zip`
      await zipFile.SetZipFileName(zipFileName)
      // 设置 zip压缩包名称 end 

      var fileName = drillConfig.value?.fileFrontPathName || ''
      var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)
      fileNameConfigList.forEach(val => {

        var newVal = getFileName(val, fileName, exportFileType, placeData.value.materialList);
        fileName = fileName.replaceAll(val, newVal)
      })

      var fileContent: string = outPutString.value
      if (isFileSort) {
        fileContent += '\n'
        fileContent += '\n'

        fileContent += outPutStringB.value
      }

      var codeStr = drillConfig.value?.ncFileEncoding || ''
      await zipFile.PushFile(`${fileName}`, fileContent, false, codeStr)
      break;
    default:
      break;
  }


  return fileRes





  // textFile.saveFile(`cncXML测试.xml`, outPutString.value)
}


function testToGetDicValue(tags) {
  let resTags: any = []
  // console.log(placeData)

  tags.forEach(tag => {
    let temp: any = {
      // sourceStr: item,  // 在源数据中的字符串
      // valueStr: valueStr,  // 剔除 包裹的字符的字符串
      type: tag.flag,    // 数据类型
      itemKey: tag.value,    // 数据id
      itemLabel: tag.label, // 数据说明
      // value: '',  // 要替换的数据 
      tagRes: null,
      tagResB: null,   // 拿回来的 B面数据
      tagResA: null   // 拿回来的 A面数据
    }

    // 分别对应4中文件导出模式
    let res1 = { ...temp }
    let res2 = { ...temp }
    let res3 = { ...temp }
    let res4 = { ...temp }
    // 1getPlaceDataValueByDic(res1, [placeData.value.blockList[0]], 0)
    // 1getPlaceDataValueByDic(res2, placeData.value.blockList, 1)
    // 1getPlaceDataValueByDic(res3, [placeData.value.materialList[0]], 2)
    // 1getPlaceDataValueByDic(res4, placeData.value.materialList, 3)

    // console.log(res1, res2, res3, res4);

    // temp.tagRes = tagRes;

    resTags.push(temp);
  });
}


// 
const hasConfig = ref(false);

// 生成一个CNC文件的内容 
/**
 * 
 * @param data  // 模板数据
 * @param level // 层级
 * @param targetSource  // 实际的基准点数据   目前可能为  block || blockList || material || materialList
 * @param exportFileType // 当前文件的生成模式 0 || 1 || 2 || 3     1 和 3  合并模式   0 和 2 独立模式
 * @param parantId  // 递归时 定位父级下标用  入口时传 -1
 * @param flag // 用来分辨 AB面 等等
 * 
 * outPutString 和 outPutStringB 操作说明
 * 若 exportFileType 为 0 或者 2 的时候  
 * 
 * 
 * 
 */

// exportFileType 为 0 的时候 不管 
function createCNC(data, level?: any, targetSource?: any, exportFileType?: any, parantId?: any, flag?: any, kidId?: any) {
  // console.log('runing',level,parantId,flag)
  //  这里都整合为 数组的形式处理
  let targetSourceList: any = [];
  if (Array.isArray(targetSource)) {
    targetSourceList = targetSource
  } else {
    targetSourceList = [targetSource]
  }
  /** 钻孔机 配置中 是否分面 */
  const isFileSort = drillConfig.value?.isFileSort

  /** 实际的板材 是否 需要分面  解析的时候可能用 */
  let isTwoFaceList = [];

  // 是否有A面加工
  let hasFaceAList: Array<boolean> = [];

  // 是否有B面加工  
  let hasFaceBList: Array<boolean> = []



  // 用来定位  板内的某个子集 （轮廓、造型、孔、等等）
  let pid = -1

  /** 导出的是是A面 还是B面 */
  let _flag = ''

  // 逻辑控制： 第一次 传入的时候 都为start 
  if (flag != undefined) {
    _flag = flag
  }

  if (parantId != undefined) {
    pid = parantId
  }
  // 用来定位  板内的某个子集的子集 （造型轮廓、造型、孔、等等）
  let kid = -1

  if (kidId != undefined) {
    kid = kidId
  }


  let lvl = level;


  if (Array.isArray(data)) {
    // 
    data.forEach(async (item, d) => {

      // 测试用
      let exportFileType = drillConfig.value.fileExportType || drillConfig.value?.machineSettingDO?.ncExportConfigsRef?.fileExportType || 0
      const ccc = { _flag, title: item.title, lvl, pid, kid, d, exportFileType }
      console.log('开始=》', ccc)




      // d:  本节点 在父节点的 层级 （）

      const {
        dataTextarea,
        dataTextarea_end,
        dataTextarea_start,
        isTab,
        isLoop,
        LoopOrderBy,
        isUsedCondition,
        loopOrderByOptions,
        conditionList,

      } = item.item
      let isUsedChangeKnife = item.item.isUsedChangeKnife || false
      /**判断 刀路是否条件显示  刀路的条件显示  生效的逻辑不一样 与 正反面 和是否调刀 逻辑不一样
       * conditionList 内 包含 2 3 4 则为是
       */
      let isCondition_Line = false
      if (isUsedCondition && Array.isArray(conditionList)) {
        for (const conditionItem of conditionList) {
          if (conditionItem.conditionOptions.includes('2') || conditionItem.conditionOptions.includes('3') || conditionItem.conditionOptions.includes('4')) {
            isCondition_Line = true
            continue
          }
        }
      }
      let tabContent = `    `
      let tabVal = '';
      if (isTab) {
        for (let i = 0; i < lvl; i++) {
          tabVal = tabVal + tabContent;
        }
      }

      // 标识符 
      let nextRow = '\n'
      // 头部

      // 主体内容
      /**
       * 关键的地方
       * 逻辑分析
       * 1、 解析当前节点 所有的数据项
       * 2、 判断当前节点的数据项 的 循环依赖项
       * 3、 根据循环 依赖性  和 数据项  判断 是否循环
       */

      // 当前节点文本
      let dataNode = dataTextarea || ''

      // 当前节点对应的  数据项
      // let dataItemList = dataNode.match(/{{\<([^>]*)>\}}/g)
      let dataItemList = []

      if (loopOrderByOptions) {
        dataItemList = loopOrderByOptions.map(e => e.value)
      }

      /** 该节点所有的数据项集合 */
      let dataObjectList: any = []

      /**
       * 注：解决翻面问题呢
       * 解决方案：解析数据的时候将 A B面的数据 都解析出来 
       * 然后在替换文件内容的时候 根据对应的 A || B  面进行解析替换 
       */


      // 当前节点循环的依据
      let loopObj: any = null

      if (isLoop == true) {

        let valueStr = ''
        try {
          valueStr = LoopOrderBy.slice(3, -3) || ''
        } catch (error) {
          console.log(error, LoopOrderBy);
          createMessage.warn(`节点${ccc.title}配置了循环，但未配置循环依据,请前往机台配置后再试！`)
        }

        let valueList = valueStr.split('|')
        if (valueList.length == 3) {
          let type = valueList[0];
          let itemKey = valueList[1];
          let itemLabel = valueList[2];
          loopObj = {
            sourceStr: LoopOrderBy,  // 在源数据中的字符串
            valueStr: valueStr,  // 剔除 包裹的字符的字符串
            type,    // 数据类型
            itemKey,    // 数据id
            itemLabel, // 数据说明
            value: '',  // 要替换的数据 
            tagRes: null,     // 拿回来的 数据
            tagResB: null,   // 拿回来的 B面数据
            tagResA: null,   // 拿回来的 A面数据
            isTwoFaceList,   // 暂时没用
            hasFaceAList,    // 暂时没用
            hasFaceBList     // 暂时没用
          }
          getPlaceDataValueByDic(loopObj, targetSourceList, exportFileType)

        }
      }



      // 解析数据项  所有数据项都解析出来
      dataItemList && dataItemList.forEach(item => {
        let valueStr = item.slice(3, -3)
        let valueList = valueStr.split('|')
        // 既定的数据 格式为 <<{ 数据类型 | 数据ID | 数据说明  }>> 不符合的不操作
        if (valueList.length == 3) {
          let type = valueList[0];
          let itemKey = valueList[1];
          let itemLabel = valueList[2];

          // if (valueStr.includes('grooves|toolDiameter|刀直径')) {
          //   
          // }
          let temp = {
            sourceStr: item,  // 在源数据中的字符串
            valueStr: valueStr,  // 剔除 包裹的字符的字符串
            type,    // 数据类型
            itemKey,    // 数据id
            itemLabel, // 数据说明
            value: '',  // 要替换的数据 
            tagRes: '',    //  从字典拿回来的数据可能要二次操作  留个位置   以防后续改动
            tagResB: null,   // 拿回来的 B面数据
            tagResA: null   // 拿回来的 A面数据
          }
          getPlaceDataValueByDic(temp, targetSourceList, exportFileType)
          dataObjectList.push(temp);
        }
      });
      //**  不同的面 循环的依据是不一样的 所以要分开写  tagRes tagResA  tagResB */
      if (isFileSort) {
        // A面
        if (_flag == 'start' || _flag == 'A') {
          if (isUsedChangeKnife == true) {
            hasConfig.value = true
          }
          // A面 头部补充内容
          if (dataTextarea_start) {
            outPutString.value = outPutString.value + (dataTextarea_start || '') + nextRow
          }
          // 判断这个节点是否循环：   判断数据的循环依赖 是否可以循环  可以的话节点进行循环  不可以直接输出
          if (isLoop && loopObj && loopObj.tagResA && Array.isArray(loopObj.tagResA)) {
            if (loopObj.tagResA.length > 0) {
              //轮廓 或者点阵等等 这一层级    会走这里
              const target = dataNode;
              let loopMore = false
              /**
               * loopObj.tagResA  板
               * loopObj.tagResA[pid] 某个造型 | 孔 
               * loopObj.tagResA[pid][i]  造型下的点
               */

              // 判断是否需要三级循环  针对 点阵 轮廓
              loopObj.tagResA.forEach(item => {
                if (Array.isArray(item)) {
                  loopMore = true
                }
              })
              let loopItem = loopObj.tagResA[pid]
              if (kid != -1 && kid != null) {
                loopItem = loopItem[kid]
              }
              if (loopMore && Array.isArray(loopItem) && pid != null && pid != -1) {
                // 需要循环的节点
                //** 到了这里 说明是 类似点阵 轮廓等信息 loopObj.tagRes 为 多维数组的时候   */ 

                loopItem.forEach((noUsed, i) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置 
                  let onceLoopVal = target
                  let changeKnifeCode: any = ''
                  // 换刀逻辑
                  if (isUsedChangeKnife) {
                    changeKnifeCode = getChangeKnifeCode(ccc, loopObj, i, isCondition_Line)

                    // onceLoopVal = changeKnifeCode + onceLoopVal
                  }

                  // 条件显示
                  if (isUsedCondition && isCondition_Line == false) {
                    let conditionInfo = checkCondition(conditionList, ccc, loopObj, i, isCondition_Line)

                    if (conditionInfo) {
                      onceLoopVal = conditionInfo.dataTextarea
                      // if (changeKnifeCode != '') {
                      //   onceLoopVal = changeKnifeCode + conditionInfo.dataTextarea
                      // }
                    } else {
                      // onceLoopVal = ''
                    }
                  }

                  onceLoopVal = changeKnifeCode + onceLoopVal

                  // 是否是三维数组   造型轮廓的时候
                  let isLev3Loop = false; 
                  let lev3LoopList: any = [];
                  
                  
                  // 替换操作 
                  dataObjectList.forEach((dataObj, q) => {
                    // 从字典拿回来的数据可能要二次操作  留个位置
                    //  二维数组
                    let tempVal = dataObj.tagResA
                    let tempDataObj = dataObj
                    // 绝大多数的返回数据  这里控制的返回都是 数组
                    // 这里是不循环的输出结构即可  A面
                    let loopOrderByList = loopObj.tagResA[pid]
                    let loopTempVal = null
                    try {
                      if (Array.isArray(tempVal)) {
                        loopTempVal = tempVal[pid]
                      } else {
                        loopTempVal = tempVal
                      }
                    } catch (error) {

                    }

                    if (kid != -1 && Array.isArray(loopTempVal)) {
                      loopTempVal = loopTempVal[kid]
                      loopOrderByList = loopOrderByList[kid]
                    }

                    if (tempVal && Array.isArray(tempVal[pid])) {
                      // let checkVal

                      // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                      if (loopTempVal.length == loopOrderByList.length) {

                        /** 
                         这里就还需要判断一层  
                         如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                         tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                         这个时候  这个循环的基准 应该 再下一级 ？？
                         
                         解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环  
  
                         
                        */


                        tempDataObj.value = loopTempVal[i] // tempDataObj.tagResA[pid][i]



                      } else if (tempVal.length == 1) {
                        // 若是只有一个项直接输出就好

                        tempDataObj.value = tempVal.join('、')
                      } else {
                        // 其它情况 直接输出

                        tempDataObj.value = tempVal.join('、')
                      }

                    } else {
                      // 其它情况 直接输出
                      tempDataObj.value = tempVal
                    }

                    // 真正的替换内容
                    if (tempDataObj.value && Array.isArray(tempDataObj.value)) {
                      isLev3Loop = true

                      if (lev3LoopList.length == tempDataObj.value.length && lev3LoopList.length) {

                        tempDataObj.value.forEach((val, j) => {
                          let temp = lev3LoopList[j]
                          let tempVal = temp.replaceAll(tempDataObj.sourceStr, val)
                          lev3LoopList[j] = tempVal
                        });
                      } else {
                        tempDataObj.value.forEach(val => {
                          let temp = onceLoopVal
                          let tempVal = temp.replaceAll(tempDataObj.sourceStr, val)
                          lev3LoopList.push(tempVal);
                        });
                      }

                    } else {
                      if (isUsedCondition && isCondition_Line == true && q == 0) {

                        let _ccc = { ...ccc, q }
                        let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                        if (conditionInfo) {
                          onceLoopVal = conditionInfo.dataTextarea
                        } else {
                          onceLoopVal = ''
                        }
                      }
                      onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                    }
                  });
                  //  出了循环了
                  if (isLev3Loop) {
                    // dataNode = onceLoopVal
                    lev3LoopList.forEach((str) => {
                      dataNode = str

                      // 判断是否缩进

                      if (isTab) {
                        outPutString.value += tabVal;
                        dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                      }
                      console.log('ccc 1 A', ccc)
                     
                      outPutString.value = outPutString.value + dataNode + nextRow
                      // 标签 子节点的内容  
                      if (item.children && Array.isArray(item.children)) {

                        createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'A', i)
                      }
                      if (dataTextarea_end) {
                        let endStr = dataTextarea_end || ''
                        if (isTab) {
                          outPutString.value += tabVal;
                          endStr = endStr.replaceAll('\n', tabVal + '\n')
                        }
                        outPutString.value = outPutString.value + (endStr || '') + nextRow
                      }
                    });
                  } else {
                    dataNode = onceLoopVal
                    // 判断是否缩进
                    if (isTab) {
                      outPutString.value += tabVal;
                      dataNode = dataNode.replaceAll('\n', tabVal + '\n')
                    }

                    console.log('ccc 2 A', ccc)
                    // console.log('test:',i,outPutString.value)
                    // outPutString.value += `// test holes ${i} `


                    outPutString.value = outPutString.value + dataNode + nextRow
                    // 标签 子节点的内容
                    if (item.children && Array.isArray(item.children)) {

                      createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'A', i)
                    }
                    if (dataTextarea_end) {
                      let endStr = dataTextarea_end || ''
                      if (isTab) {
                        outPutString.value += tabVal;
                        endStr = endStr.replaceAll('\n', '\n' + tabVal)
                      }
                      outPutString.value = outPutString.value + (endStr || '') + nextRow
                    }
                  }


                })


              } else {

                // 走到这里 一般是 板件信息 或者订单信息
                loopObj.tagResA.forEach((noUsed, i) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置 
                  let onceLoopVal = target
                  // 替换操作 
                  dataObjectList.forEach((dataObj, q) => {
                    // 从字典拿回来的数据可能要二次操作  留个位置
                    let tempVal = dataObj.tagResA
                    let tempDataObj = dataObj
                    // 绝大多数的返回数据  这里控制的返回都是 数组
                    // 这里是不循环的输出结构即可
                    if (Array.isArray(tempVal)) {
                      // let checkVal

                      // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                      if (tempVal.length == loopObj.tagResA.length) {

                        /** 
                         这里就还需要判断一层  
                         如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                         tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                         这个时候  这个循环的基准 应该 再下一级 ？？
                         
                         解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环  
                        */

                        tempDataObj.value = tempDataObj.tagResA[i]

                      } else if (tempVal.length == 1) {
                        // 若是只有一个项直接输出就好

                        tempDataObj.value = tempVal.join('、')
                      } else {
                        // 其它情况 直接输出

                        tempDataObj.value = tempVal.join('、')
                      }

                    } else {
                      // 其它情况 直接输出
                      tempDataObj.value = tempVal
                    }
                    // 真正的替换内容
                    if (isUsedCondition && isCondition_Line == true && q == 0) {
                      let _ccc = { ...ccc, q }
                      let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                      if (conditionInfo) {
                        onceLoopVal = conditionInfo.dataTextarea
                      } else {
                        onceLoopVal = ''
                      }
                    }
                    onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                  });
                  dataNode = onceLoopVal
                  // 判断是否缩进 
                  if (isTab) {
                    outPutString.value += tabVal;
                    dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                  }
                  console.log('ccc 3 A', ccc)
                  outPutString.value = outPutString.value + dataNode + nextRow
                  // 标签 子节点的内容
                  if (item.children && Array.isArray(item.children)) {

                    createCNC(item.children, lvl + 1, targetSource, exportFileType, i, 'A')
                  }

                  if (dataTextarea_end) {
                    let endStr = dataTextarea_end || ''
                    if (isTab) {
                      outPutString.value += tabVal;
                      endStr = endStr.replaceAll('\n', '\n' + tabVal)
                    }
                    outPutString.value = outPutString.value + (endStr || '') + nextRow
                  }
                });
              }
            }

          } else {
            if (isTab) {
              outPutString.value += tabVal;
              dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
            }

            dataObjectList.forEach(dataObj => {
              // 从字典拿回来的数据可能要二次操作  留个位置
              // 绝大多数的返回数据  这里控制的返回都是 数组
              // 这里是不循环的输出结构即可

              let tempVal = dataObj.tagResA

              if (Array.isArray(tempVal)) {
                if (pid != null) {
                  let target = tempVal[pid]
                  if (kid != -1) {
                    target = target[kid]
                  }
                  dataObj.value = target  //tempVal[pid]
                } else {
                  dataObj.value = tempVal.join('、')
                }

              } else {
                dataObj.value = tempVal
              }


              dataNode = dataNode.replaceAll(dataObj.sourceStr, dataObj.value)


            });
            console.log('ccc 4 A', ccc, fileDataA.value)
            outPutString.value = outPutString.value + dataNode + nextRow

            // 标签头部 结束


            // 标签 子节点的内容 A 面 不循环 节点
            if (item.children && Array.isArray(item.children)) {

              createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'A')
            }
            // 标签尾部
            if (dataTextarea_end) {
              let endStr = dataTextarea_end || ''
              if (isTab) {
                outPutString.value += tabVal;
                endStr = endStr.replaceAll('\n', '\n' + tabVal)
              }
              outPutString.value = outPutString.value + (endStr || '') + nextRow
            }
          }
        }
        if (_flag == 'start' && hasConfig.value == true) {
          // 加工完成  收刀

          // 收刀
          hasConfig.value = false
          if (allProcessList.value && allProcessList.value[allProcessList.value.length - 1]) {
            let finalWorkKnife = allProcessList.value[allProcessList.value.length - 1].processItem.knife
            outPutString.value += finalWorkKnife.axisStopCode + finalWorkKnife.knifeStopCode
          }

        }

        // B面 
        if (_flag == 'start' || _flag == 'B') {
          if (isUsedCondition == true) {
            hasConfig.value = true
          }
          // B面  头部补充内容
          if (dataTextarea_start) {

            outPutStringB.value = outPutStringB.value + (dataTextarea_start || '') + nextRow

          }
          // 判断数据的循环依赖 是否可以循环   可以的话节点进行循环  不可以直接输出
          if (isLoop && loopObj && loopObj.tagResB && Array.isArray(loopObj.tagResB)) {
            if (loopObj.tagResB.length > 0) {
              //轮廓 或者点阵等等 这一层级    会走这里
              const target = dataNode;

              let loopMore = false

              // 判断是否需要三级循环  针对 点阵 轮廓
              loopObj.tagResB.forEach(item => {
                if (Array.isArray(item)) {
                  loopMore = true
                }
              })
              let loopItem = loopObj.tagResB[pid]
              if (kid != -1 && kid != null) {
                loopItem = loopItem[kid]
              }

              if (loopMore && Array.isArray(loopItem) && pid != null && pid != -1) {
                //** 到了这里 说明是 类似点阵 轮廓等信息 loopObj.tagRes 为 多维数组的时候  循环的基准点就变了 */ 

                loopItem.forEach((noUsed, i) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置 
                  let onceLoopVal = target
                  let changeKnifeCode: any = ''
                  // 换刀逻辑
                  if (isUsedChangeKnife) {
                    changeKnifeCode = getChangeKnifeCode(ccc, loopObj, i, isCondition_Line)

                    onceLoopVal = changeKnifeCode + onceLoopVal
                  }

                  // 条件显示
                  if (isUsedCondition && isCondition_Line == false) {
                    let conditionInfo = checkCondition(conditionList, ccc, loopObj, i, isCondition_Line)

                    if (conditionInfo) {
                      onceLoopVal = conditionInfo.dataTextarea
                      // if (changeKnifeCode != '') {
                      //   onceLoopVal = changeKnifeCode + conditionInfo.dataTextarea
                      // }
                    } else {
                      onceLoopVal = ''
                    }
                  }
                  onceLoopVal = changeKnifeCode + onceLoopVal


                  let isLev3Loop = false; // 是否时三维数组   造型轮廓的时候
                  let lev3LoopList: any = [];
                  // 替换操作 
                  dataObjectList.forEach((dataObj, q) => {
                    // 从字典拿回来的数据可能要二次操作  留个位置
                    let tempVal = dataObj.tagResB
                    let tempDataObj = dataObj
                    // 绝大多数的返回数据  这里控制的返回都是 数组
                    // 这里是不循环的输出结构即可 B面
                    let loopOrderByList = loopObj.tagResB[pid]
                    let loopTempVal: any = null

                    try {
                      if (Array.isArray(tempVal)) {
                        loopTempVal = tempVal[pid]
                      } else {
                        loopTempVal = tempVal
                      }
                    } catch (error) {

                    }

                    if (kid != -1 && Array.isArray(loopTempVal)) {
                      loopTempVal = loopTempVal[kid]
                      loopOrderByList = loopOrderByList[kid]
                    }
                    // if (tempVal == null) {
                    //   loopTempVal = ''
                    // } else {
                    //   loopTempVal = tempVal[pid]
                    // }

                    // if (kid != -1) {
                    //   loopTempVal = loopTempVal[kid]
                    //   loopOrderByList = loopOrderByList[kid]
                    // }

                    if (tempVal && Array.isArray(tempVal[pid])) {
                      // let checkVal

                      // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                      if (loopTempVal.length == loopOrderByList.length) {

                        /** 
                         这里就还需要判断一层  
                         如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                         tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                         这个时候  这个循环的基准 应该 再下一级 ？？
                         
                         解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环  
                        */

                        tempDataObj.value = loopTempVal[i] // tempDataObj.tagResB[pid][i]

                      } else if (tempVal.length == 1) {
                        // 若是只有一个项直接输出就好

                        tempDataObj.value = tempVal.join('、')
                      } else {
                        // 其它情况 直接输出

                        tempDataObj.value = tempVal.join('、')
                      }

                    } else {
                      // 其它情况 直接输出
                      tempDataObj.value = tempVal
                    }


                    // 真正的替换内容
                    if (tempDataObj.value && Array.isArray(tempDataObj.value)) {
                      isLev3Loop = true

                      if (lev3LoopList.length == tempDataObj.value.length) {
                        tempDataObj.value.forEach((val, j) => {
                          let temp = lev3LoopList[j]

                          let replaceVal = val || '';
                          let tempVal = temp.replaceAll(tempDataObj.sourceStr, replaceVal)
                          lev3LoopList[j] = tempVal
                        });
                      } else {
                        tempDataObj.value.forEach(val => {
                          let temp = onceLoopVal
                          let tempVal = temp.replaceAll(tempDataObj.sourceStr, val)
                          lev3LoopList.push(tempVal);
                        });
                      }

                    } else {
                      if (isUsedCondition && isCondition_Line == true && q == 0) {
                        let _ccc = { ...ccc, q }
                        let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                        if (conditionInfo) {
                          onceLoopVal = conditionInfo.dataTextarea
                        } else {
                          onceLoopVal = ''
                        }
                      }
                      onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                    }
                  });
                  if (isLev3Loop) {
                    lev3LoopList.forEach(str => {
                      dataNode = str
                      // 判断是否缩进

                      if (isTab) {
                        outPutStringB.value += tabVal;
                        dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                      }
                      console.log('ccc 1 B', ccc)
                      outPutStringB.value = outPutStringB.value + dataNode + nextRow
                      // 标签 子节点的内容
                      if (item.children && Array.isArray(item.children)) {

                        createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'B', i)
                      }
                      if (dataTextarea_end) {
                        let endStr = dataTextarea_end || ''
                        if (isTab) {
                          outPutStringB.value += tabVal;
                          endStr = endStr.replaceAll('\n', '\n' + tabVal)
                        }
                        outPutStringB.value = outPutStringB.value + (endStr || '') + nextRow
                      }
                    })
                  } else {
                    dataNode = onceLoopVal
                    // 判断是否缩进

                    if (isTab) {
                      outPutStringB.value += tabVal;
                      dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                    }
                    console.log('ccc 2 B', ccc)
                    outPutStringB.value = outPutStringB.value + dataNode + nextRow
                    // 标签 子节点的内容
                    if (item.children && Array.isArray(item.children)) {

                      createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'B', i)
                    }
                    if (dataTextarea_end) {
                      let endStr = dataTextarea_end || ''
                      if (isTab) {
                        outPutStringB.value += tabVal;
                        endStr = endStr.replaceAll('\n', '\n' + tabVal)
                      }
                      outPutStringB.value = outPutStringB.value + (endStr || '') + nextRow
                    }
                  }

                })


              } else {
                // 走到这里 一般是 板件信息 或者订单信息
                loopObj.tagResB.forEach((noUsed, i) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置 
                  let onceLoopVal = target
                  // 替换操作 
                  dataObjectList.forEach((dataObj, q) => {
                    // 从字典拿回来的数据可能要二次操作  留个位置
                    let tempVal = dataObj.tagResB
                    let tempDataObj = dataObj
                    // 绝大多数的返回数据  这里控制的返回都是 数组
                    // 这里是不循环的输出结构即可
                    if (Array.isArray(tempVal)) {
                      // let checkVal

                      // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                      if (tempVal.length == loopObj.tagResB.length) {

                        /** 
                         这里就还需要判断一层  
                         如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                         tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                         这个时候  这个循环的基准 应该 再下一级 ？？
                         
                         解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环  
                        */

                        tempDataObj.value = tempDataObj.tagResB[i]

                      } else if (tempVal.length == 1) {
                        // 若是只有一个项直接输出就好

                        tempDataObj.value = tempVal.join('、')
                      } else {
                        // 其它情况 直接输出

                        tempDataObj.value = tempVal.join('、')
                      }

                    } else {
                      // 其它情况 直接输出
                      tempDataObj.value = tempVal
                    }
                    // 真正的替换内容
                    if (isUsedCondition && isCondition_Line == true && q == 0) {
                      let _ccc = { ...ccc, q }
                      let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                      if (conditionInfo) {
                        onceLoopVal = conditionInfo.dataTextarea
                      } else {
                        onceLoopVal = ''
                      }
                    }
                    onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                  });
                  dataNode = onceLoopVal
                  // 判断是否缩进 
                  if (isTab) {
                    outPutStringB.value += tabVal;
                    dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                  }
                  console.log('ccc 3 B', ccc)
                  outPutStringB.value = outPutStringB.value + dataNode + nextRow
                  // 标签 子节点的内容
                  if (item.children && Array.isArray(item.children)) {

                    createCNC(item.children, lvl + 1, targetSource, exportFileType, i, 'B')
                  }
                  if (dataTextarea_end) {
                    let endStr = dataTextarea_end || ''
                    if (isTab) {
                      outPutStringB.value += tabVal;
                      endStr = endStr.replaceAll('\n', '\n' + tabVal)
                    }
                    outPutStringB.value = outPutStringB.value + (dataTextarea_end || '') + nextRow
                  }
                });
              }
            }

          } else {
            if (isTab) {
              outPutStringB.value += tabVal;
              dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
            }

            dataObjectList.forEach(dataObj => {
              // 从字典拿回来的数据可能要二次操作  留个位置
              // 绝大多数的返回数据  这里控制的返回都是 数组
              // 这里是不循环的输出结构即可
              let tempVal = dataObj.tagResB

              if (Array.isArray(tempVal)) {
                if (pid != null) {
                  let target = tempVal[pid]
                  if (kid != -1) {
                    target = target[kid]
                  }
                  dataObj.value = target  //tempVal[pid]
                } else {
                  dataObj.value = tempVal.join('、')
                }
              } else {
                dataObj.value = tempVal
              }


              dataNode = dataNode.replaceAll(dataObj.sourceStr, dataObj.value)


            });
            console.log('ccc 4 B', ccc)
            outPutStringB.value = outPutStringB.value + dataNode + nextRow

            // 标签头部 结束


            // 标签 子节点的内容
            if (item.children && Array.isArray(item.children)) {
              createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'B')
            }
            // 标签尾部
            if (dataTextarea_end) {
              let endStr = dataTextarea_end || ''
              if (isTab) {
                outPutStringB.value += tabVal;
                endStr = endStr.replaceAll('\n', '\n' + tabVal)
              }
              outPutStringB.value = outPutStringB.value + (endStr || '') + nextRow
            }
          }
        }
        if (_flag == 'start' && hasConfig.value == true) {
          // 加工完成  收刀

          // 收刀
          hasConfig.value = false
          if (allProcessList.value && allProcessList.value[allProcessList.value.length - 1]) {
            let finalWorkKnife = allProcessList.value[allProcessList.value.length - 1].processItem.knife
            outPutStringB.value += finalWorkKnife.axisStopCode + finalWorkKnife.knifeStopCode
          }

        }
      } else {
        //#region  新的  写入文件 不分面

        // 判断是否分面
        if (isUsedChangeKnife == true) {
          hasConfig.value = true
        }


        // 头部补充内容
        if (dataTextarea_start) {
          outPutString.value = outPutString.value + (dataTextarea_start || '') + nextRow
        }



        // 判断数据的循环依赖 是否可以循环   可以的话节点进行循环  不可以直接输出
        if (isLoop && loopObj && loopObj.tagRes && Array.isArray(loopObj.tagRes)) {
          if (loopObj.tagRes.length > 0) {
            //轮廓 或者点阵等等 这一层级    会走这里
            const target = dataNode;

            let loopMore = false

            // 判断是否需要三级循环  针对 点阵 轮廓
            loopObj.tagRes.forEach(item => {
              if (Array.isArray(item)) {
                loopMore = true
              }
            })
            // pid  板  // kid // 造型 i
            let loopItem = loopObj.tagRes[pid]
            if (kid != -1 && kid != null) {
              loopItem = loopItem[kid]
            }
            if (loopMore && Array.isArray(loopItem) && pid != null && pid != -1) {
              //** 到了这里 说明是 类似点阵 轮廓等信息 loopObj.tagRes 为 多维数组的时候  循环的基准点就变了 */ 
              loopItem.forEach((noUsed, i) => {
                // 从字典拿回来的数据可能要二次操作  留个位置 
                let onceLoopVal = target
                let changeKnifeCode: any = ''
                // 换刀逻辑
                if (isUsedChangeKnife) {
                  changeKnifeCode = getChangeKnifeCode(ccc, loopObj, i, isCondition_Line)

                  onceLoopVal = changeKnifeCode + onceLoopVal
                }

                // 条件显示

                if (isUsedCondition && isCondition_Line == false) {
                  let conditionInfo = checkCondition(conditionList, ccc, loopObj, i, isCondition_Line)
                  if (conditionInfo) {
                    onceLoopVal = conditionInfo.dataTextarea
                    // if (changeKnifeCode != '') {
                    //   onceLoopVal = changeKnifeCode + conditionInfo.dataTextarea
                    // }
                  } else {
                    onceLoopVal = ''
                  }
                }
                onceLoopVal = changeKnifeCode + onceLoopVal

                let isLev3Loop = false; // 是否时三维数组   造型轮廓的时候
                let lev3LoopList: any = [];
                // 替换操作 
                dataObjectList.forEach((dataObj, q) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置
                  let tempVal = dataObj.tagRes
                  let tempDataObj = dataObj
                  // 绝大多数的返回数据  这里控制的返回都是 数组
                  // 这里是不循环的输出结构即可
                  let loopOrderByList = loopObj.tagRes[pid]
                  let loopTempVal = tempVal[pid]
                  if (kid != -1 && Array.isArray(tempVal)) {

                    loopTempVal = loopTempVal[kid]
                    loopOrderByList = loopOrderByList[kid]
                  }
                  if (Array.isArray(loopTempVal)) {
                    // let checkVal
                    // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                    if (loopTempVal.length == loopOrderByList.length) {

                      /** 
                       这里就还需要判断一层  
                       如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                       tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                       这个时候  这个循环的基准 应该 再下一级 ？？
                       
                       解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环   
                      */

                      // console.log('see!!!!!!!!!!!!!!!!!!!!',loopTempVal);
                      tempDataObj.value = loopTempVal[i] // tempDataObj.tagRes[pid][i]

                    } else if (tempVal.length == 1) {
                      // 若是只有一个项直接输出就好

                      tempDataObj.value = tempVal.join('、')
                    } else {
                      // 其它情况 直接输出

                      tempDataObj.value = tempVal.join('、')
                    }

                  } else {
                    // 其它情况 直接输出
                    tempDataObj.value = tempVal
                  }



                  // 真正的替换内容 
                  // 将数据写入的 当前节点的字符串

                  if (tempDataObj.value && Array.isArray(tempDataObj.value)) {
                    isLev3Loop = true

                    if (lev3LoopList.length == tempDataObj.value.length && lev3LoopList.length) {
                      tempDataObj.value.forEach((val, j) => {

                        let temp = lev3LoopList[j]
                        let tempVal = temp.replaceAll(tempDataObj.sourceStr, val)
                        lev3LoopList[j] = tempVal
                      });
                    } else {
                      tempDataObj.value.forEach((val, j) => {
                        let temp = onceLoopVal

                        let tempVal = temp.replaceAll(tempDataObj.sourceStr, val)
                        lev3LoopList.push(tempVal);
                      });
                    }

                  } else {
                    if (isUsedCondition && isCondition_Line == true && q == 0) {
                      let _ccc = { ...ccc, q }
                      let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                      if (conditionInfo) {
                        onceLoopVal = conditionInfo.dataTextarea
                      } else {
                        onceLoopVal = ''
                      }
                    }
                    onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                  }

                });
                // 将当前节点的字符串  写入到文件的字符串
                if (isLev3Loop) {
                  lev3LoopList.forEach(str => {
                    dataNode = str
                    if(str.includes('{{<')){
                       
                      dataNode = onceLoopVal
                    }
                    // 判断是否缩进

                    if (isTab) {
                      outPutString.value += tabVal;
                      dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                    }
                    console.log('ccc 1', ccc, i,)
                    outPutString.value = outPutString.value + dataNode + nextRow
                    // 标签 子节点的内容
                    if (item.children && Array.isArray(item.children)) {

                      createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'AB', i)
                    }
                    if (dataTextarea_end) {
                      let endStr = dataTextarea_end || ''
                      if (isTab) {
                        outPutString.value += tabVal;
                        endStr = endStr.replaceAll('\n', '\n' + tabVal)
                      }
                      outPutString.value = outPutString.value + (endStr || '') + nextRow
                    }
                  })
                } else {
                  dataNode = onceLoopVal
                  // 判断是否缩进

                  if (isTab) {
                    outPutString.value += tabVal;
                    dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                  }
                  console.log('ccc 2 不封边', ccc, i)
                  outPutString.value = outPutString.value + dataNode + nextRow
                  // 标签 子节点的内容
                  if (item.children && Array.isArray(item.children)) {


                    createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'AB', i)
                  }
                  if (dataTextarea_end) {
                    let endStr = dataTextarea_end || ''
                    if (isTab) {
                      outPutString.value += tabVal;
                      endStr = endStr.replaceAll('\n', '\n' + tabVal)
                    }
                    outPutString.value = outPutString.value + (endStr || '') + nextRow
                  }
                }

              })


            } else {
              // 走到这里 一般是 板件信息 或者订单信息
              loopObj.tagRes.forEach((noUsed, i) => {
                // 从字典拿回来的数据可能要二次操作  留个位置 
                let onceLoopVal = target
                // 替换操作 
                dataObjectList.forEach((dataObj, q) => {
                  // 从字典拿回来的数据可能要二次操作  留个位置
                  let tempVal = dataObj.tagRes
                  let tempDataObj = dataObj
                  // 绝大多数的返回数据  这里控制的返回都是 数组
                  // 这里是不循环的输出结构即可
                  if (Array.isArray(tempVal)) {
                    // let checkVal

                    // 这里做个判定 如果数组的长度是一样的  默认为 同一个数据集  用loopObj.tagRes的下标i就好
                    if (tempVal.length == loopObj.tagRes.length) {

                      /** 
                       这里就还需要判断一层  
                       如果是 点数据 、轮廓数据或者 3级数据的时候 (小板 =》小板明细 =》点阵|| 轮廓 数据集 =》具体细项（要显示的数据）)
                       tempDataObj.tagRes[i] 应该是个数组 一维数组的情况
                       这个时候  这个循环的基准 应该 再下一级 ？？
                       
                       解决方案（暂定）：在真正替换文本内容的时候  做个判断和循环  
                      */

                      tempDataObj.value = tempDataObj.tagRes[i]

                    } else if (tempVal.length == 1) {
                      // 若是只有一个项直接输出就好

                      tempDataObj.value = tempVal.join('、')
                    } else {
                      // 其它情况 直接输出

                      tempDataObj.value = tempVal.join('、')
                    }

                  } else {
                    // 其它情况 直接输出
                    tempDataObj.value = tempVal
                  }
                  // 真正的替换内容
                  if (isUsedCondition && isCondition_Line == true && q == 0) {
                    let _ccc = { ...ccc, q }
                    let conditionInfo = checkCondition(conditionList, _ccc, loopObj, i, isCondition_Line)

                    if (conditionInfo) {
                      onceLoopVal = conditionInfo.dataTextarea
                    } else {
                      onceLoopVal = ''
                    }
                  }
                  onceLoopVal = onceLoopVal.replaceAll(tempDataObj.sourceStr, tempDataObj.value)
                });
                dataNode = onceLoopVal
                // 判断是否缩进 
                if (isTab) {
                  outPutString.value += tabVal;
                  dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
                }
                console.log('ccc 3', ccc, i)
                outPutString.value = outPutString.value + dataNode + nextRow
                // 标签 子节点的内容
                if (item.children && Array.isArray(item.children)) {

                  createCNC(item.children, lvl + 1, targetSource, exportFileType, i, 'AB')
                }
                if (dataTextarea_end) {
                  let endStr = dataTextarea_end || ''
                  if (isTab) {
                    outPutString.value += tabVal;
                    endStr = endStr.replaceAll('\n', '\n' + tabVal)
                  }
                  outPutString.value = outPutString.value + (endStr || '') + nextRow
                }
              });
            }
          }

        } else {
          if (isTab) {
            outPutString.value += tabVal;
            dataNode = dataNode.replaceAll('\n', '\n' + tabVal)
          }

          dataObjectList.forEach(dataObj => {
            // 从字典拿回来的数据可能要二次操作  留个位置
            // 绝大多数的返回数据  这里控制的返回都是 数组
            // 这里是不循环的输出结构即可
            let tempVal = dataObj.tagRes

            if (Array.isArray(tempVal)) {
              if (pid != null) {
                let target = tempVal[pid]
                if (kid != -1) {
                  target = target[kid]
                }
                dataObj.value = target  //tempVal[pid]
              } else {
                dataObj.value = tempVal.join('、')
              }
            } else {
              dataObj.value = tempVal
            }


            dataNode = dataNode.replaceAll(dataObj.sourceStr, dataObj.value)


          });
          console.log('ccc 4', ccc)
          outPutString.value = outPutString.value + dataNode + nextRow

          // 标签头部 结束


          // 标签 子节点的内容
          if (item.children && Array.isArray(item.children)) {


            createCNC(item.children, lvl + 1, targetSource, exportFileType, pid, 'AB')
          }
          // 标签尾部
          if (dataTextarea_end) {
            let endStr = dataTextarea_end || ''
            if (isTab) {
              outPutString.value += tabVal;
              endStr = endStr.replaceAll('\n', '\n' + tabVal)
            }

            outPutString.value = outPutString.value + (endStr || '') + nextRow

          }
        }

        if (_flag == 'start' && hasConfig.value == true) {
          // 加工完成  收刀

          // 收刀
          hasConfig.value = false
          if (allProcessList.value && allProcessList.value[allProcessList.value.length - 1]) {
            let finalWorkKnife = allProcessList.value[allProcessList.value.length - 1].processItem.knife
            outPutStringB.value += finalWorkKnife.axisStopCode + finalWorkKnife.knifeStopCode
          }

        }

        //#endregion 新的 结束

      }
    })
  }

  if (flag == undefined) {
    console.log('加工结束  收刀');
  }
}








/** 翻转小板,翻转动作opType: 0翻面 1右转 2后转 3左转 */
function turnBlock(block: PlaceBlock, opType: number, checkOverlap = true) {
  let orgStyle = block.placeStyle
  let newStyle = BlockHelper.getTurnedPlaceStyle(orgStyle, opType)

  let orgPlaceX = block.placeX - block.placeOffX
  let orgPlaceY = block.placeY - block.placeOffY

  let offset = BlockSizePlus.getOffDis(block, newStyle)
  let newPlaceX = orgPlaceX + offset.x
  let newPlaceY = orgPlaceY + offset.y
  block.placeOffX = offset.x
  block.placeOffY = offset.y
  // 修改小板的放置方式
  // console.log('修改小板的放置方式', block.blockNo, block, newStyle)

  resetPlaceStyle(block, newStyle, true)
  // 重置放置,检查冲突 
  // BlockHelper.replaceBlock(block.placeMaterial, block, block.boardId, new Point(newPlaceX, newPlaceY))
}

export function resetPlaceStyle(block: PlaceBlock, newStyle: PlaceStyle, isCNC: boolean = false) {
  block.placeStyle = newStyle

  // tryFix
  let _width = block.cutWidth
  let _lenth = block.cutLength
  if (block.width > block.length) {
    block.cutWidth = Math.max(_width, _lenth)
    block.cutLength = Math.min(_width, _lenth)
  } else {
    block.cutWidth = Math.min(_width, _lenth)
    block.cutLength = Math.max(_width, _lenth)
  }


  switch (newStyle) {
    case PlaceStyle.FRONT: // 正面
      block.placeWidth = block.cutWidth
      block.placeLength = block.cutLength
      block.placeSealLeft = block.sealLeft
      block.placeSealRight = block.sealRight
      block.placeSealTop = block.sealTop
      block.placeSealBottom = block.sealBottom
      block.holeCountSideLeft = block.holeCountLeft
      block.holeCountSideRight = block.holeCountRight
      block.holeCountSideTop = block.holeCountTop
      block.holeCountSideBottom = block.holeCountBottom
      block.placeDirection = '→'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '→' : '↓'
      break
    case PlaceStyle.FRONT_TURN_RIGHT: // 正面右转
      block.placeWidth = block.cutLength
      block.placeLength = block.cutWidth
      block.placeSealLeft = block.sealBottom
      block.placeSealRight = block.sealTop
      block.placeSealTop = block.sealLeft
      block.placeSealBottom = block.sealRight
      block.holeCountSideLeft = block.holeCountBottom
      block.holeCountSideRight = block.holeCountTop
      block.holeCountSideTop = block.holeCountLeft
      block.holeCountSideBottom = block.holeCountRight

      block.modelCountSideLeft = block.modelCountBottom
      block.modelCountSideRight = block.modelCountTop
      block.modelCountSideTop = block.modelCountLeft
      block.modelCountSideBottom = block.modelCountRight

      block.placeDirection = '↓'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '↓' : '←'
      break
    case PlaceStyle.FRONT_TURN_BACK: // 正面后转
      block.placeWidth = block.cutWidth
      block.placeLength = block.cutLength
      block.placeSealLeft = block.sealRight
      block.placeSealRight = block.sealLeft
      block.placeSealTop = block.sealBottom
      block.placeSealBottom = block.sealTop
      block.holeCountSideLeft = block.holeCountRight
      block.holeCountSideRight = block.holeCountLeft
      block.holeCountSideTop = block.holeCountBottom
      block.holeCountSideBottom = block.holeCountTop
      block.placeDirection = '←'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '←' : '↑'
      break
    case PlaceStyle.FRONT_TURN_LEFT: // 正面左转
      block.placeWidth = block.cutLength
      block.placeLength = block.cutWidth
      block.placeSealLeft = block.sealTop
      block.placeSealRight = block.sealBottom
      block.placeSealTop = block.sealRight
      block.placeSealBottom = block.sealLeft
      block.holeCountSideLeft = block.holeCountTop
      block.holeCountSideRight = block.holeCountBottom
      block.holeCountSideTop = block.holeCountRight
      block.holeCountSideBottom = block.holeCountLeft
      block.placeDirection = '↑'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '↑' : '→'
      break
    case PlaceStyle.BACK: // 反面
      block.placeWidth = block.cutWidth
      block.placeLength = block.cutLength
      block.placeSealLeft = block.sealRight
      block.placeSealRight = block.sealLeft
      block.placeSealTop = block.sealTop
      block.placeSealBottom = block.sealBottom
      block.holeCountSideLeft = block.holeCountRight
      block.holeCountSideRight = block.holeCountLeft
      block.holeCountSideTop = block.holeCountTop
      block.holeCountSideBottom = block.holeCountBottom
      block.placeDirection = '→'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '→' : '↑'
      break
    case PlaceStyle.BACK_TURN_RIGHT: // 反面右转
      block.placeWidth = block.cutLength
      block.placeLength = block.cutWidth
      block.placeSealLeft = block.sealBottom
      block.placeSealRight = block.sealTop
      block.placeSealTop = block.sealRight
      block.placeSealBottom = block.sealLeft
      block.holeCountSideLeft = block.holeCountBottom
      block.holeCountSideRight = block.holeCountTop
      block.holeCountSideTop = block.holeCountRight
      block.holeCountSideBottom = block.holeCountLeft
      block.placeDirection = '↓'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '↓' : '→'
      break
    case PlaceStyle.BACK_TURN_BACK: // 反面后转
      block.placeWidth = block.cutWidth
      block.placeLength = block.cutLength
      block.placeSealLeft = block.sealLeft
      block.placeSealRight = block.sealRight
      block.placeSealTop = block.sealTop
      block.placeSealBottom = block.sealBottom
      block.holeCountSideLeft = block.holeCountLeft
      block.holeCountSideRight = block.holeCountRight
      block.holeCountSideTop = block.holeCountTop
      block.holeCountSideBottom = block.holeCountBottom
      block.placeDirection = '←'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '←' : '↓'
      break
    case PlaceStyle.BACK_TURN_LEFT: // 反面左转
      block.placeWidth = block.cutLength
      block.placeLength = block.cutWidth
      block.placeSealLeft = block.sealTop
      block.placeSealRight = block.sealBottom
      block.placeSealTop = block.sealLeft
      block.placeSealBottom = block.sealRight
      block.holeCountSideLeft = block.holeCountTop
      block.holeCountSideRight = block.holeCountBottom
      block.holeCountSideTop = block.holeCountLeft
      block.holeCountSideBottom = block.holeCountRight
      block.placeDirection = '↑'
      block.placeDirection_Length = block.length > block.width - 0.001 ? '↑' : '←'
      break
    default:
      break
  }
  // this.resetDoFace_HoleModel(block)
  resetDoFace_HoleModel(block, isCNC)
}

function resetDoFace_HoleModel(block: PlaceBlock, isCNC: boolean = false) {
   
  let isTurnOver = block.isTurnOver
 
  let orgMA = block.blockDetail.modelListFaceA  
  let orgMB = block.blockDetail.modelListFaceB  
  let orgMT = block.blockDetail.modelListThrough 
  let orgHA = block.blockDetail.holeListFaceA 
  let orgHB = block.blockDetail.holeListFaceB 
  let orgHT = block.blockDetail.holeListThrough 

  if (isTurnOver) {
    block.modelListFaceA = orgMB.concat(orgMT);
    block.modelListFaceB = orgMA;
    block.holeListFaceA = orgHB.concat(orgHT);
    block.holeListFaceB = orgHA;
  }
  else {
    block.modelListFaceA = orgMA.concat(orgMT);
    block.modelListFaceB = orgMB;
    block.holeListFaceA = orgHA.concat(orgHT);;
    block.holeListFaceB = orgHB;
  }


  //获取贴标位置
  let p = getPlaceXYInBlock(block, block.blockDetail.labelPosX, block.blockDetail.labelPosY, false, false);
  block.labelPosX = p.x;
  block.labelPosY = p.y;
}

/** 获得原坐标在放置后的新坐标 */
function getPlaceXYInBlock(block: PlaceBlock, x: number, y: number, isSealed: boolean, isFaceB = false, preCutValueOff = null): Point {
  let bwidth = block.cutWidth
  let blength = block.cutLength
  let x0: any = x
  let y0: any = y
  if (isSealed) // 已封边
  {
    bwidth = block.width
    blength = block.length
    x0 += block.offsetX
    y0 += block.offsetY
  }
  if (!isSealed && preCutValueOff) // 没封边，且有封边
  {
    bwidth += preCutValueOff.w
    blength += preCutValueOff.l
    x0 += preCutValueOff.x
    y0 += preCutValueOff.y
  }

  let _point: any = BlockHelper.getPlacedPostionInBlock(block.placeStyle, bwidth, blength, x0, y0, isFaceB)

  return _point
}
/* 设置数据源 导出CNC时 用到的数据源 */
async function _setDataSource(sourceIn, exportFileType) {
  let targetSourceList: any = [];
  if (Array.isArray(sourceIn)) {
    targetSourceList = sourceIn
  } else {
    targetSourceList = [sourceIn]
  }

  let source: Array<PlaceBlock> | Array<PlaceMaterial> = targetSourceList
  // 数据归类 A（正面）B（反面） 
  /**翻面后的数据集 反面 */
  let sourceB: Array<PlaceBlock> | Array<PlaceMaterial> = []

  /**正面的数据集  */
  let sourceA: Array<PlaceBlock> | Array<PlaceMaterial> = []

  if (drillConfig.value?.isFileSort == undefined) {
    drillConfig.value.isFileSort = false
  }

  if (drillConfig.value?.isFileSort == true) {

    if (exportFileType == 0 || exportFileType == 1) {
      // 这里的 数据源是小板的集合   做个分类

      for (const block of source) {


        let hasProcess = false
        let doSideModel = false
        let hasA = (block.blockDetail.holeListFaceA.length > 0 || block.blockDetail.modelListFaceA.length > 0)
        let hasB = (block.blockDetail.holeListFaceB.length > 0 || block.blockDetail.modelListFaceB.length > 0)


        if (hasA == false && hasB == false) {
          hasA = true
        }
        if (placeData.value.sysConfig.isExportOrderProcess) {
          hasA = true
          hasB = true
        }
        // placeData.value.sysConfig.isExportOrderProcess

        if (hasA) {
          var blockA: PlaceBlock = cloneDeep(block)
          resetPlaceStyle(blockA, blockA.placeStyle, true)
          if (blockA.blockDetail.holes.length > 0 || blockA.blockDetail.models.length > 0) {
            hasProcess = true
          }

          if (blockA.blockDetail.modelListSide.length > 0) {
            doSideModel = true
            hasProcess = true
          }

          for (const hole of blockA.holes) {
            let p = await BlockHelper.getPlaceXYInBoard(blockA, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
            hole.pointZ = getProcessItemPointZ(hole.depth, blockA.thickness, hole.face, false)
          }


          for (const hole of blockA.holeListFaceA) {
            let p = await BlockHelper.getPlaceXYInBoard(blockA, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
            hole.pointZ = getProcessItemPointZ(hole.depth, blockA.thickness, hole.face, false)
          }

          for (const hole of blockA.holeListFaceB) {
            let p = await BlockHelper.getPlaceXYInBoard(blockA, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
            hole.pointZ = getProcessItemPointZ(hole.depth, blockA.thickness, hole.face, false)
          }

          for (const model of blockA.modelListFaceA) {
            model.pointList.forEach(async point => {
              let p = await BlockHelper.getPlaceXYInBoard(blockA, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
              // point.pointZ = getProcessItemPointZ(point.depth, blockA.thickness, model.face, false)
            })
          }

          for (const model of blockA.modelListFaceB) {
            model.pointList.forEach(async point => {
              let p = await BlockHelper.getPlaceXYInBoard(blockA, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
              // point.pointZ = getProcessItemPointZ(point.depth, blockA.thickness, model.face, false)
            })
          }

          for (const sideModel of blockA.blockDetail.modelListSide) {
            if (sideModel.realPointList.length == 0) {
              sideModel.realPointList = sideModel.pointList
            }
            for (const point of sideModel.realPointList) {
              let p = await BlockHelper.getPlaceXYInBoard(blockA, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
              // point.pointZ = Math.abs(point.pointZ) - blockA.thickness
              // point.pointZ = getProcessItemPointZ(point.pointZ, blockA.thickness, sideModel.direction, true)   // Math.abs(point.pointZ) - blockA.thickness
            }
          }
          const border_final = await BlockPlus.transformBorder(blockA.blockDetail.borderContour.borderFinal, blockA.placeStyle, blockA.width, blockA.length)
          blockA.blockDetail.borderContour.borderFinal = border_final

          const border = await BlockPlus.transformBorder(blockA.blockDetail.borderContour.border, blockA.placeStyle, blockA.width, blockA.length)
          blockA.blockDetail.borderContour.border = border

          const borderOrg = await BlockPlus.transformBorder(blockA.blockDetail.borderContour.borderOrg, blockA.placeStyle, blockA.width, blockA.length)
          blockA.blockDetail.borderContour.borderOrg = borderOrg
          await transformDataValue(blockA, drillConfig.value.projectList)
          sourceA.push(blockA)
        }

        if (hasB) {
          var blockB: PlaceBlock = cloneDeep(block) //new PlaceBlock(block)
          resetPlaceStyle(blockB, blockB.placeStyle, true)
          // 穿  孔 || 造型  剔除掉   
          //#region 挖穿的 部分  有A面加工 在A面做  没有的话 在B面做
          if (hasProcess == false) {
            blockB.holeListFaceB = blockB.holeListFaceB.filter(e => e.depth < blockB.thickness);
            blockB.holeListFaceA = blockB.holeListFaceA.filter(e => e.depth < blockB.thickness);

            blockB.blockDetail.holes = blockB.holes.filter(e => e.depth < blockB.thickness)
            blockB.blockDetail.holeListFaceB = blockB.holeListFaceB.filter(e => e.depth < blockB.thickness);
            blockB.blockDetail.holeListFaceA = blockB.holeListFaceA.filter(e => e.depth < blockB.thickness);

            blockB.modelListFaceA.filter(e => e.depth < blockB.thickness)
            blockB.modelListFaceB.filter(e => e.depth < blockB.thickness)

            blockB.blockDetail.modelListFaceA.filter(e => e.depth < blockB.thickness)
            blockB.blockDetail.modelListFaceB.filter(e => e.depth < blockB.thickness)
            blockB.blockDetail.models.filter(e => e.depth < blockB.thickness)
          }
          //#endregion
          await turnBlock(blockB, 0)

          for (const hole of block.holes) {
            let p = await BlockHelper.getPlaceXYInBoard(blockB, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
          }


          for (const hole of blockB.holeListFaceA) {
            let p = await BlockHelper.getPlaceXYInBoard(blockB, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
          }

          for (const hole of blockB.holeListFaceB) {
            let p = await BlockHelper.getPlaceXYInBoard(blockB, hole.pointX, hole.pointY)

            hole.pointX = p.x
            hole.pointY = p.y
          }

          for (const model of blockB.modelListFaceA) {
            for (const point of model.pointList) {
              let p = await BlockHelper.getPlaceXYInBoard(blockB, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
            }

          }

          for (const model of blockB.modelListFaceB) {
            for (const point of model.pointList) {
              let p = await BlockHelper.getPlaceXYInBoard(blockB, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
            }
          }

          for (const sideModel of blockB.blockDetail.modelListSide) {
            if (sideModel.realPointList.length == 0) {
              sideModel.realPointList = sideModel.pointList
            }
            for (const point of sideModel.realPointList) {
              let p = await BlockHelper.getPlaceXYInBoard(blockB, point.pointX, point.pointY)
              point.pointX = p.x
              point.pointY = p.y
              point.pointZ = Math.abs(point.pointZ) - blockB.thickness
            }
          }




          const border_final = await BlockPlus.transformBorder(blockB.blockDetail.borderContour.borderFinal, blockB.placeStyle, blockB.width, blockB.length)
          blockB.blockDetail.borderContour.borderFinal = border_final


          const border = await BlockPlus.transformBorder(blockB.blockDetail.borderContour.border, blockB.placeStyle, blockB.width, blockB.length)
          blockB.blockDetail.borderContour.border = border


          const borderOrg = await BlockPlus.transformBorder(blockB.blockDetail.borderContour.borderOrg, blockB.placeStyle, blockB.width, blockB.length)
          blockB.blockDetail.borderContour.borderOrg = borderOrg

          await transformDataValue(blockB, drillConfig.value.projectList)

          sourceB.push(blockB)
          // sourceB.push(blockB)
        }
      }


    } else {
      sourceA = source.slice();
      sourceB = source.slice();
      // 这里的 数据源是大板的集合
      sourceB.forEach(board => {
        PlacePositionHelper.turnPlacePosition(board, drillConfig.value.processLocation)

        board.blockList.forEach(block => {
          turnBlock(block, 0)

          BlockHelper.resetPlaceStyle(block, block.placeStyle)

          const border_final = BlockPlus.transformBorder(block.blockDetail.borderContour.borderFinal, block.placeStyle, block.width, block.length)
          const border = BlockPlus.transformBorder(block.blockDetail.borderContour.border, block.placeStyle, block.width, block.length)
          const borderOrg = BlockPlus.transformBorder(block.blockDetail.borderContour.borderOrg, block.placeStyle, block.width, block.length)
          block.blockDetail.borderContour.borderFinal = border_final
          block.blockDetail.borderContour.border = border
          block.blockDetail.borderContour.borderOrg = borderOrg

          transformDataValue(block, drillConfig.value.projectList)
        });
      });
    }
    // console.log('sourceA:', sourceA[0]?.place, 'sourceB：', sourceB)
  }

  // 加工项做个排序  减少换刀
  //  sortSource  排序
  fileDataA.value = sourceA
  fileDataB.value = sourceB
  fileDataAB.value = source


  return
}
/**
 * 获取实际Z坐标
 * @param depth  加工深度
 * @param thickness  板厚
 * @param face 0 正面  1 反面 加工面  若侧面造型  face读取 _flag（加工面）-- =》A =0  B = 1 AB = 2 
 * @param isSide true 是  false 否 是否是侧面加工
 * 说明   条件1、  板面  或者是 台面       --机台配置 originZ0Position
 *        条件2、  是否翻面加工 (A面  B面) --根据是否分面导出判断
 *        条件3、  判断哪个面的加工
 *        若 isSide 为true  depth 要穿 pointZ
 * @returns 
 */
function getProcessItemPointZ(depth: number, thickness: number, face, isSide) {
  let z = 0
  if (isSide) {
    // 侧面加工 
    if (drillConfig.value?.originZ0Position == 0) {
      // 台面
      if (drillConfig.value?.isFileSort == true) {
        // 分面
        if (face == 0) {
          // 正面
          z = depth
        } else {
          // 反面
          z = thickness - depth
        }

      } else {
        // 不分面
        if (face == 0) {
          // 正面
          z = thickness - depth
        } else {
          // 反面
          z = depth
        }
      }
    } else {
      // 板面
      if (drillConfig.value?.isFileSort == true) {
        // 分面
        if (face == 0) {
          // 正面
          z = - depth
        } else {
          // 反面
          z = - depth
        }
      } else {
        // 不分面

        if (face == 0) {
          // 正面
          z = - depth
        } else {
          // 反面
          z = - (thickness - depth)
        }
      }
    }
  } else {
    // 垂直加工

    if (drillConfig.value?.originZ0Position == 0) {
      // 台面

      if (drillConfig.value?.isFileSort == true) {
        // 分面
        if (face == 0) {
          // 正面
          z = thickness - depth
        } else {
          // 反面
          z = thickness - depth
        }
      } else {
        // 不分面
        if (face == 0) {
          // 正面
          z = thickness - depth
        } else {
          // 反面
          z = depth
        }
      }
    } else {
      // 板面

      if (drillConfig.value?.isFileSort == true) {
        // 分面
        if (face == 0) {
          // 正面
          z = - depth
        } else {
          // 反面
          z = - depth
        }
      } else {
        // 不分面
        if (face == 0) {
          // 正面
          z = - depth
        } else {
          // 反面
          z = - (thickness - depth)
        }
      }
    }
  }

  return z
}

function sortSourceProcess(arr) {
  arr.sort((a, b) => a.radius - b.radius)
}

const fileDataA = ref<any>()
const fileDataB = ref<any>()
const fileDataAB = ref<any>()

/**
 * @param dictItem 字典 object 包含  type 大类标识 itemKey 小类标识
 * @param source 依据的数据项
 * @param exportFileType  导出模式  
 * @param boardIndex  exportFileType 为 2 大板独立模式时可能会用到
 * @param isFaceB 导出的面  为空 则导出所有   true 导出 B面（反面） false 导出 A面（正面）
 * 注： 处理正反面数据的时候  
 */
/**数据处理  取值 */
export function getPlaceDataValueByDic(dictItem: any, sourceIn?: any, exportFileType?: any) {

  // 数据归类 A（正面）B（反面） 
  /**翻面后的数据集 反面 */
  let sourceB: Array<PlaceBlock> | Array<PlaceMaterial> = []

  /**正面的数据集  */
  let sourceA: Array<PlaceBlock> | Array<PlaceMaterial> = []

  let source: Array<PlaceBlock> | Array<PlaceMaterial> = []

  source = fileDataAB.value
  sourceA = fileDataA.value
  sourceB = fileDataB.value
  // 代码控制了下  这边 source 只可能为 block 的list  或者是  material 的list   
  // 0(单条数据小板)  1（所有小板）      2 （单条商品|大板） 3  (所有商品 | 大板)


  /** 
   * 注---需要特殊处理的数据
   * 1、翻面 AB 面的数据
   *   
   * 加工-成品轮廓信息 outline
   * 加工-开料成型轮廓信息 cuttingoutline
   * 加工-坯料轮廓信息 rawoutline
   * 加工-垂直孔列表 holes
   * 加工-侧孔列表 sideHoles
   * 加工-造型/凹槽列表-垂直 grooves
   * 加工-造型/凹槽列表-水平 groovesH
   * 
   * 以上数据需要处理  dictItem.tagResB 和  dictItem.tagResA  要加载对应的数据
   * */
  // 暂不处理 的值 -- 留个位置 暂时没用
  let nodeal = 'noDeal'
  // fileInfo 

  /**
   * 辅助处理数据的 数据集
   */

  // 余料大板
  let remainBoardList: any = []
  placeData.value.materialList.forEach(material => {
    remainBoardList = remainBoardList.concat(material.remainBoardList)
  });


  // 大板 ID集合
  let goodsIdList: any = [];

  if (exportFileType == 0 || exportFileType == 1) {
    goodsIdList = source.map(e => e.goodsId)
  }
  let originZ0Position = drillConfig.value.originZ0Position
  /**解析 说明
   * 
   * 如果分AB面  解析数据的时候  只要取对应面的数据就好
   * 
   * 这里就是 如果有分AB面  将 AB面的数据 怼到 dictItem.tagResB 和  dictItem.tagResA 里面
   */

  switch (dictItem.type) {
    // 文件信息
    //#region 文件信息
    case 'fileInfo':
      switch (dictItem.itemKey) {
        case 'version':
          // 文件信息fileInfo
          dictItem.tagRes = nodeal
          break;
        case 'encoding':
          // 文件信息fileInfo
          dictItem.tagRes = placeData.value.sysConfig.ncFileEncoding
          break;
        // case 'exportTime':
        //   dictItem.tagRes = new Date().getTime()
        //   break;
        case 'sourceFlag':
          // 文件信息fileInfo
          dictItem.tagRes = nodeal
          break;
        case 'sourceType':
          // 文件信息fileInfo
          dictItem.tagRes = nodeal
          break;
        case 'exportTime':
          // 文件信息fileInfo
          dictItem.tagRes = formatToDate(new Date(), 'YYYY-MM-DD HH:mm:ss')
          break;
        case 'processFileOrderByFace':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.isFileSort || ''
          break;
        case 'processFileNameFaceA':
          // 文件信息fileInfo
          var fileName = drillConfig.value?.fileFrontPathName || ''
          var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)

          fileNameConfigList && fileNameConfigList.forEach(val => {
            if (exportFileType == 0 || exportFileType == 1) {
              var newVal = getFileName(val, exportFileType, source, true);
              fileName = fileName.replaceAll(val, newVal)
            }

          })
          dictItem.tagRes = fileName
          break;
        case 'processFileNameFaceB':
          // 文件信息fileInfo
          var fileName = drillConfig.value?.fileBackPathName || ''
          var fileNameConfigList = fileName.match(/{{\<([^>]*)>\}}/g)

          fileNameConfigList && fileNameConfigList.forEach(val => {
            if (exportFileType == 0 || exportFileType == 1) {
              var newVal = getFileName(val, exportFileType, source, true);
              fileName = fileName.replaceAll(val, newVal)
            }
          })
          dictItem.tagRes = fileName

          break;
        case 'ZipName':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.zipPathName || ''
          break;
        case 'exportEmptyOrder':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.isExportOrderNoBoard || ''
          break;
        case 'exportEmptyBoard':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.isExportOrderNoProcess || ''
          break;
        case 'exportProcessed':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.isExportOrderProcess || ''
          break;
        case 'notSignAlone':
          // 文件信息fileInfo
          dictItem.tagRes = drillConfig.value?.isAlreadyOrderProcess || ''
          break;
        default:
          break;
      }
      break;

    //#endregion 文件信息
    // 订单信息
    //#region 订单信息
    case 'orders':
      switch (dictItem.itemKey) {
        case 'orderId':
          // 订单信息 orders
          // 现在只有一个 orderId 没有  orderNo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.orderId)
          } else {
            // 如果是  2 或者 3 的时候  orderId 是个数组
            dictItem.tagRes = source.map(e => e.orderId).toString().split(',');
          }
          break;
        case 'orderNo':
          // 订单信息 orders
          // 现在只有一个 orderId 没有  orderNo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.orderId)
          } else {
            // 如果是  2 或者 3 的时候  orderId 是个数组
            dictItem.tagRes = source.map(e => e.orderId).toString().split(',');
          }
          break;
        case 'customOrderNo':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.customOrderNo)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.customOrderNo)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }

          break;
        case 'dealer':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.dealer)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.dealer)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }

          break;
        case 'orderDate':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.orderDate)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.orderDate)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }
          break;
        case 'deliveryDate':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.deliveryDate)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.deliveryDate)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }

          break;
        case 'customer':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.customer)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.customer)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }
          break;
        case 'phoneNumber':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.phoneNumber)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.phoneNumber)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }

          break;
        case 'address':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.address)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.address)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }

          break;
        case 'salesman':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.salesman)
          } else {
            let tagRes: any = [];

            source.forEach(e => {
              let res = e.blockList.map(e => e.order.salesman)
              tagRes = tagRes.concat(res)
            })

            dictItem.tagRes = tagRes
          }
          break;
        case 'splitter':
          // 订单信息 orders
          // dictItem.tagRes = source.map(e => e.order.splitter)

          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.splitter)
          } else {
            let tagRes: any = [];
            source.forEach(e => {
              let res = e.blockList.map(e => e.order.splitter)
              tagRes = tagRes.concat(res)
            })
            dictItem.tagRes = tagRes
          }
          break;
        case 'status':
          // 订单信息 orders
          // dictItem.tagRes = source.map(e => e.order.status)
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.status)
          } else {
            let tagRes: any = [];
            source.forEach(e => {
              let res = e.blockList.map(e => e.order.status)
              tagRes = tagRes.concat(res)
            })
            dictItem.tagRes = tagRes
          }
          break;
        case 'remark':
          // 订单信息 orders
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.order.remark)
          } else {
            let tagRes: any = [];
            source.forEach(e => {
              let res = e.blockList.map(e => e.order.remark)
              tagRes = tagRes.concat(res)
            })
            dictItem.tagRes = tagRes
          }
          // dictItem.tagRes = source.map(e => e.order.remark)
          break;
        default:
          break;
      }
      break;

    //#endregion 订单信息
    //排单信息

    //#region 排单信息
    case 'planOrder':
      switch (dictItem.itemKey) {
        case 'id':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.planId
          break;
        case 'plateInfoList':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.materialList.map(e => e.goodsName)
          break;
        case 'machineName':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.source.machineName
          break;
        case 'status':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.status
          break;
        case 'optimizeGoodsCount':
          //排单信息 planOrder
          let length = placeData.value.blockList.length
          let blockCountList = placeData.value.materialList.map(e => e.blockCount)
          let count = 0;
          blockCountList.forEach(e => {
            count += e;
          })
          dictItem.tagRes = length - count
          break;
        case 'cutGoodsCount':
          //排单信息 planOrder
          break;
        case 'plateNum':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.blockList.length

          break;
        case 'unOptimizePlateCount':
          //排单信息 planOrder
          break;
        case 'optimizePlateCount':
          //排单信息 planOrder
          break;
        case 'remark':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.remark
          break;
        case 'createTime':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.createTime
          break;
        case 'creator':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.creator
          break;
        case 'produceTime':
          //排单信息 planOrder
          dictItem.tagRes = placeData.value.produceTime
          break;
        default:
          break;
      }
      break

    //#endregion 排单信息
    // 大板
    //#region  大板
    case 'materials':
      switch (dictItem.itemKey) {
        case 'produceTime':
          // 大板（商品）materials
          // dictItem.tagRes = placeData.value.materialList
          break;
        case 'goodsName':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.goodsName)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.goodsName);
              }
            });
            dictItem.tagRes = res
          }

          break;
        case 'material':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.material)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.material);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => e.material)
          break;
        case 'color':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.color)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.color);
              }
            });
            dictItem.tagRes = res
          }

          break;
        case 'texture':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.hasTexture)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.hasTexture);
              }
            });
            dictItem.tagRes = res
          }
          break;
        case 'brand':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.brand)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.brand);
              }
            });
            dictItem.tagRes = res
          }
          break;
        case 'width':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.width))
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.width);
              }
            });
            dictItem.tagRes = res
          }
          break;
        case 'length':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.length))
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.length);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => outPutFloatNumberByConfig(e.length))
          break;
        case 'thickness':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.thickness)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.thickness);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => e.thickness)
          break;
        case 'oddmentLength':
          // 大板（商品）materials
          dictItem.tagRes = remainBoardList.filter(e => goodsIdList.includes(e.goodsId))?.map(e => outPutFloatNumberByConfig(e.length))
          break;
        case 'oddmentWidth':
          // 大板（商品）materials
          dictItem.tagRes = remainBoardList.filter(e => goodsIdList.includes(e.goodsId))?.map(e => outPutFloatNumberByConfig(e.width))
          // dictItem.tagRes = remainBoardList.map(e => outPutFloatNumberByConfig(e.width))
          break;
        case 'oddmentThickness':
          // 大板（商品）materials
          dictItem.tagRes = remainBoardList.filter(e => goodsIdList.includes(e.goodsId))?.map(e => e.thickness)
          // dictItem.tagRes = remainBoardList.map(e => e.thickness)
          break;
        case 'oddmentNo':
          // 大板（商品）materials
          dictItem.tagRes = remainBoardList.filter(e => goodsIdList.includes(e.goodsId))?.map(e => e.id)
          // dictItem.tagRes = remainBoardList.map(e => e.id)
          break;
        case 'spec':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.spec)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.spec);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => e.spec)
          break;
        case 'count':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.blockCount)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.blockCount);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => e.blockCount)
          break;
        case 'area':
          // 大板（商品）materials
          if (exportFileType == 2 || exportFileType == 3) {
            dictItem.tagRes = source.map(e => e.blockArea)
          } else {
            let res: any = [];
            placeData.value.materialList.forEach(material => {
              if (goodsIdList.includes(material.goodsId)) {
                res.push(material.blockArea);
              }
            });
            dictItem.tagRes = res
          }
          // dictItem.tagRes = placeData.value.materialList.map(e => e.blockArea)
          break;
        case 'unit':
          // 大板（商品）materials
          // dictItem.tagRes = placeData.value.planId
          break;
        case 'remark':
          // 大板（商品）materials
          break;
        default:
          break;
      }
      break;

    //#endregion 大板
    //小板
    //#region 小板
    case 'plates':
      switch (dictItem.itemKey) {
        case 'produceTime':
          // 小板 plates
          break;
        case 'roomName':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.roomName)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.roomName))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'bodyName':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.bodyName)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.bodyName))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'blockNo':
          // 小板 plates
          // dictItem.tagRes = 
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.blockNo)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockNo))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'labelNo':
          // 小板 plates
          // if(exportFileType == 0 || exportFileType == 1){
          //   dictItem.tagRes = source.map(e=>e.labelNo)
          // }else{
          //   let tagRes :any = [];
          //   source.forEach(material => {
          //     tagRes.concat(material.blockList.map(e=>e.labelNo)) 
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break;
        case 'blockName':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.blockName)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockName))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'openDoorType':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.openDoorType)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.openDoorType))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'count':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.count)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.count))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'id':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.id)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.id))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'plateRemark':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.plateRemark)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.plateRemark))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'remarkJsonDei':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.remarkJsonDei)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.remarkJsonDei))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'sideRemark':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.sideRemark)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.sideRemark))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'extraRemark':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.extraRemark)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.extraRemark))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'drillsRemark':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.drillsRemark)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.drillsRemark))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'remarkJson':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.remarkJson)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.remarkJson))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'specialRemark':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.specialRemark)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.specialRemark))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'specialRemarkDei':
          // 小板 plates
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e?.remarkParams?.specialRemarkDei)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.remarkParams?.specialRemarkDei))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'remarkid':
          // 小板 plates
          break;
        case 'remarkname':
          // 小板 plates
          break;
        case 'remarkinfo':
          // 小板 plates
          break;

        default:
          break;
      }
      break;

    //#endregion 小板
    // 尺寸
    //#region 尺寸
    case 'sizeInfo':
      switch (dictItem.itemKey) {

        case 'width':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e =>  outPutFloatNumberByConfig(e.width))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.width)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'length':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.length))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.length)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'thickness':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.thickness))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.thickness)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'area':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.area)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.area))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealLeft':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealLeft)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealLeft))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealRight':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealRight)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealRight))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealTop':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealTop)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealTop))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealBottom':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealBottom)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealBottom))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'cutWidth':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.cutWidth))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.cutWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'cutLength':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.cutLength))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.cutLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'cutArea':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.cutArea))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.cutArea)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeFullWidth':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeFullWidth))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeFullWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeFullLength':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeFullLength))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeFullLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sizeExpand':
          // 尺寸 sizeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sizeExpand)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sizeExpand))
            });
            dictItem.tagRes = tagRes
          }
          break
        default:
          break;
      }
      break;
    //#endregion 尺寸
    // 高级信息
    //#region 高级信息
    case 'highInfo':
      switch (dictItem.itemKey) {

        case 'isAdditionalBlock':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isAdditionalBlock)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isAdditionalBlock))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isRemainBlock':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isRemainBlock)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isRemainBlock))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'texture':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.texture)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.texture))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeHole':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.placeHole)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.placeHole))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isUnRegular':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isUnRegular)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isUnRegular))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'processGroupName':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.processGroupName)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.processGroupName))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'bigHoleInFaceA':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.bigHoleInFaceA)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.bigHoleInFaceA))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountFront':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountFront)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountFront))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountBack':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountBack)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountBack))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountThrough':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountThrough)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountThrough))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountSide':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountSide)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountSide))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'modelCountFront':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.modelCountFront)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelCountFront))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'modelCountBack':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.modelCountBack)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelCountBack))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'modelCountThrough':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.modelCountThrough)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelCountThrough))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isDoubleFaceProcess':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isDoubleFaceProcess)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isDoubleFaceProcess))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isTurnFaceToPlace':
          // 高级信息 highInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isTurnFaceToPlace)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isTurnFaceToPlace))
            });
            dictItem.tagRes = tagRes
          }
          break
        default:
          break;
      }
      break

    //#endregion 高级信息
    // 优化信息
    //#region 优化信息
    case 'placeInfo':
      switch (dictItem.itemKey) {

        case 'isOptimized':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isPlaced)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isPlaced))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'boardId':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.boardId)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.boardId))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeX':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeX))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeY':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeY))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeOffX':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeOffX))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeOffX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeOffY':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeOffY))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeOffY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeStyle':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.placeStyle)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.placeStyle))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'cutOrder':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.cutOrder)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.cutOrder))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'cutPointId':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.cutPointId)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.cutPointId))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealLeft':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealLeft)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealLeft))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealRight':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealRight)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealRight))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealTop':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealTop)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealTop))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'sealBottom':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.sealBottom)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.sealBottom))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeLength':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeLength))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeWidth':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeWidth))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isTurnOver':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isTurnOver)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isTurnOver))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isTurnBack':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isTurnBack)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isTurnBack))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isTurnDown':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isTurnDown)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isTurnDown))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isAutoPlaced':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isAutoPlaced)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isAutoPlaced))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isOverlap':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isOverlap)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isOverlap))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeDirection':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.placeDirection)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.placeDirection))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeDirection_Length':
          // 优化信息  placeInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.placeDirection_Length)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.placeDirection_Length))
            });
            dictItem.tagRes = tagRes
          }
          break
        default:
          break;
      }
      break
    //#endregion 优化信息
    // 开料信息
    //#region 开料信息
    case 'cuttingInfo':
      switch (dictItem.itemKey) {
        case 'isCutOtherFace':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isCutOtherFace)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isCutOtherFace))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isDrilling':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isDrilling)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isDrilling))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'isModeling':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.isModeling)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.isModeling))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountFaceA':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountFaceA)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountFaceA))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holeCountFaceB':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.holeCountFaceB)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeCountFaceB))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'placeOffY':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => outPutFloatNumberByConfig(e.placeOffY))
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => outPutFloatNumberByConfig(e.placeOffY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'modelCountFaceA':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.modelCountFaceA)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelCountFaceA))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'modelCountFaceB':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.modelCountFaceB)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelCountFaceB))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'codeA':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.codeA)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.codeA))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'codeB':
          // 开料信息 cuttingInfo
          if (exportFileType == 0 || exportFileType == 1) {
            dictItem.tagRes = source.map(e => e.codeB)
          } else {
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.codeB))
            });
            dictItem.tagRes = tagRes
          }
          break

        default:
          break;
      }
      break

    //#endregion 开料信息
    // 封边信息
    //#region 封边信息
    case 'edging':
      switch (dictItem.itemKey) {

        case 'xxx':
          break


        default:
          break;
      }
      break
    //#endregion 封边信息

    // 轮廓
    //#region 成品轮廓
    case 'outline':

      switch (dictItem.itemKey) {

        case 'count':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null)
              dictItem.tagResB = sourceB.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null)
            }
            dictItem.tagRes = source.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null)


          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null))
              });
              dictItem.tagResB = tagResB

            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.length : null))
            });
            dictItem.tagRes = tagRes


          }

          break

        case 'outline_id':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null)
              dictItem.tagResB = sourceB.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null)
            }
            dictItem.tagRes = source.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null)



          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderFinal) ? e.blockDetail.borderContour.borderFinal.map((c, b) => b) : null))
            });
            dictItem.tagRes = tagRes

          }

          break
        case 'outline_process':
          // 目前没有
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => {
          //     let arr = e.blockDetail.borderContour.borderFinal.map((x, i) => i)
          //     return arr
          //   })
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => {
          //       let arr = e.blockDetail.borderContour.borderFinal.map((x, i) => i)
          //       return arr
          //     }
          //     ))
          //   });
          //   dictItem.tagRes = tagRes
          // }

          break

        case 'remark':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.plateRemark)
              dictItem.tagResB = sourceB.map(e => e.plateRemark)
            }
            dictItem.tagRes = source.map(e => e.plateRemark)


          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.plateRemark))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.plateRemark))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.plateRemark))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'scale':
          // 成品轮廓 count
          //预留边  ReservedEdge  要加
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge))
              dictItem.tagResB = sourceB.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge))
            }

            dictItem.tagRes = source.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockdetail?.plateExtraRemark.extra.rectSealDetail.map(x => x.ReservedEdge)))
            });
            dictItem.tagRes = tagRes
          }

          break
        case 'preMilling':
          // 成品轮廓 count
          //  先不管  好像现在没有 
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = 0
              dictItem.tagResB = 0
            }
            dictItem.tagRes = 0
          } else {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = 0
              dictItem.tagResB = 0
            }
            dictItem.tagRes = 0
          }
          break

        case 'radius':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Radius))))
            });
            dictItem.tagRes = tagRes
          }

          break
        case 'curvature':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x?.m_Bul)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x?.m_Bul)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_Bul) || 0))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x?.m_Bul))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x?.m_Bul))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x?.m_Bul))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => {
          //     let arr = e.blockDetail.points.map((x) => x.curve)
          //     return arr
          //   })
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => {
          //       let arr = e.blockDetail.points.map((x) => x.curve)
          //       return arr
          //     }
          //     ))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'pointId':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map((x, i) => i)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'length':
          // 成品轮廓 count
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderFinal.map(x => outPutFloatNumberByConfig(x.m_StartPoint.m_Length))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'width':
          // 没有
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.cutWidth)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.cutWidth
          //     ))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'thickness':
          // 没有
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.thickness)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.thickness
          //     ))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        default:
          break;
      }
      break

    //#endregion 轮廓
    // 开料轮廓
    //#region 开料轮廓
    case 'cuttingoutline':
      switch (dictItem.itemKey) {

        case 'count':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null)
              dictItem.tagResB = sourceB.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null)
            }
            dictItem.tagRes = source.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.border) ? e.blockDetail.borderContour.border.length : null))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'referenceOffsetY':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'referenceOffsetX':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
                ))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
                ))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
              ))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'id':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => i)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'process':
          // 加工  process

          break
        case 'radius':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResB = tagResB
            }

            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_pre':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature_pre':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_pre':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }

            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_pre':
          // 开料轮廓 cuttingoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.border.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break


        default:
          break;
      }
      break

    //#endregion 开料轮廓
    // 
    //#region 胚料轮廓
    case 'rawoutline':
      switch (dictItem.itemKey) {

        case 'count':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null)
              dictItem.tagResB = sourceB.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null)
            }
            dictItem.tagRes = source.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null))
              });
              dictItem.tagResA = tagResA


              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => Array.isArray(e?.blockDetail?.borderContour?.borderOrg) ? e.blockDetail.borderContour.borderOrg.length : null))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'referenceOffsetY':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
              });
              dictItem.tagResA = tagResA


              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_Y - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_Y))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'referenceOffsetX':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
                ))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
                ))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg[0].m_StartPoint.m_X - e?.blockDetail?.borderContour?.borderFinal[0].m_StartPoint.m_X
              ))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'id':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => i)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'process':

          break
        case 'radius':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
              });
              dictItem.tagResB = tagResB
            }

            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
              dictItem.tagResB = sourceB.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
            }
            dictItem.tagRes = source.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'radius_pre':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Radius))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature_pre':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }

            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_Bul))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_pre':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_X))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_pre':
          // 胚料轮廓 rawoutline
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              })
              dictItem.tagResB = sourceB.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              })
            }
            dictItem.tagRes = source.map(e => {
              let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
              arr.unshift(0)
              arr.pop()
              return arr
            })
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                  arr.unshift(0)
                  arr.pop()
                  return arr
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => {
                let arr = e?.blockDetail?.borderContour?.borderOrg.map((x, i) => outPutFloatNumberByConfig(x.m_StartPoint.m_Y))
                arr.unshift(0)
                arr.pop()
                return arr
              }))
            });
            dictItem.tagRes = tagRes
          }
          break
        default:
          break;
      }
      break

    //#endregion 胚料轮廓

    //#region 垂直孔
    case 'holes':

      switch (dictItem.itemKey) {
        case 'count':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.length)
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.length)
            }
            dictItem.tagRes = source.map(e => e.holeListFaceA.length + e.holeListFaceB.length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeListFaceA.length + e.holeListFaceB.length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceA':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.length)
              dictItem.tagResB = sourceB.map(e => e.holeListFaceB.length)
            }
            dictItem.tagRes = source.map(e => e.holeListFaceA.length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceB.length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeListFaceA.length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceB':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceB.length)
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.length)
            }
            dictItem.tagRes = source.map(e => e.holeListFaceB.length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceB.length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.holeListFaceB.length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'countThrough':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              // 穿孔会合并到 block.holeListFaceA 里面  而 blockDetail 里面的holeListFaceA  不含穿孔 相减得到穿孔数
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.length - e.blockDetail.holeListFaceA.length)
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.length - e.blockDetail.holeListFaceB.length)
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountThrough)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.length - e.blockDetail.holeListFaceA.length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.length - e.blockDetail.holeListFaceB.length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountThrough))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'id':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => b))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => b))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => b))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => b)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => b)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => b)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'type':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c.holeType))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c.holeType))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.holeType))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.holeType)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.holeType)))
              });
              dictItem.tagResB = tagResB

            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.holeType)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'name':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.blockName)
              dictItem.tagResB = sourceB.map(e => e.blockName)
            }
            dictItem.tagRes = source.map(e => e.blockName)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockName))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockName))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockName))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'face':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c.face))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c.face))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.face))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.face)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.face)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.face)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'direction':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.placeDirection)
              dictItem.tagResB = sourceB.map(e => e.placeDirection)
            }
            dictItem.tagRes = source.map(e => e.placeDirection)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.placeDirection))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.placeDirection))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.placeDirection))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'diameter':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c.radius * 2))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c.radius * 2))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.radius * 2))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.radius * 2)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.radius * 2)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.radius * 2)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'diameterInner':
          // 垂直孔 holes
          break
        case 'depth':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c.depth))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c.depth))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.depth)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c.depth)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.depth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'depthInner':
          // 垂直孔 holes
          break

        case 'toolName':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeName))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeName)))
            });
            dictItem.tagRes = tagRes
          }
          break
        // 刀号
        case 'toolNo':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeName))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeName)))
            });
            dictItem.tagRes = tagRes
          }
          break
        // 轴号
        case 'toolBelongNo':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c.knife.axleId))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c.knife.axleId))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.knife.axleId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeName)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeName)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStartCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStartCode || ''))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStartCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.axisStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStartCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStartCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.axisStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStartCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStartCode))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStartCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStartCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStartCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStartCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStartCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupStartCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupStartCode || ''))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupStartCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.holesGroupStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupStartCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupStartCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.holesGroupStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupEndCode':
          // 垂直孔 holes

          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupEndCode || ''))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupEndCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.holesGroupEndCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupEndCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.holesGroupEndCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.holesGroupEndCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeEndCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'knifeStopCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.knifeStopCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.knifeStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'axisStopCode':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStopCode))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStopCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.axisStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStopCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => c?.knife?.axisStopCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c?.knife?.axisStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'X_start':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.pointX))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.pointX)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Y_start':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointY)))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.pointY))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.pointY)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Z_start':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              }))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => {
              let val = 0
              if (originZ0Position == 0) {
                val = -c.pointZ - e.thickness
              } else {
                val = -c.pointZ
              }
              return outPutFloatNumberByConfig(val)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => {
                  let val = 0
                  if (originZ0Position == 0) {
                    val = -c.pointZ - e.thickness
                  } else {
                    val = -c.pointZ
                  }
                  return outPutFloatNumberByConfig(val)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => {
                  let val = 0
                  if (originZ0Position == 0) {
                    val = -c.pointZ - e.thickness
                  } else {
                    val = -c.pointZ
                  }
                  return outPutFloatNumberByConfig(val)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'X_end':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.pointX))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.pointX)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Y_end':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c) => outPutFloatNumberByConfig(c.pointY)))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c) => outPutFloatNumberByConfig(c.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c) => outPutFloatNumberByConfig(c.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.holeListFaceA.map((c) => outPutFloatNumberByConfig(c.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.holeListFaceA.map((c, b) => outPutFloatNumberByConfig(c.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => outPutFloatNumberByConfig(c.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => c.pointY))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => c.pointY)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Z_end':
          // 垂直孔 holes
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.holeListFaceA.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              }))
              dictItem.tagResB = sourceB.map(e => e.holeListFaceA.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holes.map((c, b) => {
              let val = 0
              if (originZ0Position == 0) {
                val = -c.pointZ - e.thickness
              } else {
                val = -c.pointZ
              }
              return outPutFloatNumberByConfig(val)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => {
                  let val = 0
                  if (originZ0Position == 0) {
                    val = -c.pointZ - e.thickness
                  } else {
                    val = -c.pointZ
                  }
                  return outPutFloatNumberByConfig(val)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => {
                  let val = 0
                  if (originZ0Position == 0) {
                    val = -c.pointZ - e.thickness
                  } else {
                    val = -c.pointZ
                  }
                  return outPutFloatNumberByConfig(val)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holes.map((c, b) => {
                let val = 0
                if (originZ0Position == 0) {
                  val = -c.pointZ - e.thickness
                } else {
                  val = -c.pointZ
                }
                return outPutFloatNumberByConfig(val)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        // case 'toolRotateX_end':

        //   break
        // case 'toolRotateY_end':

        //   break
        // case 'toolRotateZ_end':

        //   break
        default:
          break;
      }
      break

    //#endregion 垂直孔
    //#region  侧孔
    case 'sideHoles':
      switch (dictItem.itemKey) {

        case 'count':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeCountSide
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeCountSide
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeCountSide
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountSide)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeCountSide
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeCountSide
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeCountSide
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountSide))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'countSideL':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return e.blockDetail.holeCountLeft
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeCountLeft
                }

                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeCountLeft
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountLeft)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeCountLeft
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeCountLeft
                  }

                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeCountLeft
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountLeft))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.holeCountSideLeft)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.holeCountSideLeft))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'countSideR':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return e.blockDetail.holeCountRight
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeCountRight
                }

                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeCountRight
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountRight)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeCountRight
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeCountRight
                  }

                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeCountRight
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountRight))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.holeCountSideRight)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.holeCountSideRight))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'countSideU':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return e.blockDetail.holeCountTop
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeCountTop
                }

                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeCountTop
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountTop)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeCountTop
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeCountTop
                  }

                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeCountTop
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountTop))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.holeCountSideTop)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.holeCountSideTop))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'countSideD':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return e.blockDetail.holeCountBottom
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeCountBottom
                }

                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeCountBottom
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeCountBottom)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeCountBottom
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeCountBottom
                  }

                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeCountBottom
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeCountBottom))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.holeCountSideBottom)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.holeCountSideBottom))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'countBevelled':
          // 侧孔 sideHoles
          break
        case 'countLocking':
          // 侧孔 sideHoles
          break
        case 'countBuildIn':
          // 侧孔 sideHoles
          break
        case 'countHinge':
          // 侧孔 sideHoles
          break

        case 'id':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => b)
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => b)
                }


              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => b)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => b))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => b)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => b)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => b)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => b)))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   if (drillConfig.value?.isFileSort == true) {
          //     dictItem.tagResA = sourceA.map(e => e.blockDetail.holeListSide.map((c, b) => b))
          //   }
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => b))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => b)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'type':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.holeType))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c.holeType)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.holeType)))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.cncCanNotProcessType))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.cncCanNotProcessType)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'name':
          // 侧孔 sideHoles
          // 先不管
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockName)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockName))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'face':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.sideFace))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c.sideFace)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.sideFace)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'direction':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.direct)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.direct)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c.direct)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.direct))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.direct)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.direct)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c.direct)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.direct)))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.placeDirection)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.placeDirection))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'diameter':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.radius * 2))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c.radius * 2)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.radius * 2)))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.radius * 2))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.radius * 2)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'diameterInner':
          // 侧孔 sideHoles
          break
        case 'depth':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.depth)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c.depth)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c.depth)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.depth)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.depth)))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.depth))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.depth)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'depthInner':
          // 侧孔 sideHoles
          break

        case 'toolName':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeName)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolNo':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStartCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'knifeStartCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStartCode)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'holesGroupStartCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'holesGroupEndCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.holesGroupEndCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStopCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.knifeStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStopCode':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axisStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolBelongNo':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c?.knife?.axleId)))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'X_start':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointX))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointX)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Y_start':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointY))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointY)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break
        case 'Z_start':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointZ))
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => c.pointZ)))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break

        case 'X_end':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointX2))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'Y_end':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointY2))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'Z_end':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c, b) => outPutFloatNumberByConfig(c?.pointZ))))
            });
            dictItem.tagRes = tagRes
          }

          break

        case 'X_safe':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointX = c?.pointX
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        // _pointX = 
                        break;
                      case 1:
                        // 右侧孔
                        _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                        break;
                      case 2:
                        // 上侧孔
                        break;
                      case 3:
                        // 左侧孔
                        _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })

                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointX = c?.pointX
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        // _pointX = 
                        break;
                      case 1:
                        // 右侧孔
                        _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                        break;
                      case 2:
                        // 上侧孔
                        break;
                      case 3:
                        // 左侧孔
                        _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointX = c?.pointX
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        // _pointX = 
                        break;
                      case 1:
                        // 右侧孔
                        _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                        break;
                      case 2:
                        // 上侧孔
                        break;
                      case 3:
                        // 左侧孔
                        _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c) => {
              let _pointX = c?.pointX
              switch (c.direct) {
                case 0:
                  // 下侧孔
                  // _pointX = 
                  break;
                case 1:
                  // 右侧孔
                  _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                  break;
                case 2:
                  // 上侧孔
                  break;
                case 3:
                  // 左侧孔
                  _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointX = c?.pointX
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          // _pointX = 
                          break;
                        case 1:
                          // 右侧孔
                          _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                          break;
                        case 2:
                          // 上侧孔
                          break;
                        case 3:
                          // 左侧孔
                          _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointX = c?.pointX
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          // _pointX = 
                          break;
                        case 1:
                          // 右侧孔
                          _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                          break;
                        case 2:
                          // 上侧孔
                          break;
                        case 3:
                          // 左侧孔
                          _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointX = c?.pointX
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          // _pointX = 
                          break;
                        case 1:
                          // 右侧孔
                          _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                          break;
                        case 2:
                          // 上侧孔
                          break;
                        case 3:
                          // 左侧孔
                          _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c) => {
                let _pointX = c?.pointX
                switch (c.direct) {
                  case 0:
                    // 下侧孔
                    // _pointX = 
                    break;
                  case 1:
                    // 右侧孔
                    _pointX = parseFloat(_pointX) + drillConfig.value.horiSafeSize
                    break;
                  case 2:
                    // 上侧孔
                    break;
                  case 3:
                    // 左侧孔
                    _pointX = parseFloat(_pointX) - drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'Y_safe':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointY = c?.pointY
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        // 右侧孔

                        break;
                      case 2:
                        // 上侧孔
                        _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                        break;
                      case 3:
                        // 左侧孔
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointY = c?.pointY
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        // 右侧孔

                        break;
                      case 2:
                        // 上侧孔
                        _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                        break;
                      case 3:
                        // 左侧孔
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c) => {
                    let _pointY = c?.pointY
                    switch (c.direct) {
                      case 0:
                        // 下侧孔
                        _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        // 右侧孔

                        break;
                      case 2:
                        // 上侧孔
                        _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                        break;
                      case 3:
                        // 左侧孔
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c) => {
              let _pointY = c?.pointY
              switch (c.direct) {
                case 0:
                  // 下侧孔
                  _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                  break;
                case 1:
                  // 右侧孔

                  break;
                case 2:
                  // 上侧孔
                  _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize
                  break;
                case 3:
                  // 左侧孔
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointY = c?.pointY
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          // 右侧孔

                          break;
                        case 2:
                          // 上侧孔
                          _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                          break;
                        case 3:
                          // 左侧孔
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointY = c?.pointY
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          // 右侧孔

                          break;
                        case 2:
                          // 上侧孔
                          _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                          break;
                        case 3:
                          // 左侧孔
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c) => {
                      let _pointY = c?.pointY
                      switch (c.direct) {
                        case 0:
                          // 下侧孔
                          _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          // 右侧孔

                          break;
                        case 2:
                          // 上侧孔
                          _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                          break;
                        case 3:
                          // 左侧孔
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c) => {
                let _pointY = c?.pointY
                switch (c.direct) {
                  case 0:
                    // 下侧孔
                    _pointY = parseFloat(_pointY) - drillConfig.value.horiSafeSize
                    break;
                  case 1:
                    // 右侧孔

                    break;
                  case 2:
                    // 上侧孔
                    _pointY = parseFloat(_pointY) + drillConfig.value.horiSafeSize

                    break;
                  case 3:
                    // 左侧孔
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'Z_safe':
          // 侧孔 sideHoles
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.holeListSide.map((c) => outPutFloatNumberByConfig(c?.pointZ))))
            });
            dictItem.tagRes = tagRes
          }
          break

        default:
          break;
      }
      break

    //#endregion 侧孔

    //#region 垂直造型 
    case 'models':

      switch (dictItem.itemKey) {
        case 'count':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).length)
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceA':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).length)
            }
            dictItem.tagRes = source.map(e => e.modelListFaceA.filter(x => x.isRect == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceB':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceB.filter(x => x.isRect == false).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceB.filter(x => x.isRect == false).length)
            }
            dictItem.tagRes = source.map(e => e.modelListFaceB.filter(x => x.isRect == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceB.filter(x => x.isRect == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceB.filter(x => x.isRect == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelListFaceB.filter(x => x.isRect == false).length))
            });
            dictItem.tagRes = tagRes
          }

          break

        case 'countThrough':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true && x.depth >= x.thickness).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false && x.depth >= x.thickness).length)
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListThrough.filter(x => x.isRect == false && x.depth >= x.thickness).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false && x.depth >= x.thickness).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false && x.depth >= x.thickness).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListThrough.filter(x => x.isRect == false && x.depth >= x.thickness)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'count2V':
          // 垂直造型 models

          break
        case 'count3V':
          // 垂直造型 models
          break
        case 'id':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelId))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelId))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelId)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelId)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelId)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'face':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.face))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.face))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.face))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.face)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.face)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.face)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'width':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelWidth))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelWidth))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelWidth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelWidth)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelWidth)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'length':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelLength))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelLength))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelLength))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelLength)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.modelLength)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.modelLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'depth':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.depth))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.depth))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.depth)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.depth)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.depth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        // case 'widthExtend':
        //   break
        // case 'lengthExtend':
        //   break
        // case 'depthExtend':
        //   break

        case 'toolName':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeName || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolNo':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axleId || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axleId || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolBelongNo':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axleId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axleId)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axleId)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolDiameter':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.diameter))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.diameter))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.diameter))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.diameter)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.diameter)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.diameter)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'axisStartCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axisStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStartCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupStartCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupEndCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.holesGroupEndCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStopCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.knifeStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'axisStopCode':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.axisStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_x':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_y':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_z':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'tangentAngle':
          // 没有
          break
        case 'isletCount':
          // 没有
          break
        case 'pointX_start':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))

              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))

            }
            dictItem.tagRes = source.map(e => e.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))

          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))

            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_start':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_start':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)

              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }
              ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[0]) {
                    return ''
                  }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[0]) {
                    return ''
                  }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_end':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_end':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_end':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }

                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)

              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.realPointList[model?.realPointList.length - 1]
                // let rd = - c.depth
                // if (c.face == 1) {
                //   rd = c.depth - e.thickness
                // }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }
              ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              if (!model.realPointList[model?.realPointList.length - 1]) {
                return ''
              }
              // let c = model.realPointList[model?.realPointList.length - 1]
              // let rd = - c.depth
              // if (c.face == 1) {
              //   rd = c.depth - e.thickness
              // }
              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.realPointList[model?.realPointList.length - 1]
                  // let rd = - c.depth
                  // if (c.face == 1) {
                  //   rd = c.depth - e.thickness
                  // }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.realPointList[model?.realPointList.length - 1]
                  // let rd = - c.depth
                  // if (c.face == 1) {
                  //   rd = c.depth - e.thickness
                  // }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.realPointList[model?.realPointList.length - 1]
                // let rd = - c.depth
                // if (c.face == 1) {
                //   rd = c.depth - e.thickness
                // }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {

                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                }))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                }))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'radius':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'radius_start':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature_start':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_end':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.radius))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'curvature_end':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.curve))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_pre':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
                arr.unshift(firstData)
                return arr
              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
                arr.unshift(firstData)
                return arr
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
              arr.pop()
              let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
              arr.unshift(firstData)
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
                  arr.pop()
                  let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
                  arr.unshift(firstData)
                  return arr
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
                  arr.pop()
                  let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
                  arr.unshift(firstData)
                  return arr
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointX)
                arr.unshift(firstData)
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointY_pre':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
                arr.unshift(firstData)
                return arr
              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
                arr.unshift(firstData)
                return arr
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
              arr.pop()
              let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
              arr.unshift(firstData)
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
                  arr.pop()
                  let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
                  arr.unshift(firstData)
                  return arr
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
                  arr.pop()
                  let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
                  arr.unshift(firstData)
                  return arr
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].pointY)
                arr.unshift(firstData)
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_pre':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => {
                  let c = point
                  let rd = - c.depth
                  if (model.face == 1) {
                    rd = c.depth - e.thickness
                  }
                  return outPutFloatNumberByConfig(rd)
                })
                arr.pop()
                let firstData = - model.realPointList[0].depth
                if (model.face == 1) {
                  firstData = model.realPointList[0].depth - model.thickness
                }
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr

              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => {
                  let c = point
                  let rd = - c.depth
                  if (model.face == 1) {
                    rd = c.depth - e.thickness
                  }
                  return outPutFloatNumberByConfig(rd)
                })
                arr.pop()
                let firstData = - model.realPointList[0].depth
                if (model.face == 1) {
                  firstData = model.realPointList[0].depth - model.realPointList[0].pointY.thickness
                }
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr

              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => {
                let c = point
                let rd = - c.depth
                if (model.face == 1) {
                  rd = c.depth - e.thickness
                }
                return outPutFloatNumberByConfig(rd)
              })
              arr.pop()
              let firstData = - model.realPointList[0].depth
              if (model.face == 1) {
                firstData = model.realPointList[0].depth - model.realPointList[0].pointY.thickness
              }
              arr.unshift(outPutFloatNumberByConfig(firstData))
              return arr

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = - c.depth
                    if (model.face == 1) {
                      rd = c.depth - e.thickness
                    }
                    return outPutFloatNumberByConfig(rd)
                  })
                  arr.pop()
                  let firstData = - model.realPointList[0].depth
                  if (model.face == 1) {
                    firstData = model.realPointList[0].depth - model.realPointList[0].pointY.thickness
                  }
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr

                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = - c.depth
                    if (model.face == 1) {
                      rd = c.depth - e.thickness
                    }
                    return outPutFloatNumberByConfig(rd)
                  })
                  arr.pop()
                  let firstData = - model.realPointList[0].depth
                  if (model.face == 1) {
                    firstData = model.realPointList[0].depth - model.realPointList[0].pointY.thickness
                  }
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr

                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => {
                  let c = point
                  let rd = - c.depth
                  if (model.face == 1) {
                    rd = c.depth - e.thickness
                  }
                  return outPutFloatNumberByConfig(rd)
                })
                arr.pop()
                let firstData = - model.realPointList[0].depth
                if (model.face == 1) {
                  firstData = model.realPointList[0].depth - model.realPointList[0].pointY.thickness
                }
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr

              })))
            });
            dictItem.tagRes = tagRes
          }
          break;
        case 'radius_pre':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                arr.pop()
                let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                arr.pop()
                let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
              arr.pop()
              let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
              arr.unshift(outPutFloatNumberByConfig(firstData))
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                  arr.pop()
                  let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                  arr.pop()
                  let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                arr.pop()
                let firstData = parseFloat(Number.isNaN(model.realPointList[0].radius) ? 0 : model.realPointList[0].radius)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature_pre':
          // 垂直造型 models
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                arr.pop()
                let firstData = parseFloat(model.realPointList[0].curve)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                arr.pop()
                let firstData = parseFloat(model.realPointList[0].curve)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              }))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
              arr.pop()
              let firstData = parseFloat(model.realPointList[0].curve)
              arr.unshift(outPutFloatNumberByConfig(firstData))
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                  arr.pop()
                  let firstData = parseFloat(model.realPointList[0].curve)
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == false).map(model => {
                  let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                  arr.pop()
                  let firstData = parseFloat(model.realPointList[0].curve)
                  arr.unshift(outPutFloatNumberByConfig(firstData))
                  return arr
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                arr.pop()
                let firstData = parseFloat(model.realPointList[0].curve)
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        default:
          break;
      }

      break

    //#endregion 垂直造型

    //#region  侧面造型
    case 'modelsH':
      switch (dictItem.itemKey) {
        case 'count':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                }
                if (!flag) {
                  return 0
                }

              })
              dictItem.tagResB = sourceA.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'countSideL':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 3).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideR':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 1).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideU':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 2).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideD':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 0).length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'countSideLeftBottom':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 10).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideRightBottom':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 11).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideRightTop':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 12).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideLeftTop':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false && x.direction == 13).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'id':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelId)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'face':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.direction)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'width':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'length':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.modelLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'depth':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.depth)))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'toolName':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeName || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolBelongNo':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolNo':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axleId || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolDiameter':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                  }

                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.diameter || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'axisStartCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStartCode || '' || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'knifeStartCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'holesGroupStartCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
              if (c?.knife?.holesGroupStartCode || '') {
                return c?.knife?.holesGroupStartCode || ''
              } else {
                return ''
              }
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                if (c?.knife?.holesGroupStartCode || '') {
                  return c?.knife?.holesGroupStartCode || ''
                } else {
                  return ''
                }
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'holesGroupEndCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
              if (c?.knife?.holesGroupEndCode || '') {
                return c?.knife?.holesGroupEndCode || ''
              } else {
                return ''
              }
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => {
                if (c?.knife?.holesGroupEndCode || '') {
                  return c?.knife?.holesGroupEndCode || ''
                } else {
                  return ''
                }
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'knifeStopCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.knifeStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStopCode':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c?.knife?.axisStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_x':
          // 凹槽 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
              dictItem.tagResB = sourceB.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_y':
          // 凹槽 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
              dictItem.tagResB = sourceB.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_z':
          // 凹槽 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
              dictItem.tagResB = sourceB.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model?.knife?.offsetZ)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifePointCount':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map((c, b) => c.realPointList.length || 0)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_start':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

              let _pointX = model?.realPointList[0]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                let _pointX = model?.realPointList[0]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_start':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查）  
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let _pointY = model?.realPointList[0]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.filter(x => x.isRect == false).map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.filter(x => x.isRect == false).map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.filter(x => x.isRect == false).map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointY = model?.realPointList[0]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)

              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_start':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let c = model.realPointList[0]
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                      return outPutFloatNumberByConfig(rd)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let c = model.realPointList[0]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })

                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })

                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })

                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }

                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              if (!model.modelEndPoint) {
                return ''
              }
              let c = model.modelEndPoint

              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }

                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.modelEndPoint
                  // let rd = - c.z || -c.pointZ
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                  if (!flag) {
                    return outPutFloatNumberByConfig(rd)
                  }
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.modelEndPoint
                // let rd = - c.z || -c.pointZ
                let c = model.realPointList[model?.realPointList.length - 1]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
              let _pointX = point.pointX
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + model.depth
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - model.depth
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {

                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                let _pointX = point.pointX
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
              let _pointY = point.pointY
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                let _pointY = point.pointY
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return (

                    e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return (e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)
                  })))
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return (e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                    return outPutFloatNumberByConfig(rd)
                  })))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
              let c = point
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false


              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return (e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return (e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                      if (!flag) {
                        return outPutFloatNumberByConfig(rd)
                      }
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.map(point => {
                let c = point
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break;

        case 'curvature_start':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_start':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_safe':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

              let _pointX = model?.realPointList[0]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[0]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {

                let _pointX = model?.realPointList[0]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[0]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_safe':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let _pointY = model?.realPointList[0]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[0]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY - drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY + drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.filter(x => x.isRect == false).map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.filter(x => x.isRect == false).map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.filter(x => x.isRect == false).map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let _pointY = model?.realPointList[0]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[0]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY - drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY + drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)

              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_safe':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let c = model.realPointList[0]
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                      return outPutFloatNumberByConfig(rd)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let c = model.realPointList[0]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature_safe':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.curve))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_safe':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.radius))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_safe_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_safe_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;

                        return outPutFloatNumberByConfig(_pointY)
                    }
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY - drillConfig.value.horiSafeSize
                  break;
                case 1:
                  break;
                case 2:
                  _pointY = _pointY + drillConfig.value.horiSafeSize
                  break;
                case 3:
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY - drillConfig.value.horiSafeSize
                    break;
                  case 1:
                    break;
                  case 2:
                    _pointY = _pointY + drillConfig.value.horiSafeSize
                    break;
                  case 3:
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_safe_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }

                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              if (!model.modelEndPoint) {
                return ''
              }
              let c = model.modelEndPoint

              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }

                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.modelEndPoint
                  // let rd = - c.z || -c.pointZ
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                  if (!flag) {
                    return outPutFloatNumberByConfig(rd)
                  }
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.modelEndPoint
                // let rd = - c.z || -c.pointZ
                let c = model.realPointList[model?.realPointList.length - 1]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'curvature_safe_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_safe_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'curvature_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve)) || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.curve) || ''))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_end':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius)) || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => parseFloat(parseFloat(outPutFloatNumberByConfig(model?.realPointList[model.realPointList.length - 1]?.radius))) || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'curvature':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_pre':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointX)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_pre':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.pointY)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_pre':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = - c.depth
                      if (model.face == 1) {
                        rd = c.depth - e.thickness
                      }
                      return outPutFloatNumberByConfig(rd)
                    })
                    arr.pop()
                    let firstData = - model.realPointList[0].depth
                    if (model.face == 1) {
                      firstData = model.realPointList[0].depth - e.thickness
                    }
                    arr.unshift(outPutFloatNumberByConfig(firstData))
                    return arr

                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = - c.depth
                      if (model.face == 1) {
                        rd = c.depth - e.thickness
                      }
                      return outPutFloatNumberByConfig(rd)
                    })
                    arr.pop()
                    let firstData = - model.realPointList[0].depth
                    if (model.face == 1) {
                      firstData = model.realPointList[0].depth - e.thickness
                    }
                    arr.unshift(outPutFloatNumberByConfig(firstData))
                    return arr

                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = - c.depth
                      if (model.face == 1) {
                        rd = c.depth - e.thickness
                      }
                      return outPutFloatNumberByConfig(rd)
                    })
                    arr.pop()
                    let firstData = - model.realPointList[0].depth
                    if (model.face == 1) {
                      firstData = model.realPointList[0].depth - e.thickness
                    }
                    arr.unshift(outPutFloatNumberByConfig(firstData))
                    return arr

                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => {
                let c = point
                let rd = - c.depth
                if (model.face == 1) {
                  rd = c.depth - e.thickness
                }
                return outPutFloatNumberByConfig(rd)
              })
              arr.pop()
              let firstData = - model.realPointList[0].depth
              if (model.face == 1) {
                firstData = model.realPointList[0].depth - e.thickness
              }
              arr.unshift(outPutFloatNumberByConfig(firstData))
              return arr

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => {
                        let c = point
                        let rd = - c.depth
                        if (model.face == 1) {
                          rd = c.depth - e.thickness
                        }
                        return outPutFloatNumberByConfig(rd)
                      })
                      arr.pop()
                      let firstData = - model.realPointList[0].depth
                      if (model.face == 1) {
                        firstData = model.realPointList[0].depth - e.thickness
                      }
                      arr.unshift(outPutFloatNumberByConfig(firstData))
                      return arr

                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => {
                        let c = point
                        let rd = - c.depth
                        if (model.face == 1) {
                          rd = c.depth - e.thickness
                        }
                        return outPutFloatNumberByConfig(rd)
                      })
                      arr.pop()
                      let firstData = - model.realPointList[0].depth
                      if (model.face == 1) {
                        firstData = model.realPointList[0].depth - e.thickness
                      }
                      arr.unshift(outPutFloatNumberByConfig(firstData))
                      return arr

                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => {
                        let c = point
                        let rd = - c.depth
                        if (model.face == 1) {
                          rd = c.depth - e.thickness
                        }
                        return outPutFloatNumberByConfig(rd)
                      })
                      arr.pop()
                      let firstData = - model.realPointList[0].depth
                      if (model.face == 1) {
                        firstData = model.realPointList[0].depth - e.thickness
                      }
                      arr.unshift(outPutFloatNumberByConfig(firstData))
                      return arr

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => {
                  let c = point
                  let rd = - c.depth
                  if (model.face == 1) {
                    rd = c.depth - e.thickness
                  }
                  return outPutFloatNumberByConfig(rd)
                })
                arr.pop()
                let firstData = - model.realPointList[0].depth
                if (model.face == 1) {
                  firstData = model.realPointList[0].depth - e.thickness
                }
                arr.unshift(outPutFloatNumberByConfig(firstData))
                return arr

              })))
            });
            dictItem.tagRes = tagRes
          }
          break;

        case 'curvature_pre':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                    arr.unshift(firstData)
                    return arr
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                    arr.unshift(firstData)
                    return arr
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                    arr.unshift(firstData)
                    return arr
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
              arr.pop()
              let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
              arr.unshift(firstData)
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                      arr.unshift(firstData)
                      return arr
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                      arr.unshift(firstData)
                      return arr
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                      arr.unshift(firstData)
                      return arr
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.curve))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].curve)
                arr.unshift(firstData)
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'radius_pre':
          // 侧面槽造型 modelsH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                    arr.unshift(firstData)
                    return arr
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                    arr.unshift(firstData)
                    return arr
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                    let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                    arr.pop()
                    let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                    arr.unshift(firstData)
                    return arr
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
              let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
              arr.pop()
              let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
              arr.unshift(firstData)
              return arr
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                      arr.unshift(firstData)
                      return arr
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                      arr.unshift(firstData)
                      return arr
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                      let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                      arr.pop()
                      let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                      arr.unshift(firstData)
                      return arr
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == false).map(model => {
                let arr = model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))
                arr.pop()
                let firstData = outPutFloatNumberByConfig(model.realPointList[0].radius)
                arr.unshift(firstData)
                return arr
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        default:
          break;
      }
      break
    //#endregion 侧面造型

    //#region 侧面槽  

    case 'groovesH':
      switch (dictItem.itemKey) {
        case 'count':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                }
                if (!flag) {
                  return 0
                }
              })
              dictItem.tagResB = sourceA.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                } else {
                  return 0
                }

              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                  }

                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(e => e.isRect == true && e.isTilt == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'countSideL':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                  }
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 3).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideR':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                  }
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 1).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideU':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                  }
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 2).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideD':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 0).length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'countSideLeftBottom':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                  }
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 10).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideRightBottom':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                }
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                  }

                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 11).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideRightTop':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                  }

                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 12).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countSideLeftTop':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                }

                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                }
                if (!flag) {
                  return 0
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                } else {
                  return 0
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                  }
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                  }
                  if (!flag) {
                    return 0
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length
                  } else {
                    return 0
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true && x.direction == 13).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'id':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelId)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'face':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.direction)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'width':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.modelWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'length':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => outPutFloatNumberByConfig(c.modelLength))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'depth':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.depth)))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'toolName':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeName || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolBelongNo':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolNo':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axleId || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolDiameter':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.diameter || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStartCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStartCode || '' || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStartCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStartCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupStartCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupStartCode || '') {
                      return c?.knife?.holesGroupStartCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
              if (c?.knife?.holesGroupStartCode || '') {
                return c?.knife?.holesGroupStartCode || ''
              } else {
                return ''
              }
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupStartCode || '') {
                        return c?.knife?.holesGroupStartCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                if (c?.knife?.holesGroupStartCode || '') {
                  return c?.knife?.holesGroupStartCode || ''
                } else {
                  return ''
                }
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupEndCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                    if (c?.knife?.holesGroupEndCode || '') {
                      return c?.knife?.holesGroupEndCode || ''
                    } else {
                      return ''
                    }
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
              if (c?.knife?.holesGroupEndCode || '') {
                return c?.knife?.holesGroupEndCode || ''
              } else {
                return ''
              }
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                      if (c?.knife?.holesGroupEndCode || '') {
                        return c?.knife?.holesGroupEndCode || ''
                      } else {
                        return ''
                      }
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => {
                if (c?.knife?.holesGroupEndCode || '') {
                  return c?.knife?.holesGroupEndCode || ''
                } else {
                  return ''
                }
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'knifeStopCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.knifeStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStopCode':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c?.knife?.axisStopCode || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_x':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_y':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_z':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)
                }
              })
              dictItem.tagResB = sourceB.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'knifePointCount':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                }
              })

              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map((c, b) => c.realPointList.length || 0)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_start':
          // 侧面槽 groovesH

          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

              let _pointX = model?.realPointList[0]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                let _pointX = model?.realPointList[0]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth // drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_start':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查）  
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              let _pointY = model?.realPointList[0]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.filter(x => x.isRect == true).map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.filter(x => x.isRect == true).map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.filter(x => x.isRect == true).map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointY = model?.realPointList[0]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)

              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_start':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let c = model.realPointList[0]
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                      return outPutFloatNumberByConfig(rd)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let c = model.realPointList[0]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_end':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointX = model?.realPointList[model?.realPointList.length - 1]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_end':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })

                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })

                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })

                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointY = model?.realPointList[model?.realPointList.length - 1]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointZ_end':
          // 侧面槽 groovesH

          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }

                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              if (!model.modelEndPoint) {
                return ''
              }
              let c = model.modelEndPoint

              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }

                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.modelEndPoint
                  // let rd = - c.z || -c.pointZ
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                  if (!flag) {
                    return outPutFloatNumberByConfig(rd)
                  }
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.modelEndPoint
                // let rd = - c.z || -c.pointZ
                let c = model.realPointList[model?.realPointList.length - 1]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointX = point.pointX
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + model.depth
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - model.depth
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  }))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
              let _pointX = point.pointX
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + model.depth
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - model.depth
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {

                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointX = point.pointX
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + model.depth
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - model.depth
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                let _pointX = point.pointX
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + model.depth
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - model.depth
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let _pointY = point.pointY
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  }))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
              let _pointY = point.pointY
              switch (model.direction) {
                case 0:
                  _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let _pointY = point.pointY
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                let _pointY = point.pointY
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY + model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY - model.depth // drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false

              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true

                  return (

                    e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return (e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)
                  })))
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return (e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                    let c = point
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                    return outPutFloatNumberByConfig(rd)
                  })))
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
              let c = point
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false


              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return (e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return (e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })))
                  }

                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.slice(1, -1).map(point => {
                      let c = point
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                      if (!flag) {
                        return outPutFloatNumberByConfig(rd)
                      }
                    }))
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                let c = point
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break;

        case 'pointX_safe':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                    let _pointX = model?.realPointList[0]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

              let _pointX = model?.realPointList[0]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[0]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                      let _pointX = model?.realPointList[0]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {

                let _pointX = model?.realPointList[0]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[0]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_safe':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[0]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[0]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:

                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:

                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)

                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              let _pointY = model?.realPointList[0]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[0]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY - drillConfig.value.horiSafeSize
                  break;
                case 1:

                  break;
                case 2:
                  _pointY = _pointY + drillConfig.value.horiSafeSize
                  break;
                case 3:

                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)

            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.filter(x => x.isRect == true).map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })

                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.filter(x => x.isRect == true).map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[0]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[0]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:

                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:

                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.filter(x => x.isRect == true).map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointY = model?.realPointList[0]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[0]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY - drillConfig.value.horiSafeSize
                    break;
                  case 1:

                    break;
                  case 2:
                    _pointY = _pointY + drillConfig.value.horiSafeSize
                    break;
                  case 3:

                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)

              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_safe':
          // 侧面槽 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }

                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)

                    return outPutFloatNumberByConfig(rd)

                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[0]) {
                      return ''
                    }
                    let c = model.realPointList[0]
                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let c = model.realPointList[0]
              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }

                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                      return outPutFloatNumberByConfig(rd)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      if (!model.realPointList[0]) {
                        return ''
                      }
                      let c = model.realPointList[0]
                      let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)

                      return outPutFloatNumberByConfig(rd)

                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let c = model.realPointList[0]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)

                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_safe_end':
          // 侧面槽造型 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            let flag = false
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                    if (_pointX == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointX = p.x
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:

                        break;
                      case 1:
                        _pointX = _pointX + drillConfig.value.horiSafeSize
                        break;
                      case 2:

                        break;
                      case 3:
                        _pointX = _pointX - drillConfig.value.horiSafeSize
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointX)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
              if (_pointX == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointX = p.x
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:

                  break;
                case 1:
                  _pointX = _pointX + drillConfig.value.horiSafeSize
                  break;
                case 2:

                  break;
                case 3:
                  _pointX = _pointX - drillConfig.value.horiSafeSize
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointX)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                      if (_pointX == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointX = p.x
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:

                          break;
                        case 1:
                          _pointX = _pointX + drillConfig.value.horiSafeSize
                          break;
                        case 2:

                          break;
                        case 3:
                          _pointX = _pointX - drillConfig.value.horiSafeSize
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointX)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointX = model?.realPointList[model.realPointList.length - 1]?.pointX
                if (_pointX == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointX = p.x
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:

                    break;
                  case 1:
                    _pointX = _pointX + drillConfig.value.horiSafeSize
                    break;
                  case 2:

                    break;
                  case 3:
                    _pointX = _pointX - drillConfig.value.horiSafeSize
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointX)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_safe_end':
          // 侧面槽造型 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                    if (_pointY == undefined) {
                      let _p = model?.pointList[model?.pointList.length - 1]
                      let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                      _pointY = p.y
                    }
                    /** 造型方向 -2 未知（是异常情况  要排查） 
                     * -1 斜的  0 下 1右 2上 3左 
                     * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                    switch (model.direction) {
                      case 0:
                        _pointY = _pointY - drillConfig.value.horiSafeSize
                        break;
                      case 1:
                        break;
                      case 2:
                        _pointY = _pointY + drillConfig.value.horiSafeSize
                        break;
                      case 3:
                        break;
                      default:
                        break;
                    }
                    return outPutFloatNumberByConfig(_pointY)
                  })
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
              if (_pointY == undefined) {
                let _p = model?.pointList[model?.pointList.length - 1]
                let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                _pointY = p.y
              }
              /** 造型方向 -2 未知（是异常情况  要排查） 
               * -1 斜的  0 下 1右 2上 3左 
               * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
              switch (model.direction) {
                case 0:
                  _pointY = _pointY - drillConfig.value.horiSafeSize
                  break;
                case 1:
                  break;
                case 2:
                  _pointY = _pointY + drillConfig.value.horiSafeSize
                  break;
                case 3:
                  break;
                default:
                  break;
              }
              return outPutFloatNumberByConfig(_pointY)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => {
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => {
                  if (!flag) {
                    return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                      let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                      if (_pointY == undefined) {
                        let _p = model?.pointList[model?.pointList.length - 1]
                        let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                        _pointY = p.y
                      }
                      /** 造型方向 -2 未知（是异常情况  要排查） 
                       * -1 斜的  0 下 1右 2上 3左 
                       * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                      switch (model.direction) {
                        case 0:
                          _pointY = _pointY - drillConfig.value.horiSafeSize
                          break;
                        case 1:
                          break;
                        case 2:
                          _pointY = _pointY + drillConfig.value.horiSafeSize
                          break;
                        case 3:
                          break;
                        default:
                          break;
                      }
                      return outPutFloatNumberByConfig(_pointY)
                    })
                  }
                }))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                let _pointY = model?.realPointList[model.realPointList.length - 1]?.pointY
                if (_pointY == undefined) {
                  let _p = model?.pointList[model?.pointList.length - 1]
                  let p = BlockHelper.getPlaceXYInBoard(e, _p.pointX, _p.pointY)
                  _pointY = p.y
                }
                /** 造型方向 -2 未知（是异常情况  要排查） 
                 * -1 斜的  0 下 1右 2上 3左 
                 * 10 左下斜  11 右下斜 12 右上斜  13 左上斜 */
                switch (model.direction) {
                  case 0:
                    _pointY = _pointY - drillConfig.value.horiSafeSize
                    break;
                  case 1:
                    break;
                  case 2:
                    _pointY = _pointY + drillConfig.value.horiSafeSize
                    break;
                  case 3:
                    break;
                  default:
                    break;
                }
                return outPutFloatNumberByConfig(_pointY)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ_safe_end':
          // 侧面槽造型 groovesH
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              let flag = false
              dictItem.tagResA = sourceA.map(e => {
                if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }

                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)
                  })
                }
                if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                  flag = true
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                    return outPutFloatNumberByConfig(rd)

                  })
                }

              })
              dictItem.tagResB = sourceB.map(e => {
                if (!flag) {
                  return e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                    if (!model.realPointList[model?.realPointList.length - 1]) {
                      return ''
                    }
                    let c = model.realPointList[model?.realPointList.length - 1]

                    let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                    return outPutFloatNumberByConfig(rd)

                  }
                  )
                }
              })
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
              if (!model.modelEndPoint) {
                return ''
              }
              let c = model.modelEndPoint

              let rd = getProcessItemPointZ(c.pointZ, e.thickness, 2, true)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              let flag = false
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                  if (e.holeListFaceA.length > 0 || e.modelListFaceA.length > 0) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }
                  if (flag == false && (e.holeListFaceA.length == 0 || e.modelListFaceA.length == 0) && (e.blockDetail.holeListSide.length > 0 || e.blockDetail.modelListSide.length > 0)) {
                    flag = true
                    return outPutFloatNumberByConfig(rd)
                  }

                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }
                  // let c = model.modelEndPoint
                  // let rd = - c.z || -c.pointZ
                  let c = model.realPointList[model?.realPointList.length - 1]
                  let rd = getProcessItemPointZ(c.pointZ, e.thickness, 1, true)
                  if (!flag) {
                    return outPutFloatNumberByConfig(rd)
                  }
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListSide.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }
                // let c = model.modelEndPoint
                // let rd = - c.z || -c.pointZ
                let c = model.realPointList[model?.realPointList.length - 1]
                let rd = getProcessItemPointZ(c.pointZ, e.thickness, 0, true)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        default:
          break;
      }
      break

    //#endregion 侧面槽 

    //#region 凹槽
    case 'grooves':
      switch (dictItem.itemKey) {
        case 'count':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length)
              dictItem.tagResB = sourceA.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length)
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.map(x => x.realPointList.length || 0))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(e => e.isRect == true && e.isTilt == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceA':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length)
            }
            dictItem.tagRes = source.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelListFaceA.filter(e => e.isRect == true && e.isTilt == false).length))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'countFaceB':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length)
            }
            dictItem.tagRes = source.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {

              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.modelListFaceB.filter(e => e.isRect == true && e.isTilt == false).length))
            });
            dictItem.tagRes = tagRes
          }
          // if (exportFileType == 0 || exportFileType == 1) {
          //   dictItem.tagRes = source.map(e => e.modelListFaceB.length)
          // } else {
          //   let tagRes: any = [];
          //   source.forEach(material => {
          //     tagRes = tagRes.concat(material.blockList.map(e => e.modelListFaceB.length))
          //   });
          //   dictItem.tagRes = tagRes
          // }
          break

        case 'countThrough':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true && x.depth >= x.thickness).length)
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true && x.depth >= x.thickness).length)
            }
            dictItem.tagRes = source.map(e => e.blockDetail.modelListThrough.filter(x => x.isRect == true && x.depth >= x.thickness).length)
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true && x.depth >= x.thickness).length))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true && x.depth >= x.thickness).length))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.modelListThrough.filter(x => x.isRect == true && x.depth >= x.thickness).length))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'id':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelId))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelId))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelId)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelId)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelId)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'face':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.face))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.face))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.face))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.face)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.face)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.face)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'width':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelWidth))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelWidth))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelWidth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelWidth)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelWidth)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelWidth)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'length':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelLength))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelLength))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelLength))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelLength)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.modelLength)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.modelLength)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'depth':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.depth))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.depth))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.depth))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.depth)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.depth)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.depth)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'toolName':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolNo':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || ''))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeName || '')))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolBelongNo':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axleId))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axleId))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axleId))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axleId)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axleId)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axleId)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'toolDiameter':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.diameter))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.diameter))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.diameter))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.diameter)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.diameter)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.diameter)))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'axisStartCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axisStartCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStartCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeStartCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupStartCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupStartCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'holesGroupEndCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.holesGroupEndCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'knifeStopCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.knifeStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'axisStopCode':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.axisStopCode)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_x':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetX))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetX))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetX))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetX)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_y':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetY))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetY))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetY))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetY)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'redundance_z':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model?.knife?.offsetZ)))
            });
            dictItem.tagRes = tagRes
          }
          break


        case 'knifePointCount':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.length))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.length))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.length))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.length)))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.length)))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.length)))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX_start':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {

            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_start':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.filter(x => x.isRect == true).map(e => e.modelListFaceA.map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.filter(x => x.isRect == true).map(e => e.modelListFaceA.map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.filter(x => x.isRect == true).map(e => e.blockDetail.models.map(model => outPutFloatNumberByConfig(model?.realPointList[0]?.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointZ_start':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)

              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }
              ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => {
              if (!model.realPointList[0]) {
                return ''
              }
              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[0]) {
                    return ''
                  }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[0]) {
                    return ''
                  }
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[0]) {
                  return ''
                }
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointX_end':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointX))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY_end':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY)))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => outPutFloatNumberByConfig(model?.realPointList[model?.realPointList.length - 1]?.pointY))))
            });
            dictItem.tagRes = tagRes
          }
          break

        case 'pointZ_end':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }

                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)

              }))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }

                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }
              ))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => {
              if (!model.realPointList[model?.realPointList.length - 1]) {
                return ''
              }

              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            }))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }

                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => {
                  if (!model.realPointList[model?.realPointList.length - 1]) {
                    return ''
                  }

                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                })))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => {
                if (!model.realPointList[model?.realPointList.length - 1]) {
                  return ''
                }

                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointX':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointX)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointY':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => outPutFloatNumberByConfig(point.pointY)))))
            });
            dictItem.tagRes = tagRes
          }
          break
        case 'pointZ':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              })))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
              let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
              return outPutFloatNumberByConfig(rd)
            })))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                }))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                  let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                  return outPutFloatNumberByConfig(rd)
                }))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.filter(x => x.isRect == true).map(model => model.realPointList.map(point => {
                let rd = getProcessItemPointZ(model.depth, e.thickness, model.face, false)
                return outPutFloatNumberByConfig(rd)
              }))))
            });
            dictItem.tagRes = tagRes
          }
          break;

        case 'radius':
          // 凹槽 grooves
          if (exportFileType == 0 || exportFileType == 1) {
            if (drillConfig.value?.isFileSort == true) {
              dictItem.tagResA = sourceA.map(e => e.modelListFaceA.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
              dictItem.tagResB = sourceB.map(e => e.modelListFaceA.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
            }
            dictItem.tagRes = source.map(e => e.blockDetail.models.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius))))
          } else {
            if (drillConfig.value?.isFileSort == true) {
              let tagResA: any = [];
              sourceA.forEach(material => {
                tagResA = tagResA.concat(material.blockList.map(e => e.modelListFaceA.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
              });
              dictItem.tagResA = tagResA

              let tagResB: any = [];
              sourceB.forEach(material => {
                tagResB = tagResB.concat(material.blockList.map(e => e.modelListFaceA.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
              });
              dictItem.tagResB = tagResB
            }
            let tagRes: any = [];
            source.forEach(material => {
              tagRes = tagRes.concat(material.blockList.map(e => e.blockDetail.models.map(model => model.realPointList.slice(1, -1).map(point => outPutFloatNumberByConfig(point.radius)))))
            });
            dictItem.tagRes = tagRes
          }
          break;

        default:
          break;
      }

      break

    //#endregion
    case 'Islets':
      switch (dictItem.itemKey) {

        case 'xxx':



        default:
          break;
      }
      break
    case 'Hardwares':
      switch (dictItem.itemKey) {

        case 'xxx':



        default:
          break;
      }
      break
    case 'Assemblie':
      switch (dictItem.itemKey) {

        case 'xxx':



        default:
          break;
      }
      break
    case 'machineInfo':
      switch (dictItem.itemKey) {

        case 'isOpen':
          // 没有
          break
        case 'configName':

          dictItem.tagRes = drillConfig.value.name
          break
        case 'brand':

          dictItem.tagRes = drillConfig.value.brand
          break
        // case 'configType':
        //   dictItem.tagRes = drillConfig.value.brand
        //   break
        // case 'remark':

        //   dictItem.tagRes = drillConfig.value.brand
        //   break
        default:
          break;
      }
      break
    case 'process':

      switch (dictItem.itemKey) {
        case 'verticalSafeSize':
          // 垂直安全距离
          let val = outPutFloatNumberByConfig(drillConfig.value.freeHeight + placeData.value.materialList[0].thickness)

          dictItem.tagRes = val
          dictItem.tagResA = val
          dictItem.tagResB = val
          break
        case 'horizontalSafeSize':
          // 水平安全距离
          dictItem.tagRes =  outPutFloatNumberByConfig(drillConfig.value.horiSafeSize)
          dictItem.tagResA = outPutFloatNumberByConfig(drillConfig.value.horiSafeSize)
          dictItem.tagResB = outPutFloatNumberByConfig(drillConfig.value.horiSafeSize)
          break
        case 'freeSpeed':
          // 空程速度
          dictItem.tagRes = drillConfig.value.freeSpeed
          dictItem.tagResA = drillConfig.value.freeSpeed
          dictItem.tagResB = drillConfig.value.freeSpeed
          break
        case 'startSpeed':
          // 初始加工深度
          dictItem.tagRes = drillConfig.value.modelSpeed
          dictItem.tagResA = drillConfig.value.modelSpeed
          dictItem.tagResB = drillConfig.value.modelSpeed
          break

        // case 'modelSpeed':
        //   dictItem.tagRes = drillConfig.value.modelSpeed
        //   break
        // case 'millingSpeed':
        //   dictItem.tagRes = drillConfig.value.millingSpeed
        //   break
        // case 'holeOrModeEnDepth':
        //   dictItem.tagRes = drillConfig.value.holeOrModelDeepen
        //   break
        // case 'throughModelSlantLength':
        //   dictItem.tagRes = drillConfig.value.holeOrModelKnifeDeepen
        //   break
        default:
          break;
      }
      break
    case 'machineBaseInfo':
      switch (dictItem.itemKey) {

        case 'horizontalBaeePoint':
          dictItem.tagRes = drillConfig.value.originZ0Position
          break
        case 'verticalBaeePoint':
          dictItem.tagRes = drillConfig.value.originPointPosition
          break
        case 'lengthDirection':
          dictItem.tagRes = drillConfig.value.lengthSideAxis
          break
        case 'shortDirection':
          dictItem.tagRes = drillConfig.value.widthSideAxis
          break
        case 'zDirection':
          dictItem.tagRes = drillConfig.value.heightAxis
          break
        case 'basePoint':
          dictItem.tagRes = drillConfig.value.processLocation
          break
        case 'boardStyle':
          dictItem.tagRes = drillConfig.value.setBoardStyle
          break
        case 'overLapStyle':
          dictItem.tagRes = drillConfig.value.overLapStyle
          break
        case 'isFourSideFrontBackItem':
          dictItem.tagRes = drillConfig.value.isFourSideFrontBackItem
          break
        default:
          break;
      }
      break
    case 'processHoles':
      switch (dictItem.itemKey) {

        case 'freeSpeed':
          dictItem.tagRes = drillConfig.value?.holeFreeSpeed || ''
          break
        case 'holeSpeed':
          dictItem.tagRes = drillConfig.value?.holeSpeed || ''
          break
        case 'isHoleDelay':
          dictItem.tagRes = drillConfig.value?.isHoleDelay || ''
          break
        case 'holeDelayCode':
          dictItem.tagRes = drillConfig.value?.holeDelayCode || ''
          break
        case 'isPreHoleDelay':
          dictItem.tagRes = drillConfig.value?.isPreHoleDelay || ''
          break
        case 'preHoleDelayCode':
          dictItem.tagRes = drillConfig.value?.preHoleDelayCode || ''
          break
        case 'throughHoleOneTime':
          dictItem.tagRes = drillConfig.value?.throughHoleOneTime || ''
          break
        case 'throughHoleProcessMode':
          dictItem.tagRes = drillConfig.value?.throughHoleProcessMode || ''
          break
        case 'holeEnDepth':
          dictItem.tagRes = drillConfig.value?.throughHoleDeepen || ''
          break
        default:
          break;
      }
      break
    case 'processModel':

      switch (dictItem.itemKey) {

        case 'modelSpeed':
          dictItem.tagRes = drillConfig.value?.modelSpeed || ''
          break
        case 'millingSpeed':
          dictItem.tagRes = drillConfig.value?.millingSpeed || ''
          break
        case 'holeOrModeEnDepth':
          dictItem.tagRes = drillConfig.value?.holeOrModelDeepen || ''
          break
        case 'throughModelSlantLength':
          dictItem.tagRes = drillConfig.value?.holeOrModelKnifeDeepen || ''
          break


        default:
          break;
      }
      break
    case 'processUnRegular':
      switch (dictItem.itemKey) {

        case 'overRun_unRegular':
          dictItem.tagRes = drillConfig.value?.specialShapedBorder || ''
          break



        default:
          break;
      }
      break

    // case 'knifes':
    //   switch (dictItem.itemKey) {

    //     case 'knifeId':
    //       // dictItem.tagRes = knifeId

    //       break



    //     default:
    //       break;
    //   }
    //   break
    case 'DataDefinition':
      switch (dictItem.itemKey) {

        case 'ncFileEncoding':



        default:
          break;
      }
      break
    case 'DataPreDeal':
      switch (dictItem.itemKey) {

        case 'maxGrooveWidth':
          dictItem.tagRes = drillConfig.value?.maxSlotWidth || ''
          break
        case 'minGrooveWidth':
          dictItem.tagRes = drillConfig.value?.maxSlotLength || ''
          break
        case 'GrooveExLength':
          dictItem.tagRes = drillConfig.value?.slotLengthExtend || ''
          break
        case 'GrooveExWidth':
          dictItem.tagRes = drillConfig.value?.slotWidthExtend || ''
          break
        case 'GrooveExDepth':
          dictItem.tagRes = drillConfig.value?.slotDepthExtend || ''
          break

        case 'GrooveIsInside':
          dictItem.tagRes = drillConfig.value?.isSlotLimitInsideBoard || ''
          break
        case 'GrooveDirectionX':
          dictItem.tagRes = drillConfig.value?.slotProcessDirectionY || ''
          break
        case 'GrooveDirectionY':
          dictItem.tagRes = drillConfig.value?.slotProcessDirectionX || ''
          break

        case 'sawProcessContent':
          dictItem.tagRes = drillConfig.value?.slotProcessCut || ''
          break
        case 'arcTurnPolylineLen':
          dictItem.tagRes = drillConfig.value?.arcTurnPolylineLen || ''
          break
        case 'isProcess_throuthhole':
          dictItem.tagRes = drillConfig.value?.isThroughHole2FaceProcess || ''
          break

        case 'isDBProcess_throuthModel':
          dictItem.tagRes = drillConfig.value?.isThroughModel2FaceProcess || ''
          break
        case 'DBProcess_Depth':
          dictItem.tagRes = drillConfig.value?.processDepth2Face || ''
          break
        case 'unRegularKnifeEx':
          dictItem.tagRes = drillConfig.value?.unRegularMillingKnifeExpand || ''
          break
        case 'unRegularKnifeSplite':
          dictItem.tagRes = drillConfig.value?.isUnRegularMillingKnifeSplit || ''
          break
        case 'unRegularKnifeOffset':
          dictItem.tagRes = drillConfig.value?.unRegularMillingKnifeOffset || ''
          break
        case 'unRegularKnifeDBProcess':
          dictItem.tagRes = drillConfig.value?.isUnRegularMillingKnife2FaceProcess || ''
          break
        case 'unRegularKnife_Depth':
          dictItem.tagRes = drillConfig.value?.unRegularMillingKnifeDepth || ''
          break
        case 'unRegularKnifeItem':
          dictItem.tagRes = drillConfig.value?.unRegularMillingKnife || ''
          break
        case 'highProcessContent':
          dictItem.tagRes = drillConfig.value?.advancedCode || ''
          break

        default:
          break;
      }
      break
    case 'DataSourceHandle':
      switch (dictItem.itemKey) {

        case 'boardId':
          dictItem.tagRes = drillConfig.value?.customBoardNo || ''
          break
        case 'decimalPlaces':
          dictItem.tagRes = drillConfig.value?.decimalPointPrecision || ''
          break
        case 'connectToPlaceInfo':
          dictItem.tagRes = drillConfig.value?.relateCuttingInfo || ''
          break
        default:
          break;
      }
      break
    case 'processOtherKnife':
      switch (dictItem.itemKey) {

        case 'OtherKnife_holes':
          dictItem.tagRes = drillConfig.value.isAllowDoubleSplit
          break
        case 'OtherKnife_models':
          dictItem.tagRes = drillConfig.value.isAllowModelSplit
          break
        case 'depth':
          dictItem.tagRes = drillConfig.value.splitDepth
          break
        case 'times':
          dictItem.tagRes = drillConfig.value.splitCount
          break
        default:
          break;
      }
      break
    default:
      break;
  }

  let checkArr = ['outline', 'cuttingoutline', 'rawoutline', 'holes', 'sideHoles', 'grooves', 'groovesH', 'models', 'modelsH'];



  if (dictItem.tagRes == null && dictItem.tagResA == null && dictItem.tagResB == null) {
    console.log('未处理：', dictItem.type, dictItem.itemKey, dictItem.itemLabel)
  } else {
    // console.log('已处理：', dictItem.type, dictItem.itemKey, dictItem.itemLabel, dictItem.tagRes);
  }

  if (drillConfig.value?.isFileSort == true && !checkArr.includes(dictItem.type)) {
    if (dictItem.tagResA == null && dictItem.tagRes != null) {
      dictItem.tagResA = dictItem.tagRes
    }


    if (dictItem.tagResB == null && dictItem.tagRes != null) {
      dictItem.tagResB = dictItem.tagRes
    }
  }

  if (Array.isArray(dictItem.tagResA)) {
    dictItem.tagResA = dictItem.tagResA.filter(e => e != undefined)
  }
  if (Array.isArray(dictItem.tagResB)) {
    dictItem.tagResB = dictItem.tagResB.filter(e => e != undefined)
  }

  // 做个控制  如果没有加工项  则不返回数据  达到不生成文件的目的
  // if(exportFileType == 0 || exportFileType ==1){
  //   let hasAArr = sourceA.map(block=>block.blockDetail.modelListSide.length > 0 || block.block)
  // }else{

  // }

}


/** 根据机台配置 输出小数点位数 相符的数据 */
function outPutFloatNumberByConfig(val) {
  let num = 0
  if (Number.isNaN(val) || !val) {
    num = 0
  }else{
    num = parseFloat(val)
  }
  
  /**
   * decimalPointPrecision 小数点后保留几位
   * fixFloatNumberEndZero 所有数值 后面是否保留小数位
   * intNumberAddDecimalPoint 整数值 是否保留小数位
   */
  const { decimalPointPrecision, fixFloatNumberEndZero, intNumberAddDecimalPoint } = drillConfig.value

  let res: any = ''

  if (decimalPointPrecision) {
    // 有配置小数点位数才进来
    if (fixFloatNumberEndZero) {
      // 所有数值都做保留小数位操作
      let fix = decimalPointPrecision || 0
      res = num.toFixed(decimalPointPrecision)
    } else {
      if (intNumberAddDecimalPoint) {
        if (Number.isInteger(num)) {
          // 只有整数做保留小数位操作
          res = num.toFixed(decimalPointPrecision)
        } else {
          res = num
        }
      }else{
        res = num
      }
    }
  } else {
    res = num
  }


  return res
}

/**
 * 数据转换
 * 将特定字段的值转为实际生产的对应的值
 * @param block 
 * 
 */
export function transformDataValue(block: PlaceBlock, projectList) {

  if (Array.isArray(projectList) && projectList.length > 0) {
    let dict_face = projectList.find(e => e.key == 'face')
    let dict_sideFace = projectList.find(e => e.key == 'sideFace')
    let dict_arcDirection = projectList.find(e => e.key == 'arcDirection')
    let dict_modelKnifeOffset = projectList.find(e => e.key == 'modelKnifeOffset')
    let dict_axisPoint = projectList.find(e => e.key == 'axisPoint')
    let dict_axisZeroPoint = projectList.find(e => e.key == 'axisZeroPoint')


    // 加工面转换
    if (dict_face) {
      for (const i in block.holeListFaceA) {
        let hole = block.holeListFaceA[i]
        block.holeListFaceA[i].face = _transForm(hole.face, dict_face)
      }

      for (const i in block.holeListFaceB) {
        let hole = block.holeListFaceB[i]
        block.holeListFaceB[i].face = _transForm(hole.face, dict_face)
      }

      for (const i in block.blockDetail.holes) {
        let hole = block.blockDetail.holes[i]
        block.blockDetail.holes[i].face = _transForm(hole.face, dict_face)
      }

      for (const i in block.modelListFaceA) {
        let model = block.modelListFaceA[i]
        block.modelListFaceA[i].face = _transForm(model.face, dict_face)
      }

      for (const i in block.modelListFaceB) {
        let model = block.modelListFaceB[i]
        block.modelListFaceB[i].face = _transForm(model.face, dict_face)
      }

      for (const i in block.blockDetail.models) {
        let model = block.blockDetail.models[i]
        block.blockDetail.models[i].face = _transForm(model.face, dict_face)
      }
    }
    // 侧面加工转换
    if (dict_sideFace) {
      for (const i in block.blockDetail.holeListSide) {
        let sideHole = block.blockDetail.holeListSide[i]
        block.blockDetail.holeListSide[i].direct = _transForm(sideHole.direct, dict_sideFace)
      }

      for (const i in block.blockDetail.modelListSide) {
        let sideModel = block.blockDetail.modelListSide[i]
        block.blockDetail.modelListSide[i].direction = _transForm(sideModel.direction, dict_sideFace)
      }
    }

    // 弧线方向
    if (dict_arcDirection) { }
    // 铣型刀路偏置
    if (dict_modelKnifeOffset) { }
    // 参考顶点
    if (dict_axisPoint) { }
    // 平面参考点
    if (dict_axisZeroPoint) { }
  }

  // 侧面加工

}

function _transForm(val, dict) {
  let res = val
  let item = dict.rows.find(e => e.rawValue == val)
  if (item) {
    res = item.targetValue
  }
  return res
}
/**
 * 
 * @param info 节点信息
 * @param loopObj 节点循环依据数据集 定位数据具体位置用的
 * @param i 节点下标
 * @param isCondition_Line 是否是刀路
 * @returns 
 */
function getChangeKnifeCode(info, loopObj, i, isCondition_Line) {
  //  定位加工项
  let conditionSource: any = null
  if (info._flag == 'A') {
    conditionSource = fileDataA.value
  } else if (info._flag == 'B') {
    conditionSource = fileDataB.value
  } else {
    conditionSource = fileDataAB.value
  }
  if (conditionSource == null) {
    return
  }
  let processList: any = []

  switch (loopObj.type) {
    case 'holes':
      // 垂直孔

      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].holeListFaceA
          break;
        case 'B':
          processList = conditionSource[info.pid].holeListFaceA

          break;
        default:
          processList = conditionSource[info.pid].holes
          break;
      }
      break;
    case 'sideHoles':
      // 侧孔
      processList = conditionSource[info.pid].blockDetail.holeListSide
      break;
    case 'models':
      // 垂直造型 
      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].blockDetail.modelListFaceA.filter(e => e.isRect == false)
          break;
        case 'B':
          processList = conditionSource[info.pid].blockDetail.modelListFaceB.filter(e => e.isRect == false)
          break;
        default:
          processList = conditionSource[info.pid].blockDetail.models.filter(e => e.isRect == false)
          break;
      }
      break;
    case 'modelsH':
      // 侧面造型 
      processList = conditionSource[info.pid].blockDetail.modelListSide.filter(e => e.isRect == false)
      break;
    case 'groovesH':
      // 侧面槽
      processList = conditionSource[info.pid].blockDetail.modelListSide.filter(e => e.isRect == true)
      break;
    case 'grooves':
      // 垂直槽

      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].blockDetail.modelListFaceA.filter(e => e.isRect == true)
          break;
        case 'B':
          processList = conditionSource[info.pid].blockDetail.modelListFaceB.filter(e => e.isRect == true)
          break;
        default:
          processList = conditionSource[info.pid].blockDetail.models.filter(e => e.isRect == true)
          break;
      }
      break;
    default:
      break;
  }

  let _item: any = null
  let _index = -1
  if (isCondition_Line) {
    _item = processList[info.kid]
    _index = info.kid
  } else {
    _item = processList[i]
    _index = i
  }

  let allprocessItem = {
    block: conditionSource[info.pid],
    blockNo: conditionSource[info.pid]?.blockNo,
    processItem: _item,
    processFace: info._flag,
    processType: loopObj.type,
    isLine: isCondition_Line,
    index: _index
  }

  if (_item == undefined) {
    console.log('没有对应的加工项', conditionSource, loopObj.type);
    return
  }

  // 输出的代码
  let code = ''


  // 是否要换刀
  let isChangeKnife = false
  // 是否换轴
  let isChangeAxis = false

  // 上个加工项
  let lastProcessItem = allProcessList.value[allProcessList.value.length - 1]



  if (allProcessList.value.length == 0) {
    // 第一个加工项  要调刀
    isChangeKnife = true
    isChangeAxis = true
  } else {
    if (info.exportFileType == 0 || info.exportFileType == 1) {
      if (lastProcessItem.processItem.knife.knifeId == _item?.knife.knifeId
        && allprocessItem.processFace == lastProcessItem.processFace
        && allprocessItem.blockNo == lastProcessItem.blockNo
        && allprocessItem.processFace == lastProcessItem.processFace) {
        // 同一把刀  不要换
        isChangeKnife = false
      } else {
        // 要换刀
        isChangeKnife = true
      }
    } else if (info.exportFileType == 2 || info.exportFileType == 3) {
      if (lastProcessItem.processItem.knife.knifeId == _item?.knife.knifeId
        && allprocessItem.processFace == lastProcessItem.processFace
        && allprocessItem.block.boardId == lastProcessItem.block.boardId) {
        // 同一把刀  不要换
        isChangeKnife = false

      } else {
        // 要换刀
        isChangeKnife = true
      }
    }

    // 要换刀的前提下  判断  轴变化
    if (isChangeKnife == true) {
      if (_item.knife.axleId != lastProcessItem.processItem.knife.axleId) {
        isChangeAxis = true
      }
    }
  }

  // 生成代码
  // 当前加工代码
  //轴启动代码
  let axisStartCode = addNextLineCode(_item.knife.axisStartCode)
  // 刀启动代码
  let knifeStartCode = addNextLineCode(_item.knife.knifeStartCode)
  // 轴停止代码 
  let axisStopCode = addNextLineCode(_item.knife.axisStopCode)
  // 刀停止代码
  let knifeStopCode = addNextLineCode(_item.knife.knifeStopCode)


  function addNextLineCode(str) {
    if (str.includes('\n')) {
      return str
    } else {
      return str + '\n'
    }

  }
  // 上个加工代码

  if (allProcessList.value.length == 0) {
    // 轴启动 + 刀启动
    code = axisStartCode + knifeStartCode
  } else {
    //轴启动代码
    let axisStartCode_last = addNextLineCode(lastProcessItem.processItem.knife.axisStartCode)
    // 刀启动代码
    let knifeStartCode_last = addNextLineCode(lastProcessItem.processItem.knife.knifeStartCode)
    // 轴停止代码 
    let axisStopCode_last = addNextLineCode(lastProcessItem.processItem.knife.axisStopCode)
    // 刀停止代码
    let knifeStopCode_last = addNextLineCode(lastProcessItem.processItem.knife.knifeStopCode)


    if (isChangeKnife == true && isChangeAxis == true) {
      // 轴停止 + 刀停止 + 轴启动 + 刀启动 
      code = axisStopCode_last + knifeStopCode_last + axisStartCode + knifeStartCode
    } else if (isChangeKnife == true && isChangeAxis == false) {
      // 
      code = axisStartCode + knifeStartCode
    }

  }




  return code + '\n'
}

function getKnifeUpCode() {

}


export function checkCondition(conditionList, info, loopObj, i, isCondition_Line) {
  /**
    * 0 需要调刀
    * 1 不需要调刀
    * 2 直线刀路
    * 3 逆时针圆弧刀路
    * 4 顺时针圆弧刀路
    * 5 正面加工
    * 6 反面加工
    */



  //  定位加工项
  let conditionSource = null
  if (info._flag == 'A') {
    conditionSource = fileDataA.value
  } else if (info._flag == 'B') {
    conditionSource = fileDataB.value
  } else {
    conditionSource = fileDataAB.value
  }
  let processList: any = []
  if (conditionSource == null) {
    // 有问题哦
  }
  switch (loopObj.type) {
    case 'holes':
      // 垂直孔

      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].holeListFaceA
          break;
        case 'B':
          processList = conditionSource[info.pid].holeListFaceA

          break;
        default:
          processList = conditionSource[info.pid].holes
          break;
      }
      break;
    case 'sideHoles':
      // 侧孔
      processList = conditionSource[info.pid].blockDetail.holeListSide
      break;
    case 'models':
      // 垂直造型 
      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].blockDetail.modelListFaceA.filter(e => e.isRect == false)
          break;
        case 'B':
          processList = conditionSource[info.pid].blockDetail.modelListFaceB.filter(e => e.isRect == false)
          break;
        default:
          processList = conditionSource[info.pid].blockDetail.models.filter(e => e.isRect == false)
          break;
      }
      break;
    case 'modelsH':
      // 侧面造型 
      processList = conditionSource[info.pid].blockDetail.modelListSide.filter(e => e.isRect == false)
      break;
    case 'groovesH':
      // 侧面槽
      processList = conditionSource[info.pid].blockDetail.modelListSide.filter(e => e.isRect == true)
      break;
    case 'grooves':
      // 垂直槽

      switch (info._flag) {
        case 'A':
          processList = conditionSource[info.pid].blockDetail.modelListFaceA.filter(e => e.isRect == true)
          break;
        case 'B':
          processList = conditionSource[info.pid].blockDetail.modelListFaceB.filter(e => e.isRect == true)
          break;
        default:
          processList = conditionSource[info.pid].blockDetail.models.filter(e => e.isRect == true)
          break;
      }
      break;
    default:
      break;
  }

  let _item: any = null
  let _index = -1
  if (isCondition_Line) {
    _item = processList[info.kid]
    _index = info.kid
  } else {
    _item = processList[i]
    _index = i
  }

  let allprocessItem = {
    block: conditionSource[info.pid],
    blockNo: conditionSource[info.pid].blockNo,
    processItem: _item,
    processFace: info._flag,
    processType: loopObj.type,
    isLine: isCondition_Line,
    index: _index
  }


  if (_item == undefined) {
    console.log('没有对应的加工项', conditionSource, loopObj.type);
    return
  }
  let res = null
  let isLine = false

  for (let condition of conditionList) {

    let checkArr = condition.conditionOptions

    let flag = true
    let c = 0
    if (checkArr.includes('2') || checkArr.includes('3') || checkArr.includes('4')) {
      let point: any = null

      point = { ..._item.realPointList[i] }
      if (point) {
        if (point.curve == undefined) {
          c = 0
        } else if (Number.isNaN(point.curve)) {
          c = 0
        } else {
          c = point.curve
        }
      } else {
        c = 0
      }
    }
    for (const str of checkArr) {
      switch (str) {
        case '0':
          // 0 需要调刀
          let c_0 = checkChangeKnife(allprocessItem, info);
          if (c_0 == true) {
            flag = true
          } else {
            flag = false
          }
          break;
        case '1':
          // 1 不需要调刀
          let c_1 = checkChangeKnife(allprocessItem, info);
          if (c_1 == false) {
            flag = true
          } else {
            flag = false
          }
          break;
        case '2':
          // 2 直线刀路
          if (c == 0) {
            flag = true
          } else {
            flag = false
          }
          break;
        case '3':
          // 3 逆时针圆弧刀路
          if (c > 0) {
            flag = true
          } else {
            flag = false
          }
          break;
        case '4':
          // 4 顺时针圆弧刀路
          if (c < 0) {
            flag = true
          } else {
            flag = false
          }
          break;
        case '5':
          // 5 正面加工
          if ((info._flag == 'A' || info._flag == 'AB') && _item.face == 0) {
            flag = true
          } else {
            flag = false
          }

          break;
        case '6':
          // 6 反面加工

          if ((info._flag == 'B' || info._flag == 'AB') && _item.face == 1) {
            flag = true
          } else {
            flag = false
          }
          break;
        default:
          break;
      }

      if (flag == false) {
        continue
      }
    }

    if (flag == true) {
      if (_item != undefined) {
        let orderby = 'block'
        if (info.exportFileType == '2' || info.exportFileType == '3') {
          orderby = 'board'
        }
        if (!allProcessList.value.find(e => e.blockNo == allprocessItem.blockNo && e.index == allprocessItem.index && e.processType == allprocessItem.processType && e.processFace == allprocessItem.processFace)) {
          allProcessList.value.push(allprocessItem)
        }
      }

      res = condition
      console.log('条件输出', info, res)
      return res
    } else {
      if (!allProcessList.value.find(e => e.blockNo == allprocessItem.blockNo && e.index == allprocessItem.index && e.processType == allprocessItem.processType && e.processFace == allprocessItem.processFace)) {
        allProcessList.value.push(allprocessItem)
      }
    }
  }

  function checkChangeKnife(param, _info) {
    let isChangeKnife = false
    let lastProcessItem = allProcessList.value[allProcessList.value.length - 1]
    if (allProcessList.value.length == 0) {
      // 第一个加工项  要调刀
      isChangeKnife = true
    } else {
      if (_info.exportFileType == 0 || _info.exportFileType == 1) {
        if (lastProcessItem.processItem.knife.knifeId == _item?.knife.knifeId
          && param.processFace == lastProcessItem.processFace
          && param.blockNo == lastProcessItem.blockNo
          && param.processFace == lastProcessItem.processFace) {
          // 同一把刀  不要换
          isChangeKnife = false
        } else {
          // 要换刀
          isChangeKnife = true
        }
      } else if (_info.exportFileType == 2 || _info.exportFileType == 3) {
        if (lastProcessItem.processItem.knife.knifeId == _item?.knife.knifeId
          && param.processFace == lastProcessItem.processFace
          && param.block.boardId == lastProcessItem.block.boardId) {
          // 同一把刀  不要换
          isChangeKnife = false

        } else {
          // 要换刀
          isChangeKnife = true
        }
      }

    }
    return isChangeKnife
  }

  return res

}
